@@ -1,6 +1,6 @@
 /*
     This file is part of darktable,
-    copyright (c) 2012 johannes hanika, tobias ellinghaus.
+    Copyright (C) 2012-2020 darktable developers.
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -65,9 +65,11 @@ static void usage(const char *progname)
   fprintf(stderr, "   --bpp <bpp>, unsupported\n");
   fprintf(stderr, "   --hq <0|1|false|true> default: true\n");
   fprintf(stderr, "   --upscale <0|1|false|true>, default: false\n");
+  fprintf(stderr, "   --export_masks <0|1|false|true>, default: false\n");
   fprintf(stderr, "   --style <style name>\n");
   fprintf(stderr, "   --style-overwrite\n");
   fprintf(stderr, "   --apply-custom-presets <0|1|false|true>, default: true\n");
+  fprintf(stderr, "                          disable for multiple instances\n");
   fprintf(stderr, "   --verbose\n");
   fprintf(stderr, "   --help,-h\n");
   fprintf(stderr, "   --version\n");
@@ -91,7 +93,8 @@ int main(int argc, char *arg[])
   char *style = NULL;
   int file_counter = 0;
   int width = 0, height = 0, bpp = 0;
-  gboolean verbose = FALSE, high_quality = TRUE, upscale = FALSE, style_overwrite = FALSE, custom_presets = TRUE;
+  gboolean verbose = FALSE, high_quality = TRUE, upscale = FALSE,
+           style_overwrite = FALSE, custom_presets = TRUE, export_masks = FALSE;
 
   int k;
   for(k = 1; k < argc; k++)
@@ -142,6 +145,22 @@ int main(int argc, char *arg[])
         }
         g_free(str);
       }
+      else if(!strcmp(arg[k], "--export_masks") && argc > k + 1)
+      {
+        k++;
+        gchar *str = g_ascii_strup(arg[k], -1);
+        if(!g_strcmp0(str, "0") || !g_strcmp0(str, "FALSE"))
+          export_masks = FALSE;
+        else if(!g_strcmp0(str, "1") || !g_strcmp0(str, "TRUE"))
+          export_masks = TRUE;
+        else
+        {
+          fprintf(stderr, "%s: %s\n", _("unknown option for --export_masks"), arg[k]);
+          usage(arg[0]);
+          exit(1);
+        }
+        g_free(str);
+      }
       else if(!strcmp(arg[k], "--upscale") && argc > k + 1)
       {
         k++;
@@ -429,12 +448,12 @@ int main(int argc, char *arg[])
   int num = 1;
   for(GList *iter = id_list; iter; iter = g_list_next(iter), num++)
   {
-    int id = GPOINTER_TO_INT(iter->data);
+    const int id = GPOINTER_TO_INT(iter->data);
     // TODO: have a parameter in command line to get the export presets
     dt_export_metadata_t metadata;
     metadata.flags = dt_lib_export_metadata_default_flags();
     metadata.list = NULL;
-    storage->store(storage, sdata, id, format, fdata, num, total, high_quality, upscale,
+    storage->store(storage, sdata, id, format, fdata, num, total, high_quality, upscale, export_masks,
                    icc_type, icc_filename, icc_intent, &metadata);
   }
 
