@@ -1,7 +1,6 @@
 /*
     This file is part of darktable,
-    copyright (c) 2009--2010 johannes hanika.
-    copyright (c) 2011--2017 tobias ellinghaus.
+    Copyright (C) 2010-2020 darktable developers.
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -375,9 +374,8 @@ static double _PQ_fct(double x)
 static double _HLG_fct(double x)
 {
   static const double Beta  = 0.04;
-  static const double A     = 0.17883277;
-  static const double RA    = 1.0 / A;
-  static const double B     = 1.0 - 4.0 * A;
+  static const double RA    = 5.591816309728916; // 1.0 / A where A = 0.17883277
+  static const double B     = 0.28466892; // 1.0 - 4.0 * A
   static const double C     = 0.5599107295; // 0,5 –aln(4a)
 
   double e = MAX(x * (1.0 - Beta) + Beta, 0.0);
@@ -1062,15 +1060,16 @@ error:
 void rgb2hsl(const float rgb[3], float *h, float *s, float *l)
 {
   const float r = rgb[0], g = rgb[1], b = rgb[2];
-  float pmax = fmax(r, fmax(g, b));
-  float pmin = fmin(r, fmin(g, b));
-  float delta = (pmax - pmin);
+  const float pmax = fmaxf(r, fmax(g, b));
+  const float pmin = fminf(r, fmin(g, b));
+  const float delta = (pmax - pmin);
 
   float hv = 0, sv = 0, lv = (pmin + pmax) / 2.0;
 
-  if(pmax != pmin)
+  if(delta != 0.0f)
   {
-    sv = lv < 0.5 ? delta / (pmax + pmin) : delta / (2.0 - pmax - pmin);
+    sv = lv < 0.5 ? delta / fmaxf(pmax + pmin, 1.52587890625e-05f)
+                  : delta / fmaxf(2.0 - pmax - pmin, 1.52587890625e-05f);
 
     if(pmax == r)
       hv = (g - b) / delta;
@@ -1600,55 +1599,55 @@ const char *dt_colorspaces_get_name(dt_colorspaces_color_profile_type_t type,
   case DT_COLORSPACE_FILE:
     return filename;
   case DT_COLORSPACE_SRGB:
-    return N_("sRGB");
+    return _("sRGB");
   case DT_COLORSPACE_ADOBERGB:
-    return N_("Adobe RGB (compatible)");
+    return _("Adobe RGB (compatible)");
   case DT_COLORSPACE_LIN_REC709:
-    return N_("linear Rec709 RGB");
+    return _("linear Rec709 RGB");
   case DT_COLORSPACE_LIN_REC2020:
-    return N_("linear Rec2020 RGB");
+    return _("linear Rec2020 RGB");
   case DT_COLORSPACE_XYZ:
-    return N_("linear XYZ");
+    return _("linear XYZ");
   case DT_COLORSPACE_LAB:
-    return N_("Lab");
+    return _("Lab");
   case DT_COLORSPACE_INFRARED:
-    return N_("linear infrared BGR");
+    return _("linear infrared BGR");
   case DT_COLORSPACE_DISPLAY:
-    return N_("system display profile");
+    return _("system display profile");
   case DT_COLORSPACE_EMBEDDED_ICC:
-    return N_("embedded ICC profile");
+    return _("embedded ICC profile");
   case DT_COLORSPACE_EMBEDDED_MATRIX:
-    return N_("embedded matrix");
+    return _("embedded matrix");
   case DT_COLORSPACE_STANDARD_MATRIX:
-    return N_("standard color matrix");
+    return _("standard color matrix");
   case DT_COLORSPACE_ENHANCED_MATRIX:
-    return N_("enhanced color matrix");
+    return _("enhanced color matrix");
   case DT_COLORSPACE_VENDOR_MATRIX:
-    return N_("vendor color matrix");
+    return _("vendor color matrix");
   case DT_COLORSPACE_ALTERNATE_MATRIX:
-    return N_("alternate color matrix");
+    return _("alternate color matrix");
   case DT_COLORSPACE_BRG:
-    return N_("BRG (experimental)");
+    return _("BRG (experimental)");
   case DT_COLORSPACE_EXPORT:
-    return N_("export profile");
+    return _("export profile");
   case DT_COLORSPACE_SOFTPROOF:
-    return N_("softproof profile");
+    return _("softproof profile");
   case DT_COLORSPACE_WORK:
-    return N_("work profile");
+    return _("work profile");
   case DT_COLORSPACE_DISPLAY2:
-    return N_("system display profile");
+    return _("system display profile");
   case DT_COLORSPACE_REC709:
-    return N_("gamma22 Rec709");
+    return _("gamma22 Rec709");
   case DT_COLORSPACE_PROPHOTO_RGB:
-    return N_("ProPhoto RGB");
+    return _("ProPhoto RGB");
   case DT_COLORSPACE_PQ_REC2020:
-    return N_("PQ Rec2020");
+    return _("PQ Rec2020");
   case DT_COLORSPACE_HLG_REC2020:
-    return N_("HLG Rec2020");
+    return _("HLG Rec2020");
   case DT_COLORSPACE_PQ_P3:
-    return N_("PQ P3");
+    return _("PQ P3");
   case DT_COLORSPACE_HLG_P3:
-    return N_("HLG P3");
+    return _("HLG P3");
   case DT_COLORSPACE_LAST:
     break;
   }
@@ -1942,7 +1941,7 @@ gboolean dt_colorspaces_is_profile_equal(const char *fullname, const char *filen
   // basename as recorded in an iop.
   return _colorspaces_is_base_name(filename)
     ? !strcmp(_colorspaces_get_base_name(fullname), filename)
-    : !strcmp(fullname, filename);
+    : !strcmp(_colorspaces_get_base_name(fullname), _colorspaces_get_base_name(filename));
 }
 
 static const dt_colorspaces_color_profile_t *_get_profile(dt_colorspaces_t *self,
@@ -1978,7 +1977,7 @@ const dt_colorspaces_color_profile_t *dt_colorspaces_get_profile(dt_colorspaces_
 // Copied from dcraw's pseudoinverse()
 static void dt_colorspaces_pseudoinverse(double (*in)[3], double (*out)[3], int size)
 {
-  double work[3][6], num;
+  double work[3][6];
 
   for(int i = 0; i < 3; i++) {
     for(int j = 0; j < 6; j++)
@@ -1988,7 +1987,7 @@ static void dt_colorspaces_pseudoinverse(double (*in)[3], double (*out)[3], int
         work[i][j] += in[k][i] * in[k][j];
   }
   for(int i = 0; i < 3; i++) {
-    num = work[i][i];
+    double num = work[i][i];
     for(int j = 0; j < 6; j++)
       work[i][j] /= num;
     for(int k = 0; k < 3; k++) {
