@@ -1,6 +1,6 @@
 /*
     This file is part of darktable,
-    copyright (c) 2011 tobias ellinghaus.
+    Copyright (C) 2011-2020 darktable developers.
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,6 +16,7 @@
     along with darktable.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include "control/signal.h"
 #include "common/grouping.h"
 #include "common/darktable.h"
 #include "common/debug.h"
@@ -31,6 +32,9 @@ void dt_grouping_add_to_group(const int group_id, const int image_id)
   dt_image_t *img = dt_image_cache_get(darktable.image_cache, image_id, 'w');
   img->group_id = group_id;
   dt_image_cache_write_release(darktable.image_cache, img, DT_IMAGE_CACHE_SAFE);
+  GList *imgs = NULL;
+  imgs = g_list_prepend(imgs, GINT_TO_POINTER(image_id));
+  dt_control_signal_raise(darktable.signals, DT_SIGNAL_IMAGE_INFO_CHANGED, imgs);
 }
 
 /** remove an image from a group */
@@ -38,6 +42,7 @@ int dt_grouping_remove_from_group(const int image_id)
 {
   sqlite3_stmt *stmt;
   int new_group_id = -1;
+  GList *imgs = NULL;
 
   const dt_image_t *img = dt_image_cache_get(darktable.image_cache, image_id, 'r');
   int img_group_id = img->group_id;
@@ -56,6 +61,7 @@ int dt_grouping_remove_from_group(const int image_id)
       dt_image_t *other_img = dt_image_cache_get(darktable.image_cache, other_id, 'w');
       other_img->group_id = new_group_id;
       dt_image_cache_write_release(darktable.image_cache, other_img, DT_IMAGE_CACHE_SAFE);
+      imgs = g_list_prepend(imgs, GINT_TO_POINTER(other_id));
     }
     sqlite3_finalize(stmt);
 
@@ -75,7 +81,12 @@ int dt_grouping_remove_from_group(const int image_id)
     new_group_id = wimg->group_id;
     wimg->group_id = image_id;
     dt_image_cache_write_release(darktable.image_cache, wimg, DT_IMAGE_CACHE_SAFE);
+    imgs = g_list_prepend(imgs, GINT_TO_POINTER(image_id));
+    // refresh also the group leader which may be alone now
+    imgs = g_list_prepend(imgs, GINT_TO_POINTER(img_group_id));
   }
+  dt_control_signal_raise(darktable.signals, DT_SIGNAL_IMAGE_INFO_CHANGED, imgs);
+
   return new_group_id;
 }
 
@@ -84,10 +95,11 @@ int dt_grouping_change_representative(const int image_id)
 {
   sqlite3_stmt *stmt;
 
-  dt_image_t *img = dt_image_cache_get(darktable.image_cache, image_id, 'w');
+  dt_image_t *img = dt_image_cache_get(darktable.image_cache, image_id, 'r');
   int group_id = img->group_id;
-  dt_image_cache_write_release(darktable.image_cache, img, DT_IMAGE_CACHE_SAFE);
+  dt_image_cache_read_release(darktable.image_cache, img);
 
+  GList *imgs = NULL;
   DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "SELECT id FROM main.images WHERE group_id = ?1", -1,
                               &stmt, NULL);
   DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, group_id);
@@ -97,8 +109,10 @@ int dt_grouping_change_representative(const int image_id)
     dt_image_t *other_img = dt_image_cache_get(darktable.image_cache, other_id, 'w');
     other_img->group_id = image_id;
     dt_image_cache_write_release(darktable.image_cache, other_img, DT_IMAGE_CACHE_SAFE);
+    imgs = g_list_prepend(imgs, GINT_TO_POINTER(other_id));
   }
   sqlite3_finalize(stmt);
+  dt_control_signal_raise(darktable.signals, DT_SIGNAL_IMAGE_INFO_CHANGED, imgs);
 
   return image_id;
 }
@@ -131,7 +145,7 @@ GList *dt_grouping_get_group_images(const int imgid)
   return imgs;
 }
 
-/** get images of the group */
+/** add grouped images to images list */
 void dt_grouping_add_grouped_images(GList **images)
 {
   if(!*images) return;
