@@ -1,6 +1,6 @@
 /*
     This file is part of darktable,
-    copyright (c) 2019 pascal obry.
+    Copyright (C) 2019-2020 darktable developers.
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,10 +16,11 @@
     along with darktable.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include "common/darktable.h"
-#include "common/history.h"
-#include "common/debug.h"
 #include "common/history_snapshot.h"
+#include "common/darktable.h"
+#include "common/debug.h"
+#include "common/history.h"
+#include "control/signal.h"
 
 dt_undo_lt_history_t *dt_history_snapshot_item_init(void)
 {
@@ -60,9 +61,11 @@ void dt_history_snapshot_undo_create(int32_t imgid, int *snap_id, int *history_e
   // copy current state into undo_history
 
   DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db),
-                              "INSERT INTO memory.undo_history SELECT ?1, imgid, num, module, operation, op_params, enabled, "
-                              "blendop_params, blendop_version, multi_priority, multi_name, iop_order "
-                              " FROM main.history WHERE imgid=?2", -1, &stmt, NULL);
+                              "INSERT INTO memory.undo_history"
+                              "  SELECT ?1, imgid, num, module, operation, op_params, enabled, "
+                              "         blendop_params, blendop_version, multi_priority, multi_name "
+                              "  FROM main.history"
+                              "  WHERE imgid=?2", -1, &stmt, NULL);
   DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, *snap_id);
   DT_DEBUG_SQLITE3_BIND_INT(stmt, 2, imgid);
   all_ok = all_ok && (sqlite3_step(stmt) == SQLITE_DONE);
@@ -71,8 +74,23 @@ void dt_history_snapshot_undo_create(int32_t imgid, int *snap_id, int *history_e
   // copy current state into undo_masks_history
 
   DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db),
-                              "INSERT INTO memory.undo_masks_history SELECT ?1, imgid, num, formid, form, name, version, "
-                              "points, points_count, source FROM main.masks_history WHERE imgid=?2", -1, &stmt, NULL);
+                              "INSERT INTO memory.undo_masks_history"
+                              "  SELECT ?1, imgid, num, formid, form, name, version,"
+                              "         points, points_count, source"
+                              "  FROM main.masks_history"
+                              "  WHERE imgid=?2", -1, &stmt, NULL);
+  DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, *snap_id);
+  DT_DEBUG_SQLITE3_BIND_INT(stmt, 2, imgid);
+  all_ok = all_ok && (sqlite3_step(stmt) == SQLITE_DONE);
+  sqlite3_finalize(stmt);
+
+  // copy the module order
+
+  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db),
+                              "INSERT INTO memory.undo_module_order"
+                              "  SELECT ?1, imgid, version, iop_list"
+                              "  FROM main.module_order"
+                              "  WHERE imgid=?2", -1, &stmt, NULL);
   DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, *snap_id);
   DT_DEBUG_SQLITE3_BIND_INT(stmt, 2, imgid);
   all_ok = all_ok && (sqlite3_step(stmt) == SQLITE_DONE);
@@ -97,13 +115,16 @@ static void _history_snapshot_undo_restore(int32_t imgid, int snap_id, int histo
   sqlite3_exec(dt_database_get(darktable.db), "BEGIN TRANSACTION", NULL, NULL, NULL);
 
   dt_history_delete_on_image_ext(imgid, FALSE);
+  dt_control_signal_raise(darktable.signals, DT_SIGNAL_TAG_CHANGED);
 
   // copy undo_history snapshot back as current history state
 
   DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db),
-                              "INSERT INTO main.history SELECT imgid, num, module, operation, op_params, enabled, "
-                              "blendop_params, blendop_version, multi_priority, multi_name, iop_order "
-                              " FROM memory.undo_history WHERE imgid=?2 AND id=?1", -1, &stmt, NULL);
+                              "INSERT INTO main.history"
+                              "  SELECT imgid, num, module, operation, op_params, enabled, "
+                              "         blendop_params, blendop_version, multi_priority, multi_name "
+                              "  FROM memory.undo_history"
+                              "  WHERE imgid=?2 AND id=?1", -1, &stmt, NULL);
   DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, snap_id);
   DT_DEBUG_SQLITE3_BIND_INT(stmt, 2, imgid);
   all_ok &= (sqlite3_step(stmt) != SQLITE_DONE);
@@ -112,8 +133,10 @@ static void _history_snapshot_undo_restore(int32_t imgid, int snap_id, int histo
   // copy undo_masks_history snapshot back as current masks_history state
 
   DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db),
-                              "INSERT INTO main.masks_history SELECT imgid, num, formid, form, name, version, "
-                              "points, points_count, source FROM memory.undo_masks_history WHERE imgid=?2 AND id=?1",
+                              "INSERT INTO main.masks_history"
+                              "  SELECT imgid, num, formid, form, name, version, "
+                              "         points, points_count, source FROM memory.undo_masks_history"
+                              "  WHERE imgid=?2 AND id=?1",
                               -1, &stmt, NULL);
   DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, snap_id);
   DT_DEBUG_SQLITE3_BIND_INT(stmt, 2, imgid);
@@ -129,6 +152,18 @@ static void _history_snapshot_undo_restore(int32_t imgid, int snap_id, int histo
   all_ok &= (sqlite3_step(stmt) != SQLITE_DONE);
   sqlite3_finalize(stmt);
 
+  // restore module order
+
+  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db),
+                              "INSERT INTO main.module_order"
+                              "  SELECT imgid, version, iop_list"
+                              "  FROM memory.undo_module_order"
+                              "  WHERE imgid=?2 AND id=?1", -1, &stmt, NULL);
+  DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, snap_id);
+  DT_DEBUG_SQLITE3_BIND_INT(stmt, 2, imgid);
+  all_ok &= (sqlite3_step(stmt) != SQLITE_DONE);
+  sqlite3_finalize(stmt);
+
   if(all_ok)
     sqlite3_exec(dt_database_get(darktable.db), "COMMIT", NULL, NULL, NULL);
   else
@@ -154,6 +189,13 @@ static void _clear_undo_snapshot(int32_t imgid, int snap_id)
   DT_DEBUG_SQLITE3_BIND_INT(stmt, 2, imgid);
   sqlite3_step(stmt);
   sqlite3_finalize(stmt);
+
+  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db),
+                              "DELETE FROM memory.undo_module_order WHERE id=?1 AND imgid=?2", -1, &stmt, NULL);
+  DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, snap_id);
+  DT_DEBUG_SQLITE3_BIND_INT(stmt, 2, imgid);
+  sqlite3_step(stmt);
+  sqlite3_finalize(stmt);
 }
 
 void dt_history_snapshot_undo_lt_history_data_free(gpointer data)
@@ -183,7 +225,7 @@ void dt_history_snapshot_undo_pop(gpointer user_data, dt_undo_type_t type, dt_un
     {
       _history_snapshot_undo_restore(hist->imgid, hist->after, hist->after_history_end);
     }
-  // in principle undo() routine should add imgid to imgs list to make _undo_do_undo_redo() refresh XMP file for each of them
-  // in this case the update of XMP file is done by the normal image (re)development process.
+
+    *imgs = g_list_append(*imgs, GINT_TO_POINTER(hist->imgid));
   }
 }
