@@ -1,6 +1,6 @@
 /*
     This file is part of darktable,
-    copyright (c) 2009--2011 johannes hanika.
+    Copyright (C) 2009-2020 darktable developers.
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -40,10 +40,15 @@ typedef enum dt_imageio_retval_t
 typedef enum
 {
   // the first 0x7 in flags are reserved for star ratings.
-  DT_IMAGE_DELETE = 1,
-  DT_IMAGE_OKAY = 2,
-  DT_IMAGE_NICE = 3,
-  DT_IMAGE_EXCELLENT = 4,
+  // see view.h:
+  //  DT_VIEW_DESERT = 0,
+  //  DT_VIEW_STAR_1 = 1,
+  //  DT_VIEW_STAR_2 = 2,
+  //  DT_VIEW_STAR_3 = 3,
+  //  DT_VIEW_STAR_4 = 4,
+  //  DT_VIEW_STAR_5 = 5,
+  DT_IMAGE_REJECTED = 8,
+
   // next field unused, but it used to be.
   // old DB entries might have it set.
   // To reuse : force to 0 in DB loading and force to 0 in DB saving
@@ -134,6 +139,8 @@ typedef enum dt_image_loader_t
   LOADER_GM = 8,
   LOADER_RAWSPEED = 9,
   LOADER_PNM = 10,
+  LOADER_AVIF = 11,
+  LOADER_IM = 12,
 } dt_image_loader_t;
 
 typedef struct dt_image_geoloc_t
@@ -150,6 +157,7 @@ typedef struct dt_image_t
   int32_t exif_inited;
   dt_image_orientation_t orientation;
   float exif_exposure;
+  float exif_exposure_bias;
   float exif_aperture;
   float exif_iso;
   float exif_focal_length;
@@ -171,12 +179,16 @@ typedef struct dt_image_t
   // common stuff
 
   // to understand this, look at comment for dt_histogram_roi_t
-  int32_t width, height, verified_size, final_width, final_height;
+  int32_t width, height, verified_size, final_width, final_height, p_width, p_height;
   int32_t crop_x, crop_y, crop_width, crop_height;
   float aspect_ratio;
 
   // used by library
   int32_t num, flags, film_id, id, group_id, version;
+
+  //timestamps
+  time_t import_timestamp, change_timestamp, export_timestamp, print_timestamp;
+
   dt_image_loader_t loader;
 
   dt_iop_buffer_dsc_t buf_dsc;
@@ -241,10 +253,13 @@ void dt_image_path_append_version_no_db(int version, char *pathname, size_t path
 void dt_image_path_append_version(int imgid, char *pathname, size_t pathname_len);
 /** prints a one-line exif information string. */
 void dt_image_print_exif(const dt_image_t *img, char *line, size_t line_len);
+/* set rating to img flags */
+void dt_image_set_xmp_rating(dt_image_t *img, const int rating);
+/* get rating from img flags */
+int dt_image_get_xmp_rating(const dt_image_t *img);
+int dt_image_get_xmp_rating_from_flags(const int flags);
 /** finds all xmp duplicates for the given image in the database. */
 GList* dt_image_find_duplicates(const char* filename);
-/** look for duplicate's xmp files and read them. */
-void dt_image_read_duplicates(uint32_t id, const char *filename);
 /** imports a new image from raw/etc file and adds it to the data base and image cache. Use from threads other than lua.*/
 uint32_t dt_image_import(int32_t film_id, const char *filename, gboolean override_ignore_jpegs);
 /** imports a new image from raw/etc file and adds it to the data base and image cache. Use from lua thread.*/
@@ -264,22 +279,30 @@ dt_image_orientation_t dt_image_get_orientation(const int imgid);
 /** get max width and height of the final processed image with its current hisotry stack */
 gboolean dt_image_get_final_size(const int32_t imgid, int *width, int *height);
 void dt_image_reset_final_size(const int32_t imgid);
-/** set image location lon/lat */
-void dt_image_set_location(const int32_t imgid, dt_image_geoloc_t *geoloc, const gboolean undo_on, const gboolean group_on);
-/** get image location lon/lat */
+/** set image location lon/lat/ele */
+void dt_image_set_location(const int32_t imgid, const dt_image_geoloc_t *geoloc,
+                           const gboolean undo_on, const gboolean group_on);
+/** set images location lon/lat/ele */
+void dt_image_set_locations(const GList *img, const dt_image_geoloc_t *geoloc,
+                           const gboolean undo_on);
+/** get image location lon/lat/ele */
 void dt_image_get_location(const int32_t imgid, dt_image_geoloc_t *geoloc);
-/** returns 1 if there is history data found for this image, 0 else. */
+/** returns TRUE if current hash is not basic nor auto_apply, FALSE otherwise. */
 gboolean dt_image_altered(const uint32_t imgid);
+/** returns TRUE if if current has is basic, FALSE otherwise. */
+gboolean dt_image_basic(const uint32_t imgid);
 /** set the image final/cropped aspect ratio */
-double dt_image_set_aspect_ratio(const int32_t imgid);
+double dt_image_set_aspect_ratio(const int32_t imgid, gboolean raise);
 /** set the image raw aspect ratio */
 void dt_image_set_raw_aspect_ratio(const int32_t imgid);
 /** set the image final/cropped aspect ratio */
-void dt_image_set_aspect_ratio_to(const int32_t imgid, double aspect_ratio);
+void dt_image_set_aspect_ratio_to(const int32_t imgid, double aspect_ratio, gboolean raise);
 /** set the image final/cropped aspect ratio if different from stored*/
-void dt_image_set_aspect_ratio_if_different(const int32_t imgid, double aspect_ratio);
+void dt_image_set_aspect_ratio_if_different(const int32_t imgid, double aspect_ratio, gboolean raise);
 /** reset the image final/cropped aspect ratio to 0.0 */
-void dt_image_reset_aspect_ratio(const int32_t imgid);
+void dt_image_reset_aspect_ratio(const int32_t imgid, gboolean raise);
+/** get the ratio of cropped raw sensor data */
+double dt_image_get_sensor_ratio(const dt_image_t *img);
 /** returns the orientation bits of the image from exif. */
 static inline dt_image_orientation_t dt_image_orientation(const dt_image_t *img)
 {
@@ -334,9 +357,8 @@ void dt_image_local_copy_synch(void);
 // xmp functions:
 void dt_image_write_sidecar_file(int imgid);
 void dt_image_synch_xmp(const int selected);
+void dt_image_synch_xmps(const GList *img);
 void dt_image_synch_all_xmp(const gchar *pathname);
-// return the iop-order-version used by imgid (0 if unknown iop-order-version)
-int dt_image_get_iop_order_version(const int32_t imgid);
 
 // add an offset to the exif_datetime_taken field
 void dt_image_add_time_offset(const int imgid, const long int offset);
@@ -348,6 +370,8 @@ char *dt_image_get_audio_path_from_path(const char *image_path);
 char *dt_image_get_text_path(const int32_t imgid);
 char *dt_image_get_text_path_from_path(const char *image_path);
 
+float dt_image_get_exposure_bias(const struct dt_image_t *image_storage);
+
 // modelines: These editor modelines have been set for all relevant files by tools/update_modelines.sh
 // vim: shiftwidth=2 expandtab tabstop=2 cindent
 // kate: tab-indents: off; indent-width 2; replace-tabs on; indent-mode cstyle; remove-trailing-spaces modified;
