@@ -1,6 +1,6 @@
 /*
     This file is part of darktable,
-    copyright (c) 2011-2014 johannes hanika.
+    Copyright (C) 2009-2020 darktable developers.
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -25,6 +25,7 @@
 #include "develop/develop.h"
 
 #include <sqlite3.h>
+#include <inttypes.h>
 
 void dt_image_cache_allocate(void *data, dt_cache_entry_t *entry)
 {
@@ -34,14 +35,15 @@ void dt_image_cache_allocate(void *data, dt_cache_entry_t *entry)
   dt_image_init(img);
   entry->data = img;
   // load stuff from db and store in cache:
-  char *str;
   sqlite3_stmt *stmt;
   DT_DEBUG_SQLITE3_PREPARE_V2(
       dt_database_get(darktable.db),
       "SELECT id, group_id, film_id, width, height, filename, maker, model, lens, exposure, "
       "aperture, iso, focal_length, datetime_taken, flags, crop, orientation, focus_distance, "
       "raw_parameters, longitude, latitude, altitude, color_matrix, colorspace, version, raw_black, "
-      "raw_maximum, aspect_ratio FROM main.images WHERE id = ?1",
+      "raw_maximum, aspect_ratio, exposure_bias, "
+      "import_timestamp, change_timestamp, export_timestamp, print_timestamp "
+      "FROM main.images WHERE id = ?1",
       -1, &stmt, NULL);
   DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, entry->key);
   if(sqlite3_step(stmt) == SQLITE_ROW)
@@ -54,6 +56,7 @@ void dt_image_cache_allocate(void *data, dt_cache_entry_t *entry)
     img->crop_x = img->crop_y = img->crop_width = img->crop_height = 0;
     img->filename[0] = img->exif_maker[0] = img->exif_model[0] = img->exif_lens[0]
         = img->exif_datetime_taken[0] = '\0';
+    char *str;
     str = (char *)sqlite3_column_text(stmt, 5);
     if(str) g_strlcpy(img->filename, str, sizeof(img->filename));
     str = (char *)sqlite3_column_text(stmt, 6);
@@ -105,6 +108,14 @@ void dt_image_cache_allocate(void *data, dt_cache_entry_t *entry)
       img->aspect_ratio = sqlite3_column_double(stmt, 27);
     else
       img->aspect_ratio = 0.0;
+    if(sqlite3_column_type(stmt, 28) == SQLITE_FLOAT)
+      img->exif_exposure_bias = sqlite3_column_double(stmt, 28);
+    else
+      img->exif_exposure_bias = NAN;
+    img->import_timestamp = sqlite3_column_int(stmt, 29);
+    img->change_timestamp = sqlite3_column_int(stmt, 30);
+    img->export_timestamp = sqlite3_column_int(stmt, 31);
+    img->print_timestamp = sqlite3_column_int(stmt, 32);
 
     // buffer size? colorspace?
     if(img->flags & DT_IMAGE_LDR)
@@ -139,7 +150,7 @@ void dt_image_cache_allocate(void *data, dt_cache_entry_t *entry)
   else
   {
     img->id = -1;
-    fprintf(stderr, "[image_cache_allocate] failed to open image %d from database: %s\n", entry->key,
+    fprintf(stderr, "[image_cache_allocate] failed to open image %" PRIu32 " from database: %s\n", entry->key,
             sqlite3_errmsg(dt_database_get(darktable.db)));
   }
   sqlite3_finalize(stmt);
@@ -230,6 +241,7 @@ void dt_image_cache_write_release(dt_image_cache_t *cache, dt_image_t *img, dt_i
       img->aspect_ratio = (float )img->height / (float )img->width;
   }
   if(img->id <= 0) return;
+
   sqlite3_stmt *stmt;
   DT_DEBUG_SQLITE3_PREPARE_V2(
       dt_database_get(darktable.db),
@@ -238,7 +250,9 @@ void dt_image_cache_write_release(dt_image_cache_t *cache, dt_image_t *img, dt_i
       "focus_distance = ?11, film_id = ?12, datetime_taken = ?13, flags = ?14, "
       "crop = ?15, orientation = ?16, raw_parameters = ?17, group_id = ?18, longitude = ?19, "
       "latitude = ?20, altitude = ?21, color_matrix = ?22, colorspace = ?23, raw_black = ?24, "
-      "raw_maximum = ?25, aspect_ratio = ROUND(?26,1) WHERE id = ?27",
+      "raw_maximum = ?25, aspect_ratio = ROUND(?26,1), exposure_bias = ?27, "
+      "change_timestamp = ?28, change_timestamp = ?29, export_timestamp = ?30, print_timestamp = ?31 "
+      "WHERE id = ?32",
       -1, &stmt, NULL);
   DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, img->width);
   DT_DEBUG_SQLITE3_BIND_INT(stmt, 2, img->height);
@@ -267,7 +281,12 @@ void dt_image_cache_write_release(dt_image_cache_t *cache, dt_image_t *img, dt_i
   DT_DEBUG_SQLITE3_BIND_INT(stmt, 24, img->raw_black_level);
   DT_DEBUG_SQLITE3_BIND_INT(stmt, 25, img->raw_white_point);
   DT_DEBUG_SQLITE3_BIND_DOUBLE(stmt, 26, img->aspect_ratio);
-  DT_DEBUG_SQLITE3_BIND_INT(stmt, 27, img->id);
+  DT_DEBUG_SQLITE3_BIND_DOUBLE(stmt, 27, img->exif_exposure_bias);
+  DT_DEBUG_SQLITE3_BIND_INT(stmt, 28, img->import_timestamp);
+  DT_DEBUG_SQLITE3_BIND_INT(stmt, 29, img->change_timestamp);
+  DT_DEBUG_SQLITE3_BIND_INT(stmt, 30, img->export_timestamp);
+  DT_DEBUG_SQLITE3_BIND_INT(stmt, 31, img->print_timestamp);
+  DT_DEBUG_SQLITE3_BIND_INT(stmt, 32, img->id);
   const int rc = sqlite3_step(stmt);
   if(rc != SQLITE_DONE) fprintf(stderr, "[image_cache_write_release] sqlite3 error %d\n", rc);
   sqlite3_finalize(stmt);
@@ -289,7 +308,54 @@ void dt_image_cache_remove(dt_image_cache_t *cache, const uint32_t imgid)
   dt_cache_remove(&cache->cache, imgid);
 }
 
+/* set timestamps */
+void dt_image_cache_set_change_timestamp(dt_image_cache_t *cache, const uint32_t imgid)
+{
+  if(imgid <= 0) return;
+  dt_cache_entry_t *entry = dt_cache_get(&cache->cache, imgid, DT_IMAGE_CACHE_SAFE);
+  if(!entry) return;
+  ASAN_UNPOISON_MEMORY_REGION(entry->data, sizeof(dt_image_t));
+  dt_image_t *img = (dt_image_t *)entry->data;
+  img->cache_entry = entry;
+  img->change_timestamp = time(0);
+  dt_image_cache_write_release(cache, img, DT_IMAGE_CACHE_SAFE);
+}
 
+void dt_image_cache_unset_change_timestamp(dt_image_cache_t *cache, const uint32_t imgid)
+{
+  if(imgid <= 0) return;
+  dt_cache_entry_t *entry = dt_cache_get(&cache->cache, imgid, DT_IMAGE_CACHE_SAFE);
+  if(!entry) return;
+  ASAN_UNPOISON_MEMORY_REGION(entry->data, sizeof(dt_image_t));
+  dt_image_t *img = (dt_image_t *)entry->data;
+  img->cache_entry = entry;
+  img->change_timestamp = -1;
+  dt_image_cache_write_release(cache, img, DT_IMAGE_CACHE_SAFE);
+}
+
+void dt_image_cache_set_export_timestamp(dt_image_cache_t *cache, const uint32_t imgid)
+{
+  if(imgid <= 0) return;
+  dt_cache_entry_t *entry = dt_cache_get(&cache->cache, imgid, DT_IMAGE_CACHE_SAFE);
+  if(!entry) return;
+  ASAN_UNPOISON_MEMORY_REGION(entry->data, sizeof(dt_image_t));
+  dt_image_t *img = (dt_image_t *)entry->data;
+  img->cache_entry = entry;
+  img->export_timestamp = time(0);
+  dt_image_cache_write_release(cache, img, DT_IMAGE_CACHE_SAFE);
+}
+
+void dt_image_cache_set_print_timestamp(dt_image_cache_t *cache, const uint32_t imgid)
+{
+  if(imgid <= 0) return;
+  dt_cache_entry_t *entry = dt_cache_get(&cache->cache, imgid, DT_IMAGE_CACHE_SAFE);
+  if(!entry) return;
+  ASAN_UNPOISON_MEMORY_REGION(entry->data, sizeof(dt_image_t));
+  dt_image_t *img = (dt_image_t *)entry->data;
+  img->cache_entry = entry;
+  img->print_timestamp = time(0);
+  dt_image_cache_write_release(cache, img, DT_IMAGE_CACHE_SAFE);
+}
 
 // modelines: These editor modelines have been set for all relevant files by tools/update_modelines.sh
 // vim: shiftwidth=2 expandtab tabstop=2 cindent
