@@ -1,6 +1,6 @@
 /*
     This file is part of darktable,
-    copyright (c) 2010--2014 Henrik Andersson.
+    Copyright (C) 2010-2020 darktable developers.
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -100,11 +100,17 @@ dt_imageio_retval_t dt_imageio_open_exr(dt_image_t *img, const char *filename, d
     return DT_IMAGEIO_FILE_CORRUPTED;
   }
 
-  // read back exif data
-  const Imf::BlobAttribute *exif = header.findTypedAttribute<Imf::BlobAttribute>("exif");
-  // we append a jpg-compatible exif00 string, so get rid of that again:
-  if(exif && exif->value().size > 6)
-    dt_exif_read_from_blob(img, ((uint8_t *)(exif->value().data.get())) + 6, exif->value().size - 6);
+  if(!img->exif_inited)
+  {
+    // read back exif data
+    // if another software is able to update these exif data, the former test
+    // should be removed to take the potential changes in account (not done
+    // by normal import image flow)
+    const Imf::BlobAttribute *exif = header.findTypedAttribute<Imf::BlobAttribute>("exif");
+    // we append a jpg-compatible exif00 string, so get rid of that again:
+    if(exif && exif->value().size > 6)
+      dt_exif_read_from_blob(img, ((uint8_t *)(exif->value().data.get())) + 6, exif->value().size - 6);
+  }
 
   /* Get image width and height from displayWindow */
   dw = header.displayWindow();
