@@ -1,6 +1,6 @@
 /*
     This file is part of darktable,
-    copyright (c) 2014 Henrik Andersson.
+    Copyright (C) 2014-2020 darktable developers.
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -44,11 +44,18 @@ typedef struct dt_import_session_t
 static void _import_session_cleanup_filmroll(dt_import_session_t *self)
 {
   if(self->film == NULL) return;
-
   /* if current filmroll for session is empty, remove it */
-  /* TODO: check if dt_film_remove actual removes directories */
-  if(dt_film_is_empty(self->film->id)) dt_film_remove(self->film->id);
-
+  if(dt_film_is_empty(self->film->id))
+  {
+    dt_film_remove(self->film->id);
+    if(self->current_path != NULL && g_file_test(self->current_path, G_FILE_TEST_IS_DIR) && dt_util_is_dir_empty(self->current_path))
+    {
+      // no need to ask for rmdir as it'll be re-created if it's needed
+      // by another import session with same path params
+      g_rmdir(self->current_path);
+      self->current_path = NULL;
+    }
+  }
   dt_film_cleanup(self->film);
 
   g_free(self->film);
@@ -180,7 +187,7 @@ void dt_import_session_import(struct dt_import_session_t *self)
   int id = dt_image_import(self->film->id, self->current_filename, TRUE);
   if(id)
   {
-    dt_view_filmstrip_set_active_image(darktable.view_manager, id);
+    dt_control_signal_raise(darktable.signals, DT_SIGNAL_VIEWMANAGER_THUMBTABLE_ACTIVATE, id);
     dt_control_queue_redraw();
   }
 }
@@ -230,17 +237,20 @@ const char *dt_import_session_name(struct dt_import_session_t *self)
   return self->vp->jobcode;
 }
 
-
-const char *dt_import_session_filename(struct dt_import_session_t *self, gboolean current)
+/* This returns a unique filename using session path **and** the filename.
+   If current is true we will use the original filename otherwise use the pattern.
+*/
+const char *dt_import_session_filename(struct dt_import_session_t *self, gboolean use_filename)
 {
   const char *path;
   char *fname, *previous_fname;
   char *pattern;
-
-  if(current && self->current_filename != NULL) return self->current_filename;
+  gchar *result_fname;
 
   /* expand next filename */
   g_free((void *)self->current_filename);
+  self->current_filename = NULL;
+
   pattern = _import_session_filename_pattern();
   if(pattern == NULL)
   {
@@ -250,7 +260,12 @@ const char *dt_import_session_filename(struct dt_import_session_t *self, gboolea
 
   /* verify that expanded path and filename yields a unique file */
   path = dt_import_session_path(self, TRUE);
-  gchar *result_fname = dt_variables_expand(self->vp, pattern, TRUE);
+
+  if(use_filename)
+    result_fname = g_strdup(self->vp->filename);
+  else
+    result_fname = dt_variables_expand(self->vp, pattern, TRUE);
+
   previous_fname = fname = g_build_path(G_DIR_SEPARATOR_S, path, result_fname, (char *)NULL);
   if(g_file_test(fname, G_FILE_TEST_EXISTS) == TRUE)
   {
@@ -317,6 +332,7 @@ const char *dt_import_session_path(struct dt_import_session_t *self, gboolean cu
   /* we need to initialize a new filmroll for the new path */
   if(_import_session_initialize_filmroll(self, new_path) != 0)
   {
+    g_free(new_path);
     fprintf(stderr, "[import_session] Failed to get session path.\n");
     return NULL;
   }
