@@ -1,7 +1,6 @@
 /*
     This file is part of darktable,
-    copyright (c) 2010 tobias ellinghaus.
-    copyright (c) 2019 philippe weyland
+    Copyright (C) 2010-2020 darktable developers.
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,11 +21,157 @@
 #include "common/collection.h"
 #include "common/undo.h"
 #include "common/grouping.h"
+#include "control/conf.h"
 #include "views/view.h"
 #include "control/signal.h"
 
 #include <stdlib.h>
 
+// this array should contain all dt metadata
+// add the new metadata at the end when needed
+// Dependencies
+//    Must match with dt_metadata_t in metadata.h.
+//    Exif.cc: add the new metadata into dt_xmp_keys[]
+//    libs/metadata.c increment version and change legacy_param() accordingly
+// CAUTION : key, subkey (last term of key) & name must be unique
+
+static const struct
+{
+  char *key;
+  char *name;
+  int type;
+  uint32_t display_order;
+} dt_metadata_def[] = {
+  // clang-format off
+  {"Xmp.dc.creator", N_("creator"), DT_METADATA_TYPE_USER, 2},
+  {"Xmp.dc.publisher", N_("publisher"), DT_METADATA_TYPE_USER, 3},
+  {"Xmp.dc.title", N_("title"), DT_METADATA_TYPE_USER, 0},
+  {"Xmp.dc.description", N_("description"), DT_METADATA_TYPE_USER, 1},
+  {"Xmp.dc.rights", N_("rights"), DT_METADATA_TYPE_USER, 4},
+  {"Xmp.acdsee.notes", N_("notes"), DT_METADATA_TYPE_USER, 5},
+  {"Xmp.darktable.version_name", N_("version name"), DT_METADATA_TYPE_OPTIONAL, 6}
+  // clang-format on
+};
+
+const char *dt_metadata_get_name_by_display_order(const uint32_t order)
+{
+  if(order < DT_METADATA_NUMBER)
+  {
+    for(unsigned int i = 0; i < DT_METADATA_NUMBER; i++)
+    {
+      if(order == dt_metadata_def[i].display_order)
+        return dt_metadata_def[i].name;
+    }
+  }
+  return NULL;
+}
+
+const dt_metadata_t dt_metadata_get_keyid_by_display_order(const uint32_t order)
+{
+  if(order < DT_METADATA_NUMBER)
+  {
+    for(unsigned int i = 0; i < DT_METADATA_NUMBER; i++)
+    {
+      if(order == dt_metadata_def[i].display_order)
+        return i;
+    }
+  }
+  return -1;
+}
+
+const int dt_metadata_get_type_by_display_order(const uint32_t order)
+{
+  if(order < DT_METADATA_NUMBER)
+  {
+    for(unsigned int i = 0; i < DT_METADATA_NUMBER; i++)
+    {
+      if(order == dt_metadata_def[i].display_order)
+        return dt_metadata_def[i].type;
+    }
+  }
+  return 0;
+}
+
+const char *dt_metadata_get_name(const uint32_t keyid)
+{
+  if(keyid < DT_METADATA_NUMBER)
+    return dt_metadata_def[keyid].name;
+  else
+    return NULL;
+}
+
+const dt_metadata_t dt_metadata_get_keyid(const char* key)
+{
+  for(unsigned int i = 0; i < DT_METADATA_NUMBER; i++)
+  {
+    if(strncmp(key, dt_metadata_def[i].key, strlen(dt_metadata_def[i].key)) == 0)
+      return i;
+  }
+  return -1;
+}
+
+const char *dt_metadata_get_key(const uint32_t keyid)
+{
+  if(keyid < DT_METADATA_NUMBER)
+    return dt_metadata_def[keyid].key;
+  else
+    return NULL;
+}
+
+const char *dt_metadata_get_subkey(const uint32_t keyid)
+{
+  if(keyid < DT_METADATA_NUMBER)
+  {
+    char *t = g_strrstr(dt_metadata_def[keyid].key, ".");
+    if(t) return t + 1;
+  }
+  return NULL;
+}
+
+const char *dt_metadata_get_key_by_subkey(const char *subkey)
+{
+  if(subkey)
+  {
+    for(unsigned int i = 0; i < DT_METADATA_NUMBER; i++)
+    {
+      char *t = g_strrstr(dt_metadata_def[i].key, ".");
+      if(t && !g_strcmp0(t + 1, subkey))
+        return dt_metadata_def[i].key;
+    }
+  }
+  return NULL;
+}
+
+const int dt_metadata_get_type(const uint32_t keyid)
+{
+  if(keyid < DT_METADATA_NUMBER)
+    return dt_metadata_def[keyid].type;
+  else
+    return 0;
+}
+
+void dt_metadata_init()
+{
+  for(unsigned int i = 0; i < DT_METADATA_NUMBER; i++)
+  {
+    const int type = dt_metadata_get_type(i);
+    const char *name = (gchar *)dt_metadata_get_name(i);
+    char *setting = dt_util_dstrcat(NULL, "plugins/lighttable/metadata/%s_flag", name);
+    if(!dt_conf_key_exists(setting))
+    {
+      // per default should be imported
+      uint32_t flag = DT_METADATA_FLAG_IMPORTED;
+      if(type == DT_METADATA_TYPE_OPTIONAL)
+      {
+        // per default this one should be hidden
+        flag |= DT_METADATA_FLAG_HIDDEN;
+      }
+      dt_conf_set_int(setting, flag);
+    }
+    g_free(setting);
+  }
+}
+
 typedef struct dt_undo_metadata_t
 {
   int imgid;
@@ -86,7 +231,9 @@ static gchar *_get_tb_added_metadata_string_values(const int img, GList *before,
     }
     if((!same_key || different_value) && value[0])
     {
-      metadata_list = dt_util_dstrcat(metadata_list, "(%d,%d,\"%s\"),", GPOINTER_TO_INT(img), atoi(a->data), (char *)a2->data);
+      char *escaped_text = sqlite3_mprintf("%q", value);
+      metadata_list = dt_util_dstrcat(metadata_list, "(%d,%d,'%s'),", GPOINTER_TO_INT(img), atoi(a->data), escaped_text);
+      sqlite3_free(escaped_text);
     }
     a = g_list_next(a);
     a = g_list_next(a);
@@ -206,13 +353,13 @@ gchar *_cleanup_metadata_value(const gchar *value)
   return c;
 }
 
-static GList *dt_metadata_get_xmp(const int id, const char *key, uint32_t *count)
+GList *dt_metadata_get(const int id, const char *key, uint32_t *count)
 {
   GList *result = NULL;
   sqlite3_stmt *stmt;
   uint32_t local_count = 0;
 
-  int keyid = dt_metadata_get_keyid(key);
+  const int keyid = dt_metadata_get_keyid(key);
   // key not found in db. Maybe it's one of our "special" keys (rating, tags and colorlabels)?
   if(keyid == -1)
   {
@@ -319,208 +466,9 @@ static GList *dt_metadata_get_xmp(const int id, const char *key, uint32_t *count
   return result;
 }
 
-/*
-  Dear Mister Dijkstra,
-  I hereby make a formal apology for using goto statements in the following
-  function. While I am fully aware that I will rot in the deepest hells for
-  this ultimate sin and that I'm not worth to be called a "programmer" from
-  now on, I have one excuse to bring up: I never did so before, and this way
-  the code gets a lot smaller and less repetitive. And since you are dead
-  while I am not (yet) I will stick with my gotos.
-  See you in hell
-  houz
-*/
-static GList *dt_metadata_get_exif(const int id, const char *key, uint32_t *count)
+static void _metadata_add_metadata_to_list(GList **list, const GList *metadata)
 {
-  GList *result = NULL;
-  sqlite3_stmt *stmt;
-  uint32_t local_count = 0;
-
-  // the doubles
-  if(strncmp(key, "Exif.Photo.ExposureTime", 23) == 0)
-  {
-    if(id == -1)
-    {
-      DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "SELECT exposure FROM main.images WHERE id IN "
-                                                                 "(SELECT imgid FROM main.selected_images)",
-                                  -1, &stmt, NULL);
-    }
-    else // single image under mouse cursor
-    {
-      DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "SELECT exposure FROM main.images WHERE id = ?1",
-                                  -1, &stmt, NULL);
-      DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, id);
-    }
-  }
-  else if(strncmp(key, "Exif.Photo.ApertureValue", 24) == 0)
-  {
-    if(id == -1)
-    {
-      DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "SELECT aperture FROM main.images WHERE id IN "
-                                                                 "(SELECT imgid FROM main.selected_images)",
-                                  -1, &stmt, NULL);
-    }
-    else // single image under mouse cursor
-    {
-      DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "SELECT aperture FROM main.images WHERE id = ?1",
-                                  -1, &stmt, NULL);
-      DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, id);
-    }
-  }
-  else if(strncmp(key, "Exif.Photo.ISOSpeedRatings", 26) == 0)
-  {
-    if(id == -1)
-    {
-      DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db),
-                                  "SELECT iso FROM main.images WHERE id IN "
-                                  "(SELECT imgid FROM main.selected_images)", -1, &stmt, NULL);
-    }
-    else // single image under mouse cursor
-    {
-      DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "SELECT iso FROM main.images WHERE id = ?1", -1,
-                                  &stmt, NULL);
-      DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, id);
-    }
-  }
-  else if(strncmp(key, "Exif.Photo.FocalLength", 22) == 0)
-  {
-    if(id == -1)
-    {
-      DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db),
-                                  "SELECT focal_length FROM main.images WHERE id IN "
-                                  "(SELECT imgid FROM main.selected_images)",
-                                  -1, &stmt, NULL);
-    }
-    else // single image under mouse cursor
-    {
-      DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db),
-                                  "SELECT focal_length FROM main.images WHERE id = ?1", -1, &stmt, NULL);
-      DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, id);
-    }
-  }
-  else
-  {
-
-    // the strings
-    if(strncmp(key, "Exif.Photo.DateTimeOriginal", 27) == 0)
-    {
-      if(id == -1)
-      {
-        DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db),
-                                    "SELECT datetime_taken FROM main.images WHERE id IN "
-                                    "(SELECT imgid FROM main.selected_images)",
-                                    -1, &stmt, NULL);
-      }
-      else // single image under mouse cursor
-      {
-        DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db),
-                                    "SELECT datetime_taken FROM main.images WHERE id = ?1", -1, &stmt, NULL);
-        DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, id);
-      }
-    }
-    else if(strncmp(key, "Exif.Image.Make", 15) == 0)
-    {
-      if(id == -1)
-      {
-        DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "SELECT maker FROM main.images WHERE id IN "
-                                                                   "(SELECT imgid FROM main.selected_images)",
-                                    -1, &stmt, NULL);
-      }
-      else // single image under mouse cursor
-      {
-        DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "SELECT maker FROM main.images WHERE id = ?1",
-                                    -1, &stmt, NULL);
-        DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, id);
-      }
-    }
-    else if(strncmp(key, "Exif.Image.Model", 16) == 0)
-    {
-      if(id == -1)
-      {
-        DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "SELECT model FROM main.images WHERE id IN "
-                                                                   "(SELECT imgid FROM main.selected_images)",
-                                    -1, &stmt, NULL);
-      }
-      else // single image under mouse cursor
-      {
-        DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "SELECT model FROM main.images WHERE id = ?1",
-                                    -1, &stmt, NULL);
-        DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, id);
-      }
-    }
-    else
-    {
-      goto END;
-    }
-    while(sqlite3_step(stmt) == SQLITE_ROW)
-    {
-      local_count++;
-      result = g_list_append(result, g_strdup((char *)sqlite3_column_text(stmt, 0)));
-    }
-    sqlite3_finalize(stmt);
-    goto END;
-  }
-
-  // the double queries
-  while(sqlite3_step(stmt) == SQLITE_ROW)
-  {
-    local_count++;
-    double *tmp = (double *)malloc(sizeof(double));
-    *tmp = sqlite3_column_double(stmt, 0);
-    result = g_list_append(result, tmp);
-  }
-  sqlite3_finalize(stmt);
-
-END:
-  if(count != NULL) *count = local_count;
-  return result;
-}
-
-// for everything which doesn't fit anywhere else (our made up stuff)
-static GList *dt_metadata_get_dt(const int id, const char *key, uint32_t *count)
-{
-  GList *result = NULL;
-  sqlite3_stmt *stmt;
-  uint32_t local_count = 0;
-
-  if(strncmp(key, "darktable.Lens", 14) == 0)
-  {
-    if(id == -1)
-    {
-      DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "SELECT lens FROM main.images WHERE id IN "
-                                                                 "(SELECT imgid FROM main.selected_images)",
-                                  -1, &stmt, NULL);
-    }
-    else // single image under mouse cursor
-    {
-      DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "SELECT lens FROM main.images WHERE id = ?1", -1,
-                                  &stmt, NULL);
-      DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, id);
-    }
-    while(sqlite3_step(stmt) == SQLITE_ROW)
-    {
-      local_count++;
-      result = g_list_append(result, g_strdup((char *)sqlite3_column_text(stmt, 0)));
-    }
-    sqlite3_finalize(stmt);
-  }
-  else if(strncmp(key, "darktable.Name", 14) == 0)
-  {
-    result = g_list_append(result, g_strdup(PACKAGE_NAME));
-    local_count = 1;
-  }
-  else if(strncmp(key, "darktable.Version", 17) == 0)
-  {
-    result = g_list_append(result, g_strdup(darktable_package_version));
-    local_count = 1;
-  }
-  if(count != NULL) *count = local_count;
-  return result;
-}
-
-static void _metadata_add_metadata_to_list(GList **list, GList *metadata)
-{
-  GList *m = metadata;
+  const GList *m = metadata;
   while(m)
   {
     GList *m2 = g_list_next(m);
@@ -545,15 +493,39 @@ static void _metadata_add_metadata_to_list(GList **list, GList *metadata)
   }
 }
 
+static void _metadata_remove_metadata_from_list(GList **list, const GList *metadata)
+{
+  // caution: metadata is a simple list here
+  const GList *m = metadata;
+  while(m)
+  {
+    GList *same_key = g_list_find_custom(*list, m->data, _compare_metadata);
+    if(same_key)
+    {
+      // same key for that image - remove metadata item
+      GList *same2 = g_list_next(same_key);
+      *list = g_list_remove_link(*list, same_key);
+      g_free(same_key->data);
+      g_list_free(same_key);
+      *list = g_list_remove_link(*list, same2);
+      g_free(same2->data);
+      g_list_free(same2);
+    }
+    m = g_list_next(m);
+  }
+}
+
 typedef enum dt_tag_actions_t
 {
   DT_MA_SET = 0,
-  DT_MA_ADD
+  DT_MA_ADD,
+  DT_MA_REMOVE
 } dt_tag_actions_t;
 
-static void _metadata_execute(GList *imgs, GList *metadata, GList **undo, const gboolean undo_on, const gint action)
+static void _metadata_execute(const GList *imgs, const GList *metadata, GList **undo,
+                              const gboolean undo_on, const gint action)
 {
-  GList *images = imgs;
+  const GList *images = imgs;
   while(images)
   {
     const int image_id = GPOINTER_TO_INT(images->data);
@@ -564,12 +536,16 @@ static void _metadata_execute(GList *imgs, GList *metadata, GList **undo, const
     switch(action)
     {
       case DT_MA_SET:
-        undometadata->after = metadata ? g_list_copy_deep(metadata, (GCopyFunc)g_strdup, NULL) : NULL;
+        undometadata->after = metadata ? g_list_copy_deep((GList *)metadata, (GCopyFunc)g_strdup, NULL) : NULL;
         break;
       case DT_MA_ADD:
         undometadata->after = g_list_copy_deep(undometadata->before, (GCopyFunc)g_strdup, NULL);
         _metadata_add_metadata_to_list(&undometadata->after, metadata);
         break;
+      case DT_MA_REMOVE:
+        undometadata->after = g_list_copy_deep(undometadata->before, (GCopyFunc)g_strdup, NULL);
+        _metadata_remove_metadata_from_list(&undometadata->after, metadata);
+        break;
       default:
         undometadata->after = g_list_copy_deep(undometadata->before, (GCopyFunc)g_strdup, NULL);
         break;
@@ -585,7 +561,7 @@ static void _metadata_execute(GList *imgs, GList *metadata, GList **undo, const
   }
 }
 
-void dt_metadata_set(const int imgid, const char *key, const char *value, const gboolean undo_on, const gboolean group_on)
+void dt_metadata_set(const int imgid, const char *key, const char *value, const gboolean undo_on)
 {
   if(!key || !imgid) return;
 
@@ -594,13 +570,12 @@ void dt_metadata_set(const int imgid, const char *key, const char *value, const
   {
     GList *imgs = NULL;
     if(imgid == -1)
-      imgs = dt_collection_get_selected(darktable.collection, -1);
+      imgs = g_list_copy((GList *)dt_view_get_images_to_act_on(TRUE, TRUE));
     else
       imgs = g_list_append(imgs, GINT_TO_POINTER(imgid));
     if(imgs)
     {
       GList *undo = NULL;
-      if(group_on) dt_grouping_add_grouped_images(&imgs);
       if(undo_on) dt_undo_start_group(darktable.undo, DT_UNDO_METADATA);
 
       const gchar *ckey = dt_util_dstrcat(NULL, "%d", keyid);
@@ -618,12 +593,50 @@ void dt_metadata_set(const int imgid, const char *key, const char *value, const
         dt_undo_record(darktable.undo, NULL, DT_UNDO_METADATA, undo, _pop_undo, _metadata_undo_data_free);
         dt_undo_end_group(darktable.undo);
       }
-      dt_control_signal_raise(darktable.signals, DT_SIGNAL_MOUSE_OVER_IMAGE_CHANGE);
     }
   }
 }
 
-void dt_metadata_set_list(const int imgid, GList *key_value, const gboolean undo_on, const gboolean group_on)
+void dt_metadata_set_import(const int imgid, const char *key, const char *value)
+{
+  if(!key || !imgid || imgid == -1) return;
+
+  const int keyid = dt_metadata_get_keyid(key);
+
+  if(keyid != -1) // known key
+  {
+    gboolean imported = TRUE;
+    if(dt_metadata_get_type(keyid) != DT_METADATA_TYPE_INTERNAL)
+    {
+      const gchar *name = dt_metadata_get_name(keyid);
+      char *setting = dt_util_dstrcat(NULL, "plugins/lighttable/metadata/%s_flag", name);
+      imported = dt_conf_get_int(setting) & DT_METADATA_FLAG_IMPORTED;
+      g_free(setting);
+    }
+    if(imported)
+    {
+      GList *imgs = NULL;
+      imgs = g_list_append(imgs, GINT_TO_POINTER(imgid));
+      if(imgs)
+      {
+        GList *undo = NULL;
+
+        const gchar *ckey = dt_util_dstrcat(NULL, "%d", keyid);
+        const gchar *cvalue = _cleanup_metadata_value(value);
+        GList *metadata = NULL;
+        metadata = g_list_append(metadata, (gpointer)ckey);
+        metadata = g_list_append(metadata, (gpointer)cvalue);
+
+        _metadata_execute(imgs, metadata, &undo, FALSE, DT_MA_ADD);
+
+        g_list_free_full(metadata, g_free);
+        g_list_free(imgs);
+      }
+    }
+  }
+}
+
+void dt_metadata_set_list(const GList *imgs, GList *key_value, const gboolean undo_on)
 {
   GList *metadata = NULL;
   GList *kv = key_value;
@@ -650,75 +663,67 @@ void dt_metadata_set_list(const int imgid, GList *key_value, const gboolean undo
     }
   }
 
-  if(metadata)
-  {
-    GList *imgs = NULL;
-    if(imgid == -1)
-      imgs = dt_collection_get_selected(darktable.collection, -1);
-    else
-      imgs = g_list_append(imgs, GINT_TO_POINTER(imgid));
-    if(imgs)
-    {
-      GList *undo = NULL;
-      if(group_on) dt_grouping_add_grouped_images(&imgs);
-      if(undo_on) dt_undo_start_group(darktable.undo, DT_UNDO_METADATA);
-
-      _metadata_execute(imgs, metadata, &undo, undo_on, DT_MA_ADD);
-
-      g_list_free(imgs);
-      if(undo_on)
-      {
-        dt_undo_record(darktable.undo, NULL, DT_UNDO_METADATA, undo, _pop_undo, _metadata_undo_data_free);
-        dt_undo_end_group(darktable.undo);
-      }
-    }
-  }
-}
-
-GList *dt_metadata_get(const int id, const char *key, uint32_t *count)
-{
-  if(strncmp(key, "Xmp.", 4) == 0) return dt_metadata_get_xmp(id, key, count);
-  if(strncmp(key, "Exif.", 5) == 0) return dt_metadata_get_exif(id, key, count);
-  if(strncmp(key, "darktable.", 10) == 0) return dt_metadata_get_dt(id, key, count);
-  return NULL;
-}
-
-void dt_metadata_clear(const int imgid, const gboolean undo_on, const gboolean group_on)
-{
-  GList *imgs = NULL;
-  if(imgid == -1)
-    imgs = dt_collection_get_selected(darktable.collection, -1);
-  else
-    imgs = g_list_append(imgs, GINT_TO_POINTER(imgid));
-  if(imgs)
+  if(metadata && imgs)
   {
     GList *undo = NULL;
-    if(group_on) dt_grouping_add_grouped_images(&imgs);
     if(undo_on) dt_undo_start_group(darktable.undo, DT_UNDO_METADATA);
 
-    _metadata_execute(imgs, NULL, &undo, undo_on, DT_MA_SET);
+    _metadata_execute(imgs, metadata, &undo, undo_on, DT_MA_ADD);
 
-    g_list_free(imgs);
     if(undo_on)
     {
       dt_undo_record(darktable.undo, NULL, DT_UNDO_METADATA, undo, _pop_undo, _metadata_undo_data_free);
       dt_undo_end_group(darktable.undo);
     }
-    dt_control_signal_raise(darktable.signals, DT_SIGNAL_MOUSE_OVER_IMAGE_CHANGE);
+
+    g_list_free_full(metadata, g_free);
   }
 }
 
-void dt_metadata_set_list_id(const int imgid, GList *metadata, const gboolean clear_on, const gboolean undo_on, const gboolean group_on)
+void dt_metadata_clear(const GList *imgs, const gboolean undo_on)
 {
-  GList *imgs = NULL;
-  if(imgid == -1)
-    imgs = dt_collection_get_selected(darktable.collection, -1);
-  else
-    imgs = g_list_append(imgs, GINT_TO_POINTER(imgid));
+  // do not clear internal or hidden metadata
+  GList *metadata = NULL;
+  for(unsigned int i = 0; i < DT_METADATA_NUMBER; i++)
+  {
+    if(dt_metadata_get_type(i) != DT_METADATA_TYPE_INTERNAL)
+    {
+      const gchar *name = dt_metadata_get_name(i);
+      char *setting = dt_util_dstrcat(NULL, "plugins/lighttable/metadata/%s_flag", name);
+      const gboolean hidden = dt_conf_get_int(setting) & DT_METADATA_FLAG_HIDDEN;
+      g_free(setting);
+      if(!hidden)
+      {
+        // caution: metadata is a simple list here
+        metadata = g_list_append(metadata, dt_util_dstrcat(NULL, "%d", i));
+      }
+    }
+  }
+
+  if(metadata)
+  {
+    GList *undo = NULL;
+    if(undo_on) dt_undo_start_group(darktable.undo, DT_UNDO_METADATA);
+
+    _metadata_execute(imgs, metadata, &undo, undo_on, DT_MA_REMOVE);
+
+    if(undo_on)
+    {
+      dt_undo_record(darktable.undo, NULL, DT_UNDO_METADATA, undo, _pop_undo, _metadata_undo_data_free);
+      dt_undo_end_group(darktable.undo);
+    }
+
+    g_list_free_full(metadata, g_free);
+  }
+}
+
+void dt_metadata_set_list_id(const GList *img, const GList *metadata, const gboolean clear_on,
+                             const gboolean undo_on)
+{
+  GList *imgs = g_list_copy((GList *)img);
   if(imgs)
   {
     GList *undo = NULL;
-    if(group_on) dt_grouping_add_grouped_images(&imgs);
     if(undo_on) dt_undo_start_group(darktable.undo, DT_UNDO_METADATA);
 
     _metadata_execute(imgs, metadata, &undo, undo_on, clear_on ? DT_MA_SET : DT_MA_ADD);
@@ -729,7 +734,6 @@ void dt_metadata_set_list_id(const int imgid, GList *metadata, const gboolean cl
       dt_undo_record(darktable.undo, NULL, DT_UNDO_METADATA, undo, _pop_undo, _metadata_undo_data_free);
       dt_undo_end_group(darktable.undo);
     }
-    dt_control_signal_raise(darktable.signals, DT_SIGNAL_MOUSE_OVER_IMAGE_CHANGE);
   }
 }
 
