@@ -1,6 +1,6 @@
 /*
     This file is part of darktable,
-    copyright (c) 2010 Tobias Ellinghaus.
+    Copyright (C) 2010-2020 darktable developers.
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -57,7 +57,7 @@ typedef enum dt_logo_season_t
 /** returns the dt logo season to use right now */
 dt_logo_season_t dt_util_get_logo_season(void);
 
-cairo_surface_t *dt_util_get_logo(float size);
+cairo_surface_t *dt_util_get_logo(const float size);
 
 gchar *dt_util_latitude_str(float latitude);
 gchar *dt_util_longitude_str(float longitude);
@@ -71,6 +71,9 @@ gboolean dt_util_gps_elevation_to_number(const double r_1, const double r_2, cha
 // make paths absolute and try to normalize on Windows. also deal with character encoding on Windows.
 gchar *dt_util_normalize_path(const gchar *input);
 
+// format exposure time string
+gchar *dt_util_format_exposure(const float exposuretime);
+
 // modelines: These editor modelines have been set for all relevant files by tools/update_modelines.sh
 // vim: shiftwidth=2 expandtab tabstop=2 cindent
 // kate: tab-indents: off; indent-width 2; replace-tabs on; indent-mode cstyle; remove-trailing-spaces modified;
