@@ -1,6 +1,6 @@
 /*
     This file is part of darktable,
-    copyright (c) 2014 tobias ellinghaus.
+    Copyright (C) 2014-2020 darktable developers.
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -75,9 +75,16 @@ GList *dt_control_crawler_run()
     const int id = sqlite3_column_int(stmt, 0);
     const time_t timestamp = sqlite3_column_int(stmt, 1);
     const int version = sqlite3_column_int(stmt, 2);
-    gchar *image_path = (gchar *)sqlite3_column_text(stmt, 3);
+    const gchar *image_path = (char *)sqlite3_column_text(stmt, 3);
     int flags = sqlite3_column_int(stmt, 4);
 
+    // if the image is missing we ignore it.
+    if(!g_file_test(image_path, G_FILE_TEST_EXISTS))
+    {
+      dt_print(DT_DEBUG_CONTROL, "[crawler] `%s' (id: %d) is missing.\n", image_path, id);
+      continue;
+    }
+
     // no need to look for xmp files if none get written anyway.
     if(look_for_xmp)
     {
@@ -94,7 +101,11 @@ GList *dt_control_crawler_run()
       xmp_path[len] = '\0';
 
       struct stat statbuf;
-      if(stat(xmp_path, &statbuf) == -1) continue; // TODO: shall we report these?
+      // on Windows the encoding might not be UTF8
+      gchar *xmp_path_locale = g_locale_from_utf8(xmp_path, -1, NULL, NULL, NULL);
+      const int stat_res = stat(xmp_path_locale, &statbuf);
+      g_free(xmp_path_locale);
+      if(stat_res == -1) continue; // TODO: shall we report these?
 
       // step 1: check if the xmp is newer than our db entry
       // FIXME: allow for a few seconds difference?
@@ -118,11 +129,12 @@ GList *dt_control_crawler_run()
 
     // step 2: check if the image has associated files (.txt, .wav)
     size_t len = strlen(image_path);
-    char *c = image_path + len;
-    while((c > image_path) && (*c != '.')) *c-- = '\0';
+    const char *c = image_path + len;
+    while((c > image_path) && (*c != '.')) c--;
     len = c - image_path + 1;
 
-    char *extra_path = g_strndup(image_path, len + 3);
+    char *extra_path = (char *)calloc(len + 3 + 1, sizeof(char));
+    g_strlcpy(extra_path, image_path, len + 1);
 
     extra_path[len] = 't';
     extra_path[len + 1] = 'x';
@@ -170,7 +182,7 @@ GList *dt_control_crawler_run()
       sqlite3_clear_bindings(inner_stmt);
     }
 
-    g_free(extra_path);
+    free(extra_path);
   }
 
   sqlite3_exec(dt_database_get(darktable.db), "COMMIT", NULL, NULL, NULL);
@@ -293,8 +305,7 @@ void _overwrite_button_clicked(GtkButton *button, gpointer user_data)
   _clear_select_all(gui);
 }
 
-// show a popup window with a list of updated images/xmp files and allow the user to tell dt what to do about
-// them
+// show a popup window with a list of updated images/xmp files and allow the user to tell dt what to do about them
 void dt_control_crawler_show_image_list(GList *images)
 {
   if(!images) return;
