@@ -1,6 +1,6 @@
 /*
     This file is part of darktable,
-    copyright (c) 2010 -- 2014 Henrik Andersson.
+    Copyright (C) 2010-2020 darktable developers.
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -37,7 +37,7 @@ static int32_t dt_image_load_job_run(dt_job_t *job)
   if (buf.buf && buf.height && buf.width)
   {
     const double aspect_ratio = (double)buf.width / (double)buf.height;
-    dt_image_set_aspect_ratio_if_different(params->imgid, aspect_ratio);
+    dt_image_set_aspect_ratio_if_different(params->imgid, aspect_ratio, FALSE);
   }
 
   // drop read lock, as this is only speculative async loading.
@@ -79,7 +79,7 @@ static int32_t dt_image_import_job_run(dt_job_t *job)
   const int id = dt_image_import(params->film_id, params->filename, TRUE);
   if(id)
   {
-    dt_view_filmstrip_set_active_image(darktable.view_manager, id);
+    dt_control_signal_raise(darktable.signals, DT_SIGNAL_VIEWMANAGER_THUMBTABLE_ACTIVATE, id);
     dt_control_queue_redraw();
   }
 
