@@ -1,8 +1,6 @@
 /*
     This file is part of darktable,
-    copyright (c) 2009--2012 johannes hanika.
-    copyright (c) 2011 henrik andersson.
-    copyright (c) 2012 tobias ellinghaus.
+    Copyright (C) 2009-2020 darktable developers.
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -90,21 +88,18 @@ typedef enum dt_iop_flags_t
   IOP_FLAGS_NONE = 0,
 
   /** Flag for the iop module to be enabled/included by default when creating a style */
-  IOP_FLAGS_INCLUDE_IN_STYLES = 1 << 0,
-  IOP_FLAGS_SUPPORTS_BLENDING = 1 << 1, // Does provide blending modes
-  IOP_FLAGS_DEPRECATED = 1 << 2,
-  IOP_FLAGS_BLEND_ONLY_LIGHTNESS
-  = 1 << 3, // Does only blend with L-channel in Lab space. Keeps a, b of original image.
-  IOP_FLAGS_ALLOW_TILING = 1 << 4, // Does allow tile-wise processing (valid for CPU and GPU processing)
-  IOP_FLAGS_HIDDEN = 1 << 5,       // Hide the iop from userinterface
-  IOP_FLAGS_TILING_FULL_ROI
-  = 1 << 6, // Tiling code has to expect arbitrary roi's for this module (incl. flipping, mirroring etc.)
-  IOP_FLAGS_ONE_INSTANCE = 1 << 7, // The module doesn't support multiple instances
-  IOP_FLAGS_PREVIEW_NON_OPENCL
-  = 1 << 8, // Preview pixelpipe of this module must not run on GPU but always on CPU
-  IOP_FLAGS_NO_HISTORY_STACK = 1 << 9, // This iop will never show up in the history stack
-  IOP_FLAGS_NO_MASKS = 1 << 10,         // The module doesn't support masks (used with SUPPORT_BLENDING)
-  IOP_FLAGS_FENCE = 1 << 11              // No module can be moved pass this one
+  IOP_FLAGS_INCLUDE_IN_STYLES  = 1 << 0,
+  IOP_FLAGS_SUPPORTS_BLENDING  = 1 << 1,  // Does provide blending modes
+  IOP_FLAGS_DEPRECATED         = 1 << 2,
+  IOP_FLAGS_ALLOW_TILING       = 1 << 4,  // Does allow tile-wise processing (valid for CPU and GPU processing)
+  IOP_FLAGS_HIDDEN             = 1 << 5,  // Hide the iop from userinterface
+  IOP_FLAGS_TILING_FULL_ROI    = 1 << 6,  // Tiling code has to expect arbitrary roi's for this module (incl. flipping, mirroring etc.)
+  IOP_FLAGS_ONE_INSTANCE       = 1 << 7,  // The module doesn't support multiple instances
+  IOP_FLAGS_PREVIEW_NON_OPENCL = 1 << 8,  // Preview pixelpipe of this module must not run on GPU but always on CPU
+  IOP_FLAGS_NO_HISTORY_STACK   = 1 << 9,  // This iop will never show up in the history stack
+  IOP_FLAGS_NO_MASKS           = 1 << 10, // The module doesn't support masks (used with SUPPORT_BLENDING)
+  IOP_FLAGS_FENCE              = 1 << 11, // No module can be moved pass this one
+  IOP_FLAGS_ALLOW_FAST_PIPE    = 1 << 12  // Module can work with a fast pipe
 } dt_iop_flags_t;
 
 /** status of a module*/
@@ -124,9 +119,8 @@ typedef void dt_iop_global_data_t;
 /** color picker request */
 typedef enum dt_dev_request_colorpick_flags_t
 {
-  DT_REQUEST_COLORPICK_OFF = 0,         // off
-  DT_REQUEST_COLORPICK_MODULE = 1 << 0, // requested by module (should take precedence)
-  DT_REQUEST_COLORPICK_BLEND = 1 << 1   // requested by parametric blending gui
+  DT_REQUEST_COLORPICK_OFF = 0,   // off
+  DT_REQUEST_COLORPICK_MODULE = 1 // requested by module (should take precedence)
 } dt_dev_request_colorpick_flags_t;
 
 /** colorspace enums, must be in synch with dt_iop_colorspace_type_t in color_conversion.cl */
@@ -177,6 +171,13 @@ typedef struct dt_iop_module_so_t
   int (*flags)(void);
 
   const char *(*description)(void);
+  /* should return a string with 5 lines:
+     line 1 : summary of what it does
+     line 2 : oriented creative or corrective ?
+     line 3 : working space
+     line 4 : input space
+     line 5 : output space
+  */
 
   int (*operation_tags)(void);
   int (*operation_tags_filter)(void);
@@ -208,6 +209,8 @@ typedef struct dt_iop_module_so_t
   void (*gui_reset)(struct dt_iop_module_t *self);
   void (*gui_update)(struct dt_iop_module_t *self);
   void (*gui_init)(struct dt_iop_module_t *self);
+  void (*color_picker_apply)(struct dt_iop_module_t *self, GtkWidget *picker, struct dt_dev_pixelpipe_iop_t *piece);
+  void (*gui_changed)(struct dt_iop_module_t *self, GtkWidget *widget, void *previous);
   void (*gui_cleanup)(struct dt_iop_module_t *self);
   void (*gui_post_expose)(struct dt_iop_module_t *self, cairo_t *cr, int32_t width, int32_t height,
                           int32_t pointerx, int32_t pointery);
@@ -294,7 +297,7 @@ typedef struct dt_iop_module_t
   /** used to identify this module in the history stack. */
   int32_t instance;
   /** order of the module on the pipe. the pipe will be sorted by iop_order. */
-  double iop_order;
+  int iop_order;
   /** module sets this if the enable checkbox should be hidden. */
   int32_t hide_enable_button;
   /** set to DT_REQUEST_COLORPICK_MODULE if you want an input color picked during next eval. gui mode only. */
@@ -305,9 +308,8 @@ typedef struct dt_iop_module_t
   int request_mask_display;
   /** set to 1 if you want the blendif mask to be suppressed in the module in focus. gui mode only. */
   int32_t suppress_mask;
-  /** color picker proxys */
+  /** color picker proxy */
   struct dt_iop_color_picker_t *picker;
-  struct dt_iop_color_picker_t *blend_picker;
   /** bounding box in which the mean color is requested. */
   float color_picker_box[4];
   /** single point to pick if in point mode */
@@ -441,6 +443,10 @@ typedef struct dt_iop_module_t
   void (*gui_reset)(struct dt_iop_module_t *self);
   /** construct widget. */
   void (*gui_init)(struct dt_iop_module_t *self);
+  /** apply color picker results */
+  void (*color_picker_apply)(struct dt_iop_module_t *self, GtkWidget *picker, struct dt_dev_pixelpipe_iop_t *piece);
+  /** called by standard widget callbacks after value changed */
+  void (*gui_changed)(struct dt_iop_module_t *self, GtkWidget *widget, void *previous);
   /** destroy widget. */
   void (*gui_cleanup)(struct dt_iop_module_t *self);
   /** optional method called after darkroom expose. */
@@ -600,6 +606,8 @@ GtkWidget *dt_iop_gui_get_pluginui(dt_iop_module_t *module);
 
 /** requests the focus for this plugin (to draw overlays over the center image) */
 void dt_iop_request_focus(dt_iop_module_t *module);
+/** allocate and load default settings from introspection. */
+void dt_iop_default_init(dt_iop_module_t *module);
 /** loads default settings from database. */
 void dt_iop_load_default_params(dt_iop_module_t *module);
 /** reloads certain gui/param defaults when the image was switched. */
@@ -618,8 +626,17 @@ int dt_iop_breakpoint(struct dt_develop_t *dev, struct dt_dev_pixelpipe_t *pipe)
 void dt_iop_nap(int32_t usec);
 
 /** get module by name and colorout, works only with a dev mode */
-dt_iop_module_t *get_colorout_module(void);
-dt_iop_module_t *get_module_by_name(const char *op);
+dt_iop_module_t *dt_iop_get_colorout_module(void);
+/* returns the iop-module found in list with the given name */
+dt_iop_module_t *dt_iop_get_module_from_list(GList *iop_list, const char *op);
+dt_iop_module_t *dt_iop_get_module(const char *op);
+/** returns module with op + multi_priority or NULL if not found on the list,
+    if multi_priority == -1 do not checl for it */
+dt_iop_module_t *dt_iop_get_module_by_op_priority(GList *modules, const char *operation, const int multi_priority);
+/** returns module with op + multi_name or NULL if not found on the list,
+    if multi_name == NULL do not checl for it */
+dt_iop_module_t *dt_iop_get_module_by_instance_name(GList *modules, const char *operation, const char *multi_name);
+
 
 /** get module flags, works in dev and lt mode */
 int get_module_flags(const char *op);
@@ -641,6 +658,21 @@ dt_iop_module_t *dt_iop_gui_get_previous_visible_module(dt_iop_module_t *module)
 /** returns the next visible module on the module list */
 dt_iop_module_t *dt_iop_gui_get_next_visible_module(dt_iop_module_t *module);
 
+// initializes memory.darktable_iop_names
+void dt_iop_set_darktable_iop_table();
+
+/** adds keyboard accels to the first module in the pipe to handle where there are multiple instances */
+void dt_iop_connect_accels_multi(dt_iop_module_so_t *module);
+
+/** adds keyboard accels for all modules in the pipe */
+void dt_iop_connect_accels_all();
+
+/** get the module that accelerators are attached to for the current so */
+dt_iop_module_t *dt_iop_get_module_accel_curr(dt_iop_module_so_t *module);
+
+/** count instances of a module **/
+int dt_iop_count_instances(dt_iop_module_so_t *module);
+
 // modelines: These editor modelines have been set for all relevant files by tools/update_modelines.sh
 // vim: shiftwidth=2 expandtab tabstop=2 cindent
 // kate: tab-indents: off; indent-width 2; replace-tabs on; indent-mode cstyle; remove-trailing-spaces modified;
