@@ -1,6 +1,6 @@
 /*
     This file is part of darktable,
-    copyright (c) 2010 Henrik Andersson.
+    Copyright (C) 2010-2020 darktable developers.
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -45,17 +45,9 @@ static gboolean _togglebutton_draw(GtkWidget *widget, cairo_t *cr)
 
   GtkStateFlags state = gtk_widget_get_state_flags(widget);
 
-  GdkRGBA bg_color, fg_color;
+  GdkRGBA fg_color;
   GtkStyleContext *context = gtk_widget_get_style_context(widget);
-  if(button->icon_flags & CPF_CUSTOM_BG)
-    bg_color = button->bg;
-  else
-  {
-    GdkRGBA *bc;
-    gtk_style_context_get(context, state, "background-color", &bc, NULL);
-    bg_color = *bc;
-    gdk_rgba_free(bc);
-  }
+
   if(button->icon_flags & CPF_CUSTOM_FG)
     fg_color = button->fg;
   else if(button->icon_flags & CPF_IGNORE_FG_STATE)
@@ -67,15 +59,19 @@ static gboolean _togglebutton_draw(GtkWidget *widget, cairo_t *cr)
   int flags = DTGTK_TOGGLEBUTTON(widget)->icon_flags;
 
   /* update active state paint flag */
-  gboolean active = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
+  const gboolean active = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
   if(active)
-  {
     flags |= CPF_ACTIVE;
-  }
   else
-  {
-    flags &= ~(CPF_ACTIVE);
-  }
+    flags &= ~CPF_ACTIVE;
+
+  /* update focus state paint flag */
+  const gboolean hasfocus = ((DTGTK_TOGGLEBUTTON(widget)->icon_data == darktable.develop->gui_module)
+                         && darktable.develop->gui_module);
+  if(hasfocus)
+    flags |= CPF_FOCUS;
+  else
+    flags &= ~CPF_FOCUS;
 
   /* prelight */
   if(state & GTK_STATE_FLAG_PRELIGHT)
@@ -84,37 +80,41 @@ static gboolean _togglebutton_draw(GtkWidget *widget, cairo_t *cr)
     flags &= ~CPF_PRELIGHT;
 
   /* begin cairo drawing */
+  /* get button total allocation */
   GtkAllocation allocation;
   gtk_widget_get_allocation(widget, &allocation);
   int width = allocation.width;
   int height = allocation.height;
 
+  /* get the css geometry properties of the button */
+  GtkBorder margin, border, padding;
+  gtk_style_context_get_margin(context, state, &margin);
+  gtk_style_context_get_border(context, state, &border);
+  gtk_style_context_get_padding(context, state, &padding);
+
+  /* for button frame and background, we remove css margin from allocation */
+  int startx = margin.left;
+  int starty = margin.top;
+  int cwidth = width - margin.left - margin.right;
+  int cheight = height - margin.top - margin.bottom;
+
   /* draw standard button background if not transparent nor flat styled */
   if((flags & CPF_STYLE_FLAT))
   {
-    if(flags & CPF_PRELIGHT || (flags & CPF_ACTIVE && !(flags & CPF_BG_TRANSPARENT)))
+    if((flags & CPF_PRELIGHT) || ((flags & CPF_ACTIVE) && !(flags & CPF_BG_TRANSPARENT)))
     {
       // When CPF_BG_TRANSPARENT is set, change the background on
       // PRELIGHT, but not on ACTIVE
       if(!(flags & CPF_BG_TRANSPARENT) || (flags & CPF_PRELIGHT))
-      {
-        cairo_rectangle(cr, 0, 0, width, height);
-        gdk_cairo_set_source_rgba(cr, &bg_color);
-        cairo_fill(cr);
-      }
+        gtk_render_background(context, cr, startx, starty, cwidth, cheight);
     }
     else if(!(flags & CPF_ACTIVE) || (flags & CPF_IGNORE_FG_STATE))
-    {
       fg_color.alpha = CLAMP(fg_color.alpha / 2.0, 0.3, 1.0);
-    }
   }
   else if(!(flags & CPF_BG_TRANSPARENT))
-  {
-    /* draw default boxed button */
-    gtk_render_background(context, cr, 0, 0, width, height);
-    if(!(flags & CPF_DO_NOT_USE_BORDER))
-      gtk_render_frame(context, cr, 0, 0, width, height);
-  }
+    gtk_render_background(context, cr, startx, starty, cwidth, cheight);
+
+  gtk_render_frame(context, cr, startx, starty, cwidth, cheight);
 
   /* create pango text settings if label exists */
   PangoLayout *layout = NULL;
@@ -134,16 +134,31 @@ static gboolean _togglebutton_draw(GtkWidget *widget, cairo_t *cr)
   /* draw icon */
   if(DTGTK_TOGGLEBUTTON(widget)->icon)
   {
-    /* set inner border and icon size */
-    float f_border = (1 + 0.66 * (darktable.gui->dpi_factor-1)) * ((flags & CPF_DO_NOT_USE_BORDER) ? 4.0 : 6.0);
-    int border = round(f_border);
-    int icon_width = round(text ? height - (f_border * 2) : width - (f_border * 2));
-    int icon_height = round(height - (f_border * 2));
+    /* calculate the button content allocation */
+    startx += border.left + padding.left;
+    starty += border.top + padding.top;
+    cwidth -= border.left + border.right + padding.left + padding.right;
+    cheight -= border.top + border.bottom + padding.top + padding.bottom;
+
+    /* we have to leave some breathing room to the cairo icon paint function to possibly    */
+    /* draw slightly outside the bounding box, for optical alignment and balancing of icons */
+    /* we do this by putting a drawing area widget inside the button and using the CSS      */
+    /* margin property in px of the drawing area as extra room in percent (DPI safe)        */
+    /* we do this because Gtk+ does not support CSS size in percent                         */
+    /* this extra margin can be also (slightly) negative                                    */
+    GtkStyleContext *ccontext = gtk_widget_get_style_context(DTGTK_TOGGLEBUTTON(widget)->canvas);
+    GtkBorder cmargin;
+    gtk_style_context_get_margin(ccontext, state, &cmargin);
+
+    startx += round(cmargin.left * cwidth / 100.0f);
+    starty += round(cmargin.top * cheight / 100.0f);
+    cwidth = round((float)cwidth * (1.0 - (cmargin.left + cmargin.right) / 100.0f));
+    cheight = round((float)cheight * (1.0 - (cmargin.top + cmargin.bottom) / 100.0f));
 
     void *icon_data = DTGTK_TOGGLEBUTTON(widget)->icon_data;
 
-    if(icon_width > 0 && icon_height > 0)
-        DTGTK_TOGGLEBUTTON(widget)->icon(cr, border, border, icon_width, icon_height, flags, icon_data);
+    if(cwidth > 0 && cheight > 0)
+        DTGTK_TOGGLEBUTTON(widget)->icon(cr, startx, starty, cwidth, cheight, flags, icon_data);
   }
 
 
@@ -171,7 +186,10 @@ GtkWidget *dtgtk_togglebutton_new(DTGTKCairoPaintIconFunc paint, gint paintflags
   button->icon = paint;
   button->icon_flags = paintflags;
   button->icon_data = paintdata;
+  button->canvas = gtk_drawing_area_new();
+  gtk_container_add(GTK_CONTAINER(button), button->canvas);
   gtk_widget_set_name(GTK_WIDGET(button), "dt-toggle-button");
+  gtk_widget_set_name(GTK_WIDGET(button->canvas), "button-canvas");
   return (GtkWidget *)button;
 }
 
