@@ -160,6 +160,7 @@ static void dt_dcraw_adobe_coeff(const char *name, float cam_xyz[1][12])
     { "Fujifilm FinePix F77", { 13690,-5358,-1474,-3369,11600,1998,-132,1554,4395 } },
     { "Fujifilm FinePix F7", { 10004,-3219,-1201,-7036,15047,2107,-1863,2565,7736 } },
     { "Fujifilm FinePix F8", { 13690,-5358,-1474,-3369,11600,1998,-132,1554,4395 } },
+    { "Fujifilm FinePix S1", { 12297,-4882,-1202,-2106,10691,1623,-88,1312,4790 } },
     { "Fujifilm FinePix S100FS", { 11521,-4355,-1065,-6524,13767,3058,-1466,1984,6045 } },
     { "Fujifilm FinePix S200EXR", { 11401,-4498,-1312,-5088,12751,2613,-838,1568,5941 } },
     { "Fujifilm FinePix S20Pro", { 10004,-3219,-1201,-7036,15047,2107,-1863,2565,7736 } },
@@ -199,6 +200,7 @@ static void dt_dcraw_adobe_coeff(const char *name, float cam_xyz[1][12])
     { "Fujifilm X70", { 10450,-4329,-878,-3217,11105,2421,-752,1758,6519 } },
     { "Fujifilm X-Pro1", { 10413,-3996,-993,-3721,11640,2361,-733,1540,6011 } },
     { "Fujifilm X-Pro2", { 11434,-4948,-1210,-3746,12042,1903,-666,1479,5235 } },
+    { "Fujifilm X-Pro3", { 13426,-6334,-1177,-4244,12136,2371,-580,1303,5980 } },
     { "Fujifilm X-A1", { 11086,-4555,-839,-3512,11310,2517,-815,1341,5940 } },
     { "Fujifilm X-A2", { 10763,-4560,-917,-3346,11311,2322,-475,1135,5843 } },
     { "Fujifilm X-A3", { 12407,-5222,-1086,-2971,11116,2120,-294,1029,5284 } },
@@ -215,9 +217,11 @@ static void dt_dcraw_adobe_coeff(const char *name, float cam_xyz[1][12])
     { "Fujifilm X-T20", { 10592,-4262,-1008,-3514,11355,2465,-870,2025,6386 } },
     { "Fujifilm X-T30", { 13426,-6334,-1177,-4244,12136,2371,-580,1303,5980 } },
     { "Fujifilm X-T100", { 11673,-4760,-1041,-3988,12058,2166,-771,1417,5569 } },
+    { "Fujifilm X-T200", { 15055,-7391,-1274,-4062,12071,2238,-610,1217,6147 } },
     { "Fujifilm X-T1", { 8458,-2451,-855,-4597,12447,2407,-1475,2482,6526 } },
     { "Fujifilm X-T2", { 11434,-4948,-1210,-3746,12042,1903,-666,1479,5235 } },
     { "Fujifilm X-T3", { 16393,-7740,-1436,-4238,12131,2371,-633,1424,6553 } },
+    { "Fujifilm X-T4", { 13426,-6334,-1177,-4244,12136,2371,-580,1303,5980 } },
     { "Fujifilm XF1", { 13509,-6199,-1254,-4430,12733,1865,-331,1441,5022 } },
     { "Fujifilm XF10", { 11673,-4760,-1041,-3988,12058,2166,-771,1417,5569 } },
     { "Fujifilm XQ1", { 9252,-2704,-1064,-5893,14265,1717,-1101,2341,4349 } },
@@ -227,6 +231,7 @@ static void dt_dcraw_adobe_coeff(const char *name, float cam_xyz[1][12])
     { "Hasselblad H3D", { 3857,452,-46,-6008,14477,1596,-2627,4481,5718 } },
     { "Hasselblad H4D", { 9662,-684,-279,-4903,12293,2950,-344,1669,6024 } },
     { "Hasselblad H4D-40", { 6159,-1402,-177,-5439,12762,3007,-955,2200,7104 } },
+    { "Hasselblad H4D-50", { 5656,-659,-346,-3923,12306,1791,-1602,3509,5442 } },
     { "Hasselblad H5D-40", { 6159,-1402,-177,-5439,12762,3007,-955,2200,7104 } },
     { "Hasselblad H5D-50c", { 4932,-835,141,-4878,11868,3437,-1138,1961,7067 } },
     { "Hasselblad H6D-100cMS", { 5656,-659,-346,-3923,12306,1791,-1602,3509,5442 } },
@@ -234,6 +239,7 @@ static void dt_dcraw_adobe_coeff(const char *name, float cam_xyz[1][12])
     { "Hasselblad CFV-50", { 5656,-659,-346,-3923,12306,1791,-1602,3509,5442 } },
     { "Hasselblad CF132", {8519,-3260,-280,-5081,13459,1738,-1449,2960,7809} },
     { "Hasselblad X1D", { 4932,-835,141,-4878,11868,3437,-1138,1961,7067 } },
+    { "Hasselblad X1DM2-50c", { 5002,-878,111,-4856,11929,3338,-1183,2041,7022 } },
     { "Imacon Ixpress", { 7025,-1415,-704,-5188,13765,1424,-1248,2742,6038 } }, /* DJC */
     { "KODAK NC2000", { 13891,-6055,-803,-465,9919,642,2121,82,1291 } },
     { "Kodak DCS315C", { 17523,-4827,-2510,756,8546,-137,6113,1649,2250 } },
@@ -335,6 +341,7 @@ static void dt_dcraw_adobe_coeff(const char *name, float cam_xyz[1][12])
     { "Nikon D7500", { 8813,-3210,-1036,-4703,12868,2021,-1054,1940,6129 } },
     { "Nikon D700", { 8139,-2171,-663,-8747,16541,2295,-1925,2008,8093 } },
     { "Nikon D750", { 9020,-2890,-715,-4535,12436,2348,-934,1919,7086 } },
+    { "Nikon D780", { 9943,-3269,-839,-5323,13269,2259,-1198,2083,7557 } },
     { "Nikon D70s", { 7732,-2422,-789,-8238,15884,2498,-859,783,7330 } },
     { "Nikon D70", { 7732,-2422,-789,-8238,15884,2498,-859,783,7330 } },
     { "Nikon D800E", { 7866,-2108,-555,-4869,12483,2681,-1176,2069,7501 } },
@@ -382,6 +389,7 @@ static void dt_dcraw_adobe_coeff(const char *name, float cam_xyz[1][12])
     { "Nikon 1 S2", { 6612,-1342,-618,-3338,11055,2623,-174,1792,5075 } },
     { "Nikon Z 6", { 8210,-2534,-683,-5355,13338,2212,-1143,1929,6464 } },
     { "Nikon Z 7", { 10405,-3755,-1270,-5461,13787,1793,-1040,2015,6785 } },
+    { "Nikon Z 50", { 11640,-4829,-1079,-5107,13006,2325,-972,1711,7380 } },
     { "Olympus C5050Z", { 10508,-3124,-1273,-6079,14294,1901,-1653,2306,6237 } },
     { "Olympus C5060WZ", { 10445,-3362,-1307,-7662,15690,2058,-1135,1176,7602 } },
     { "Olympus C7070WZ", { 10252,-3531,-1095,-7114,14850,2436,-1451,1723,6365 } },
@@ -417,6 +425,7 @@ static void dt_dcraw_adobe_coeff(const char *name, float cam_xyz[1][12])
     { "Olympus E-PL7", { 9197,-3190,-659,-2606,10830,2039,-458,1250,5458 } },
     { "Olympus E-PL8", { 9197,-3190,-659,-2606,10830,2039,-458,1250,5458 } },
     { "Olympus E-PL9", { 8380,-2630,-639,-2887,10725,2496,-627,1427,5438 } },
+    { "Olympus E-PL10", { 9197,-3190,-659,-2606,10830,2039,-458,1250,5458 } },
     { "Olympus E-PM1", { 7575,-2159,-571,-3722,11341,2725,-1434,2819,6271 } },
     { "Olympus E-PM2", { 8380,-2630,-639,-2887,10725,2496,-627,1427,5438 } },
     { "Olympus E-M10", { 8380,-2630,-639,-2887,10725,2496,-627,1427,5438 } },
@@ -514,6 +523,7 @@ static void dt_dcraw_adobe_coeff(const char *name, float cam_xyz[1][12])
     { "Panasonic DMC-LX15", { 7790,-2736,-755,-3452,11870,1769,-628,1647,4898 } },
     { "Panasonic DMC-CM1", { 8770,-3194,-820,-2871,11281,1803,-513,1552,4434 } },
     { "Panasonic DMC-FZ1000", { 7830,-2696,-763,-3325,11667,1866,-641,1712,4824 } },
+    { "Panasonic DC-FZ10002", { 9803,-4185,-992,-4066,12578,1628,-838,1824,5288 } },
     { "Panasonic DMC-FZ100", { 16197,-6146,-1761,-2393,10765,1869,366,2238,5248 } },
     { "Leica V-LUX 2", { 16197,-6146,-1761,-2393,10765,1869,366,2238,5248 } },
     { "Panasonic DMC-FZ150", { 11904,-4541,-1189,-2355,10899,1662,-296,1586,4289 } },
@@ -558,12 +568,15 @@ static void dt_dcraw_adobe_coeff(const char *name, float cam_xyz[1][12])
     { "Panasonic DC-GH5S", { 6929,-2355,-708,-4192,12534,1828,-1097,1989,5195 } },
     { "Panasonic DC-LX100M2", { 8585,-3127,-833,-4005,12250,1953,-650,1494,4862 } },
     { "Panasonic DC-S1", { 9744,-3905,-779,-4899,12807,2324,-798,1630,5827 } },
+    { "Panasonic DC-S1H", { 9397,-3719,-805,-5425,13326,2309,-972,1715,6034 } },
     { "Panasonic DC-S1R", { 11822,-5321,-1249,-5958,15114,766,-614,1264,7043 } },
     { "Panasonic DC-G9", { 7685,-2375,-634,-3687,11700,2249,-748,1546,5111 } },
     { "Panasonic DC-G95", { 9657,-3963,-748,-3361,11378,2258,-568,1415,5158 } },
     { "Panasonic DC-GF9", { 7610,-2780,-576,-4614,12195,2733,-1375,2393,6490 } },
+    { "Panasonic DC-GX880", { 7610,-2780,-576,-4614,12195,2733,-1375,2393,6490 } },
     { "Panasonic DC-GX9", { 7564,-2263,-606,-3148,11239,2177,-540,1435,4853 } },
     { "Panasonic DC-TZ90", { 9052,-3117,-883,-3045,11346,1927,-205,1520,4730 } },
+    { "Panasonic DC-TZ96", { 12194,-5340,-1329,-3035,11394,1858,-50,1418,5219 } },
     { "Panasonic DC-TZ202", { 7790,-2736,-755,-3452,11870,1769,-628,1647,4898 } },
     { "Phase One H 20", { 1313,1855,-109,-6715,15908,808,-327,1840,6020 } }, /* DJC */
     { "Phase One H 25", { 2905,732,-237,-8134,16626,1476,-3038,4253,7517 } },
@@ -661,11 +674,13 @@ static void dt_dcraw_adobe_coeff(const char *name, float cam_xyz[1][12])
     { "Sony ILCE-5000", { 5991,-1456,-455,-4764,12135,2980,-707,1425,6701 } },
     { "Sony ILCE-5100", { 5991,-1456,-455,-4764,12135,2980,-707,1425,6701 } },
     { "Sony ILCE-6000", { 5991,-1456,-455,-4764,12135,2980,-707,1425,6701 } },
+    { "Sony ILCE-6100", { 7657,-2847,-607,-4083,11966,2389,-684,1418,5844 } },
     { "Sony ILCE-6300", { 5973,-1695,-419,-3826,11797,2293,-639,1398,5789 } },
     { "Sony ILCE-6400", { 7657,-2847,-607,-4083,11966,2389,-684,1418,5844 } },
     { "Sony ILCE-6500", { 5973,-1695,-419,-3826,11797,2293,-639,1398,5789 } },
     { "Sony ILCE-6600", { 7657,-2847,-607,-4083,11966,2389,-684,1418,5844 } },
     { "Sony ILCE-9", { 6389,-1703,-378,-4562,12265,2587,-670,1489,6550 } },
+    { "Sony ILCE-9M2", { 6389,-1703,-378,-4562,12265,2587,-670,1489,6550 } },
     { "Sony NEX-5N", { 5991,-1456,-455,-4764,12135,2980,-707,1425,6701 } },
     { "Sony NEX-5R", { 6129,-1545,-418,-4930,12490,2743,-977,1693,6615 } },
     { "Sony NEX-5T", { 6129,-1545,-418,-4930,12490,2743,-977,1693,6615 } },
