@@ -20,11 +20,13 @@
 */
 
 #include "decoders/RawDecoder.h"
-#include "common/Common.h"                          // for uint32_t, roundUpD...
+#include "common/Common.h"                          // for roundUpDivision
 #include "common/Point.h"                           // for iPoint2D, iRecta...
 #include "decoders/RawDecoderException.h"           // for ThrowRDE
 #include "decompressors/UncompressedDecompressor.h" // for UncompressedDeco...
-#include "io/Buffer.h"                              // for Buffer
+#include "io/Buffer.h"                              // for Buffer, DataBuffer
+#include "io/ByteStream.h"                          // for ByteStream
+#include "io/Endianness.h"                          // for Endianness, Endi...
 #include "io/FileIOException.h"                     // for FileIOException
 #include "io/IOException.h"                         // for IOException
 #include "metadata/BlackArea.h"                     // for BlackArea
@@ -36,6 +38,7 @@
 #include "tiff/TiffEntry.h"                         // for TiffEntry
 #include "tiff/TiffIFD.h"                           // for TiffIFD
 #include "tiff/TiffTag.h"                           // for BITSPERSAMPLE
+#include <algorithm>                                // for max
 #include <array>                                    // for array
 #include <cassert>                                  // for assert
 #include <string>                                   // for string, basic_st...
