@@ -1,6 +1,6 @@
 /*
     This file is part of darktable,
-    copyright (c) 2011 robert bieber.
+    Copyright (C) 2011-2020 darktable developers.
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -67,6 +67,11 @@ void dt_accel_path_lua(char *s, size_t n, const char *path)
   snprintf(s, n, "<Darktable>/%s/%s", NC_("accel", "lua"), path);
 }
 
+void dt_accel_path_manual(char *s, size_t n, const char *full_path)
+{
+  snprintf(s, n, "<Darktable>/%s", full_path);
+}
+
 static void dt_accel_path_global_translated(char *s, size_t n, const char *path)
 {
   snprintf(s, n, "<Darktable>/%s/%s", C_("accel", "global"), g_dpgettext2(NULL, "accel", path));
@@ -114,6 +119,11 @@ static void dt_accel_path_lua_translated(char *s, size_t n, const char *path)
   snprintf(s, n, "<Darktable>/%s/%s", C_("accel", "lua"), g_dpgettext2(NULL, "accel", path));
 }
 
+static void dt_accel_path_manual_translated(char *s, size_t n, const char *full_path)
+{
+  snprintf(s, n, "<Darktable>/%s", g_dpgettext2(NULL, "accel", full_path));
+}
+
 void dt_accel_register_global(const gchar *path, guint accel_key, GdkModifierType mods)
 {
   gchar accel_path[256];
@@ -169,6 +179,42 @@ void dt_accel_register_iop(dt_iop_module_so_t *so, gboolean local, const gchar *
   darktable.control->accelerator_list = g_slist_prepend(darktable.control->accelerator_list, accel);
 }
 
+void dt_accel_register_lib_as_view(gchar *view_name, const gchar *path, guint accel_key, GdkModifierType mods)
+{
+  //register a lib shortcut but place it in the path of a view
+  gchar accel_path[256];
+  dt_accel_path_view(accel_path, sizeof(accel_path), view_name, path);
+  if (dt_accel_find_by_path(accel_path)) return; // return if nothing to add, to avoid multiple entries
+
+  dt_accel_t *accel = (dt_accel_t *)g_malloc(sizeof(dt_accel_t));
+  gtk_accel_map_add_entry(accel_path, accel_key, mods);
+  g_strlcpy(accel->path, accel_path, sizeof(accel->path));
+
+  snprintf(accel_path, sizeof(accel_path), "<Darktable>/%s/%s/%s", C_("accel", "views"),
+           g_dgettext(NULL, view_name),
+           g_dpgettext2(NULL, "accel", path));
+
+  g_strlcpy(accel->translated_path, accel_path, sizeof(accel->translated_path));
+
+  g_strlcpy(accel->module, view_name, sizeof(accel->module));
+  accel->local = FALSE;
+
+  if(strcmp(view_name, "lighttable") == 0)
+    accel->views = DT_VIEW_LIGHTTABLE;
+  else if(strcmp(view_name, "darkroom") == 0)
+    accel->views = DT_VIEW_DARKROOM;
+  else if(strcmp(view_name, "print") == 0)
+    accel->views = DT_VIEW_PRINT;
+  else if(strcmp(view_name, "slideshow") == 0)
+    accel->views = DT_VIEW_SLIDESHOW;
+  else if(strcmp(view_name, "map") == 0)
+    accel->views = DT_VIEW_MAP;
+  else if(strcmp(view_name, "tethering") == 0)
+    accel->views = DT_VIEW_TETHERING;
+
+  darktable.control->accelerator_list = g_slist_prepend(darktable.control->accelerator_list, accel);
+}
+
 void dt_accel_register_lib_for_views(dt_lib_module_t *self, dt_view_type_flags_t views, const gchar *path,
                                      guint accel_key, GdkModifierType mods)
 {
@@ -212,11 +258,80 @@ void dt_accel_register_lib(dt_lib_module_t *self, const gchar *path, guint accel
     else if(strcmp(views[i], "*") == 0)
       v |= DT_VIEW_DARKROOM | DT_VIEW_LIGHTTABLE | DT_VIEW_TETHERING | DT_VIEW_MAP | DT_VIEW_PRINT
            | DT_VIEW_SLIDESHOW;
-    i++;  
+    i++;
   }
   dt_accel_register_lib_for_views(self, v, path, accel_key, mods);
 }
 
+void dt_accel_register_combobox_iop(dt_iop_module_so_t *so, gboolean local, const gchar *path)
+{
+  gchar accel_next_path[256];
+  gchar accel_next_path_trans[256];
+  gchar accel_prev_path[256];
+  gchar accel_prev_path_trans[256];
+  gchar accel_dynamic_path[256];
+  gchar accel_dynamic_path_trans[256];
+
+  dt_accel_t *accel_next = (dt_accel_t *)g_malloc(sizeof(dt_accel_t));
+  dt_accel_t *accel_prev = (dt_accel_t *)g_malloc(sizeof(dt_accel_t));
+  dt_accel_t *accel_dynamic = (dt_accel_t *)g_malloc(sizeof(dt_accel_t));
+
+  gchar *module_name_fixed = dt_util_str_replace(so->name(), "/", "-");
+
+  //accel to select next value
+  snprintf(accel_next_path, 256, "<Darktable>/%s/%s/%s/%s", NC_("accel", "image operations"), so->op, path,
+           NC_("accel", "next"));
+  snprintf(accel_next_path_trans, 256, "<Darktable>/%s/%s/%s/%s", C_("accel", "image operations"), module_name_fixed,
+           g_dpgettext2(NULL, "accel", path), C_("accel", "next"));
+  gtk_accel_map_add_entry(accel_next_path, 0, 0);
+  g_strlcpy(accel_next->path, accel_next_path, sizeof(accel_next->path));
+  g_strlcpy(accel_next->translated_path, accel_next_path_trans, sizeof(accel_next->translated_path));
+  g_strlcpy(accel_next->module, so->op, sizeof(accel_next->module));
+  accel_next->local = local;
+  accel_next->views = DT_VIEW_DARKROOM;
+  darktable.control->accelerator_list = g_slist_prepend(darktable.control->accelerator_list, accel_next);
+
+  //accel to select previous value
+  snprintf(accel_prev_path, 256, "<Darktable>/%s/%s/%s/%s", NC_("accel", "image operations"), so->op, path,
+           NC_("accel", "previous"));
+  snprintf(accel_prev_path_trans, 256, "<Darktable>/%s/%s/%s/%s", C_("accel", "image operations"), module_name_fixed,
+           g_dpgettext2(NULL, "accel", path), C_("accel", "previous"));
+  gtk_accel_map_add_entry(accel_prev_path, 0, 0);
+  g_strlcpy(accel_prev->path, accel_prev_path, sizeof(accel_prev->path));
+  g_strlcpy(accel_prev->translated_path, accel_prev_path_trans, sizeof(accel_prev->translated_path));
+  g_strlcpy(accel_prev->module, so->op, sizeof(accel_prev->module));
+  accel_prev->local = local;
+  accel_prev->views = DT_VIEW_DARKROOM;
+  darktable.control->accelerator_list = g_slist_prepend(darktable.control->accelerator_list, accel_prev);
+
+  // dynamic accel
+  snprintf(accel_dynamic_path, 256, "<Darktable>/%s/%s/%s/%s", NC_("accel", "image operations"), so->op, path,
+           NC_("accel", "dynamic"));
+  snprintf(accel_dynamic_path_trans, 256, "<Darktable>/%s/%s/%s/%s", C_("accel", "image operations"), module_name_fixed,
+           g_dpgettext2(NULL, "accel", path), C_("accel", "dynamic"));
+  gtk_accel_map_add_entry(accel_dynamic_path, 0, 0);
+  g_strlcpy(accel_dynamic->path, accel_dynamic_path, sizeof(accel_dynamic->path));
+  g_strlcpy(accel_dynamic->translated_path, accel_dynamic_path_trans, sizeof(accel_dynamic->translated_path));
+  g_strlcpy(accel_dynamic->module, so->op, sizeof(accel_dynamic->module));
+  accel_dynamic->local = local;
+  accel_dynamic->views = DT_VIEW_DARKROOM;
+  darktable.control->accelerator_list = g_slist_prepend(darktable.control->accelerator_list, accel_dynamic);
+
+  dt_accel_dynamic_t *daccel = (dt_accel_dynamic_t *)g_malloc0(sizeof(dt_accel_dynamic_t));
+
+  g_strlcpy(daccel->path, accel_dynamic_path, sizeof(daccel->path));
+  g_strlcpy(daccel->translated_path, accel_dynamic_path_trans, sizeof(daccel->translated_path));
+  g_strlcpy(daccel->module, so->op, sizeof(daccel->module));
+  daccel->local = local;
+  daccel->views = DT_VIEW_DARKROOM;
+  daccel->mod_so = so;
+
+  darktable.control->dynamic_accelerator_list
+      = g_slist_prepend(darktable.control->dynamic_accelerator_list, daccel);
+
+  g_free(module_name_fixed);
+}
+
 void dt_accel_register_slider_iop(dt_iop_module_so_t *so, gboolean local, const gchar *path)
 {
   gchar increase_path[256];
@@ -234,13 +349,12 @@ void dt_accel_register_slider_iop(dt_iop_module_so_t *so, gboolean local, const
   char *paths_trans[]
       = { increase_path_trans, decrease_path_trans, reset_path_trans, edit_path_trans, dynamic_path_trans };
 
-  int i = 0;
   dt_accel_t *accel = NULL;
 
   dt_accel_paths_slider_iop(paths, 256, so->op, path);
   dt_accel_paths_slider_iop_translated(paths_trans, 256, so, path);
 
-  for(i = 0; i < 5; i++)
+  for(int i = 0; i < 5; i++)
   {
     gtk_accel_map_add_entry(paths[i], 0, 0);
     accel = (dt_accel_t *)g_malloc(sizeof(dt_accel_t));
@@ -286,6 +400,24 @@ void dt_accel_register_lua(const gchar *path, guint accel_key, GdkModifierType m
   darktable.control->accelerator_list = g_slist_prepend(darktable.control->accelerator_list, accel);
 }
 
+void dt_accel_register_manual(const gchar *full_path, dt_view_type_flags_t views, guint accel_key,
+                              GdkModifierType mods)
+{
+  gchar accel_path[256];
+  dt_accel_t *accel = (dt_accel_t *)g_malloc(sizeof(dt_accel_t));
+
+  dt_accel_path_manual(accel_path, sizeof(accel_path), full_path);
+  gtk_accel_map_add_entry(accel_path, accel_key, mods);
+
+  g_strlcpy(accel->path, accel_path, sizeof(accel->path));
+  dt_accel_path_manual_translated(accel_path, sizeof(accel_path), full_path);
+  g_strlcpy(accel->translated_path, accel_path, sizeof(accel->translated_path));
+
+  *(accel->module) = '\0';
+  accel->local = FALSE;
+  accel->views = views;
+  darktable.control->accelerator_list = g_slist_prepend(darktable.control->accelerator_list, accel);
+}
 
 static dt_accel_t *_lookup_accel(const gchar *path)
 {
@@ -319,6 +451,37 @@ void dt_accel_connect_view(dt_view_t *self, const gchar *path, GClosure *closure
   self->accel_closures = g_slist_prepend(self->accel_closures, laccel);
 }
 
+dt_accel_t *dt_accel_connect_lib_as_view(dt_lib_module_t *module, gchar *view_name, const gchar *path, GClosure *closure)
+{
+  gchar accel_path[256];
+  dt_accel_path_view(accel_path, sizeof(accel_path), view_name, path);
+  gtk_accel_group_connect_by_path(darktable.control->accelerators, accel_path, closure);
+
+  dt_accel_t *accel = _lookup_accel(accel_path);
+  if(!accel) return NULL; // this happens when the path doesn't match any accel (typos, ...)
+
+  accel->closure = closure;
+
+  module->accel_closures = g_slist_prepend(module->accel_closures, accel);
+  return accel;
+}
+
+dt_accel_t *dt_accel_connect_lib_as_global(dt_lib_module_t *module, const gchar *path, GClosure *closure)
+{
+  gchar accel_path[256];
+  dt_accel_path_global(accel_path, sizeof(accel_path), path);
+
+  dt_accel_t *accel = _lookup_accel(accel_path);
+  if(!accel) return NULL; // this happens when the path doesn't match any accel (typos, ...)
+
+  gtk_accel_group_connect_by_path(darktable.control->accelerators, accel_path, closure);
+
+  accel->closure = closure;
+
+  module->accel_closures = g_slist_prepend(module->accel_closures, accel);
+  return accel;
+}
+
 static void _connect_local_accel(dt_iop_module_t *module, dt_accel_t *accel)
 {
   module->accel_closures_local = g_slist_prepend(module->accel_closures_local, accel);
@@ -326,12 +489,10 @@ static void _connect_local_accel(dt_iop_module_t *module, dt_accel_t *accel)
 
 dt_accel_t *dt_accel_connect_iop(dt_iop_module_t *module, const gchar *path, GClosure *closure)
 {
-  dt_accel_t *accel = NULL;
   gchar accel_path[256];
   dt_accel_path_iop(accel_path, sizeof(accel_path), module->op, path);
   // Looking up the entry in the global accelerators list
-  accel = _lookup_accel(accel_path);
-
+  dt_accel_t *accel = _lookup_accel(accel_path);
   if(accel) accel->closure = closure;
 
   if(accel && accel->local)
@@ -373,6 +534,16 @@ void dt_accel_connect_lua(const gchar *path, GClosure *closure)
   gtk_accel_group_connect_by_path(darktable.control->accelerators, accel_path, closure);
 }
 
+void dt_accel_connect_manual(GSList **list_ptr, const gchar *full_path, GClosure *closure)
+{
+  gchar accel_path[256];
+  dt_accel_path_manual(accel_path, sizeof(accel_path), full_path);
+  dt_accel_t *accel = _lookup_accel(accel_path);
+  accel->closure = closure;
+  gtk_accel_group_connect_by_path(darktable.control->accelerators, accel_path, closure);
+  *list_ptr = g_slist_prepend(*list_ptr, accel);
+}
+
 static gboolean _press_button_callback(GtkAccelGroup *accel_group, GObject *acceleratable, guint keyval,
                                        GdkModifierType modifier, gpointer data)
 {
@@ -426,6 +597,16 @@ void dt_accel_connect_button_lib(dt_lib_module_t *module, const gchar *path, Gtk
     g_signal_connect(G_OBJECT(button), "query-tooltip", G_CALLBACK(_tooltip_callback), NULL);
 }
 
+void dt_accel_connect_button_lib_as_global(dt_lib_module_t *module, const gchar *path, GtkWidget *button)
+{
+  GClosure *closure = g_cclosure_new(G_CALLBACK(_press_button_callback), button, NULL);
+  dt_accel_t *accel = dt_accel_connect_lib_as_global(module, path, closure);
+  g_object_set_data(G_OBJECT(button), "dt-accel", accel);
+
+  if(gtk_widget_get_has_tooltip(button))
+    g_signal_connect(G_OBJECT(button), "query-tooltip", G_CALLBACK(_tooltip_callback), NULL);
+}
+
 static gboolean bauhaus_slider_edit_callback(GtkAccelGroup *accel_group, GObject *acceleratable, guint keyval,
                                              GdkModifierType modifier, gpointer data)
 {
@@ -436,6 +617,74 @@ static gboolean bauhaus_slider_edit_callback(GtkAccelGroup *accel_group, GObject
   return TRUE;
 }
 
+void dt_accel_widget_toast(GtkWidget *widget)
+{
+  dt_bauhaus_widget_t *w = (dt_bauhaus_widget_t *)DT_BAUHAUS_WIDGET(widget);
+
+  if(!gtk_widget_is_visible(GTK_WIDGET(w)) && !darktable.gui->reset)
+  {
+    char *text = NULL;
+    int show = 1;
+
+    switch(w->type){
+      case DT_BAUHAUS_SLIDER:
+      {
+        dt_bauhaus_slider_data_t *d = &w->data.slider;
+        show = !d->is_dragging;
+        text = dt_bauhaus_slider_get_text(widget);
+        break;
+      }
+      case DT_BAUHAUS_COMBOBOX:
+        text = g_strdup(dt_bauhaus_combobox_get_text(widget));
+        break;
+      default: //literally impossible but hey
+        return;
+        break;
+    }
+
+    if (show)
+    {
+      if(w->label[0] != '\0')
+      { // label is not empty
+        if(w->module && w->module->multi_name[0] != '\0')
+          dt_toast_log(_("%s %s / %s: %s"), w->module->name(), w->module->multi_name, w->label, text);
+        else if(w->module && !strstr(w->module->name(), w->label))
+          dt_toast_log(_("%s / %s: %s"), w->module->name(), w->label, text);
+        else
+          dt_toast_log(_("%s: %s"), w->label, text);
+      }
+      else
+      { //label is empty
+        if(w->module && w->module->multi_name[0] != '\0')
+          dt_toast_log(_("%s %s / %s"), w->module->name(), w->module->multi_name, text);
+        else if(w->module)
+          dt_toast_log(_("%s / %s"), w->module->name(), text);
+        else
+          dt_toast_log(_("%s"), text);
+      }
+    }
+
+    g_free(text);
+  }
+
+}
+
+float dt_accel_get_slider_scale_multiplier()
+{
+  const int slider_precision = dt_conf_get_int("accel/slider_precision");
+  
+  if(slider_precision == DT_IOP_PRECISION_COARSE)
+  {
+    return dt_conf_get_float("darkroom/ui/scale_rough_step_multiplier");
+  }
+  else if(slider_precision == DT_IOP_PRECISION_FINE)
+  {
+    return dt_conf_get_float("darkroom/ui/scale_precise_step_multiplier");
+  }
+
+  return dt_conf_get_float("darkroom/ui/scale_step_multiplier");
+}
+
 static gboolean bauhaus_slider_increase_callback(GtkAccelGroup *accel_group, GObject *acceleratable,
                                                  guint keyval, GdkModifierType modifier, gpointer data)
 {
@@ -443,10 +692,17 @@ static gboolean bauhaus_slider_increase_callback(GtkAccelGroup *accel_group, GOb
 
   float value = dt_bauhaus_slider_get(slider);
   float step = dt_bauhaus_slider_get_step(slider);
+  float multiplier = dt_accel_get_slider_scale_multiplier();
 
-  dt_bauhaus_slider_set(slider, value + step);
+  const float min_visible = powf(10.0f, -dt_bauhaus_slider_get_digits(slider));
+  if(fabsf(step*multiplier) < min_visible) 
+    multiplier = min_visible / fabsf(step);
+
+  dt_bauhaus_slider_set(slider, value + step * multiplier);
 
   g_signal_emit_by_name(G_OBJECT(slider), "value-changed");
+
+  dt_accel_widget_toast(slider);
   return TRUE;
 }
 
@@ -457,10 +713,17 @@ static gboolean bauhaus_slider_decrease_callback(GtkAccelGroup *accel_group, GOb
 
   float value = dt_bauhaus_slider_get(slider);
   float step = dt_bauhaus_slider_get_step(slider);
+  float multiplier = dt_accel_get_slider_scale_multiplier();
 
-  dt_bauhaus_slider_set(slider, value - step);
+  const float min_visible = powf(10.0f, -dt_bauhaus_slider_get_digits(slider));
+  if(fabsf(step*multiplier) < min_visible) 
+    multiplier = min_visible / fabsf(step);
+
+  dt_bauhaus_slider_set(slider, value - step * multiplier);
 
   g_signal_emit_by_name(G_OBJECT(slider), "value-changed");
+
+  dt_accel_widget_toast(slider);
   return TRUE;
 }
 
@@ -472,9 +735,87 @@ static gboolean bauhaus_slider_reset_callback(GtkAccelGroup *accel_group, GObjec
   dt_bauhaus_slider_reset(slider);
 
   g_signal_emit_by_name(G_OBJECT(slider), "value-changed");
+
+  dt_accel_widget_toast(slider);
   return TRUE;
 }
 
+static gboolean bauhaus_combobox_next_callback(GtkAccelGroup *accel_group, GObject *acceleratable,
+                                                 guint keyval, GdkModifierType modifier, gpointer data)
+{
+  GtkWidget *combobox = GTK_WIDGET(data);
+
+  const int currentval = dt_bauhaus_combobox_get(combobox);
+  const int nextval = currentval + 1 >= dt_bauhaus_combobox_length(combobox) ? 0 : currentval + 1;
+  dt_bauhaus_combobox_set(combobox, nextval);
+
+  dt_accel_widget_toast(combobox);
+
+  return TRUE;
+}
+
+static gboolean bauhaus_combobox_prev_callback(GtkAccelGroup *accel_group, GObject *acceleratable,
+                                                 guint keyval, GdkModifierType modifier, gpointer data)
+{
+  GtkWidget *combobox = GTK_WIDGET(data);
+
+  const int currentval = dt_bauhaus_combobox_get(combobox);
+  const int prevval = currentval - 1 < 0 ? dt_bauhaus_combobox_length(combobox) : currentval - 1;
+  dt_bauhaus_combobox_set(combobox, prevval);
+
+  dt_accel_widget_toast(combobox);
+
+  return TRUE;
+}
+
+void dt_accel_connect_combobox_iop(dt_iop_module_t *module, const gchar *path, GtkWidget *combobox)
+{
+  gchar accel_next_path[256];
+  gchar accel_prev_path[256];
+  gchar accel_dynamic_path[256];
+
+  assert(DT_IS_BAUHAUS_WIDGET(combobox));
+
+  GClosure *closure;
+
+  //accel to select next value
+  snprintf(accel_next_path, 256, "<Darktable>/%s/%s/%s/%s", NC_("accel", "image operations"), module->so->op, path,
+           NC_("accel", "next"));
+  closure = g_cclosure_new(G_CALLBACK(bauhaus_combobox_next_callback), (gpointer)combobox, NULL);
+  dt_accel_t *accel = _lookup_accel(accel_next_path);
+  if(accel) accel->closure = closure;
+  gtk_accel_group_connect_by_path(darktable.control->accelerators, accel_next_path, closure);
+  module->accel_closures = g_slist_prepend(module->accel_closures, accel);
+
+  //accel to select previous value
+  snprintf(accel_prev_path, 256, "<Darktable>/%s/%s/%s/%s", NC_("accel", "image operations"), module->so->op, path,
+           NC_("accel", "previous"));
+  closure = g_cclosure_new(G_CALLBACK(bauhaus_combobox_prev_callback), (gpointer)combobox, NULL);
+  accel = _lookup_accel(accel_prev_path);
+  if(accel) accel->closure = closure;
+  gtk_accel_group_connect_by_path(darktable.control->accelerators, accel_prev_path, closure);
+  module->accel_closures = g_slist_prepend(module->accel_closures, accel);
+
+  //dynamic
+  snprintf(accel_dynamic_path, 256, "<Darktable>/%s/%s/%s/%s", NC_("accel", "image operations"), module->so->op, path,
+           NC_("accel", "dynamic"));
+
+  // dynamic accel : no closure, as we'll use key_press/release/scroll
+  GSList *l = darktable.control->dynamic_accelerator_list;
+  while(l)
+  {
+    dt_accel_dynamic_t *da = (dt_accel_dynamic_t *)l->data;
+    if(da && !strcmp(da->path, accel_dynamic_path))
+    {
+      da->widget = combobox;
+      break;
+    }
+    l = g_slist_next(l);
+  }
+  accel = _lookup_accel(accel_dynamic_path);
+  module->accel_closures = g_slist_prepend(module->accel_closures, accel);
+}
+
 void dt_accel_connect_slider_iop(dt_iop_module_t *module, const gchar *path, GtkWidget *slider)
 {
   gchar increase_path[256];
@@ -482,7 +823,6 @@ void dt_accel_connect_slider_iop(dt_iop_module_t *module, const gchar *path, Gtk
   gchar reset_path[256];
   gchar edit_path[256];
   gchar dynamic_path[256];
-  dt_accel_t *accel = NULL;
   GClosure *closure;
   char *paths[] = { increase_path, decrease_path, reset_path, edit_path, dynamic_path };
   dt_accel_paths_slider_iop(paths, 256, module->op, path);
@@ -491,7 +831,7 @@ void dt_accel_connect_slider_iop(dt_iop_module_t *module, const gchar *path, Gtk
 
   closure = g_cclosure_new(G_CALLBACK(bauhaus_slider_increase_callback), (gpointer)slider, NULL);
 
-  accel = _lookup_accel(increase_path);
+  dt_accel_t *accel = _lookup_accel(increase_path);
 
   if(accel) accel->closure = closure;
 
@@ -571,12 +911,10 @@ void dt_accel_connect_slider_iop(dt_iop_module_t *module, const gchar *path, Gtk
 
 void dt_accel_connect_locals_iop(dt_iop_module_t *module)
 {
-  dt_accel_t *accel;
   GSList *l = module->accel_closures_local;
-
   while(l)
   {
-    accel = (dt_accel_t *)l->data;
+    dt_accel_t *accel = (dt_accel_t *)l->data;
     if(accel) gtk_accel_group_connect_by_path(darktable.control->accelerators, accel->path, accel->closure);
     l = g_slist_next(l);
   }
@@ -584,27 +922,26 @@ void dt_accel_connect_locals_iop(dt_iop_module_t *module)
   module->local_closures_connected = TRUE;
 }
 
-void dt_accel_disconnect_list(GSList *list)
+void dt_accel_disconnect_list(GSList **list_ptr)
 {
-  dt_accel_t *accel;
+  GSList *list = *list_ptr;
   while(list)
   {
-    accel = (dt_accel_t *)list->data;
+    dt_accel_t *accel = (dt_accel_t *)list->data;
     if(accel) gtk_accel_group_disconnect(darktable.control->accelerators, accel->closure);
     list = g_slist_delete_link(list, list);
   }
+  *list_ptr = NULL;
 }
 
 void dt_accel_disconnect_locals_iop(dt_iop_module_t *module)
 {
-  dt_accel_t *accel;
-  GSList *l = module->accel_closures_local;
-
   if(!module->local_closures_connected) return;
 
+  GSList *l = module->accel_closures_local;
   while(l)
   {
-    accel = (dt_accel_t *)l->data;
+    dt_accel_t *accel = (dt_accel_t *)l->data;
     if(accel)
     {
       g_closure_ref(accel->closure);
@@ -618,11 +955,10 @@ void dt_accel_disconnect_locals_iop(dt_iop_module_t *module)
 
 void dt_accel_cleanup_locals_iop(dt_iop_module_t *module)
 {
-  dt_accel_t *accel;
   GSList *l = module->accel_closures_local;
   while(l)
   {
-    accel = (dt_accel_t *)l->data;
+    dt_accel_t *accel = (dt_accel_t *)l->data;
     if(accel && module->local_closures_connected)
       gtk_accel_group_disconnect(darktable.control->accelerators, accel->closure);
     l = g_slist_delete_link(l, l);
@@ -700,7 +1036,6 @@ static gboolean preset_iop_module_callback(GtkAccelGroup *accel_group, GObject *
 
 void dt_accel_connect_preset_iop(dt_iop_module_t *module, const gchar *path)
 {
-  GClosure *closure = NULL;
   char build_path[1024];
   gchar *name = g_strdup(path);
   snprintf(build_path, sizeof(build_path), "%s/%s", _("preset"), name);
@@ -709,8 +1044,8 @@ void dt_accel_connect_preset_iop(dt_iop_module_t *module, const gchar *path)
   callback_description->module = module;
   callback_description->name = name;
 
-  closure = g_cclosure_new(G_CALLBACK(preset_iop_module_callback), callback_description,
-                           preset_iop_module_callback_destroyer);
+  GClosure *closure = g_cclosure_new(G_CALLBACK(preset_iop_module_callback), callback_description,
+                                     preset_iop_module_callback_destroyer);
   dt_accel_connect_iop(module, build_path, closure);
 }
 
@@ -785,7 +1120,6 @@ static gboolean preset_lib_module_callback(GtkAccelGroup *accel_group, GObject *
 
 void dt_accel_connect_preset_lib(dt_lib_module_t *module, const gchar *path)
 {
-  GClosure *closure = NULL;
   char build_path[1024];
   gchar *name = g_strdup(path);
   snprintf(build_path, sizeof(build_path), "%s/%s", _("preset"), name);
@@ -794,8 +1128,8 @@ void dt_accel_connect_preset_lib(dt_lib_module_t *module, const gchar *path)
   callback_description->module = module;
   callback_description->name = name;
 
-  closure = g_cclosure_new(G_CALLBACK(preset_lib_module_callback), callback_description,
-                           preset_lib_module_callback_destroyer);
+  GClosure *closure = g_cclosure_new(G_CALLBACK(preset_lib_module_callback), callback_description,
+                                     preset_lib_module_callback_destroyer);
   dt_accel_connect_lib(module, build_path, closure);
 }
 
@@ -869,10 +1203,9 @@ void dt_accel_deregister_iop(dt_iop_module_t *module, const gchar *path)
 
 void dt_accel_deregister_lib(dt_lib_module_t *module, const gchar *path)
 {
-  GSList *l;
   char build_path[1024];
   dt_accel_path_lib(build_path, sizeof(build_path), module->plugin_name, path);
-  l = module->accel_closures;
+  GSList *l = module->accel_closures;
   while(l)
   {
     dt_accel_t *accel = (dt_accel_t *)l->data;
@@ -906,10 +1239,9 @@ void dt_accel_deregister_lib(dt_lib_module_t *module, const gchar *path)
 
 void dt_accel_deregister_global(const gchar *path)
 {
-  GSList *l;
   char build_path[1024];
   dt_accel_path_global(build_path, sizeof(build_path), path);
-  l = darktable.control->accelerator_list;
+  GSList *l = darktable.control->accelerator_list;
   while(l)
   {
     dt_accel_t *accel = (dt_accel_t *)l->data;
@@ -929,10 +1261,9 @@ void dt_accel_deregister_global(const gchar *path)
 
 void dt_accel_deregister_lua(const gchar *path)
 {
-  GSList *l;
   char build_path[1024];
   dt_accel_path_lua(build_path, sizeof(build_path), path);
-  l = darktable.control->accelerator_list;
+  GSList *l = darktable.control->accelerator_list;
   while(l)
   {
     dt_accel_t *accel = (dt_accel_t *)l->data;
@@ -950,6 +1281,43 @@ void dt_accel_deregister_lua(const gchar *path)
   }
 }
 
+void dt_accel_deregister_manual(GSList *list, const gchar *full_path)
+{
+  GSList *l;
+  char build_path[1024];
+  dt_accel_path_manual(build_path, sizeof(build_path), full_path);
+  l = list;
+  while(l)
+  {
+    dt_accel_t *accel = (dt_accel_t *)l->data;
+    if(accel && !strncmp(accel->path, build_path, 1024))
+    {
+      list = g_slist_delete_link(list, l);
+      gtk_accel_group_disconnect(darktable.control->accelerators, accel->closure);
+      l = NULL;
+    }
+    else
+    {
+      l = g_slist_next(l);
+    }
+  }
+  l = darktable.control->accelerator_list;
+  while(l)
+  {
+    dt_accel_t *accel = (dt_accel_t *)l->data;
+    if(accel && !strncmp(accel->path, build_path, 1024))
+    {
+      darktable.control->accelerator_list = g_slist_delete_link(darktable.control->accelerator_list, l);
+      l = NULL;
+      g_free(accel);
+    }
+    else
+    {
+      l = g_slist_next(l);
+    }
+  }
+}
+
 gboolean find_accel_internal(GtkAccelKey *key, GClosure *closure, gpointer data)
 {
   return (closure == data);
@@ -957,13 +1325,12 @@ gboolean find_accel_internal(GtkAccelKey *key, GClosure *closure, gpointer data)
 
 void dt_accel_rename_preset_iop(dt_iop_module_t *module, const gchar *path, const gchar *new_path)
 {
-  dt_accel_t *accel;
-  GSList *l = module->accel_closures;
   char build_path[1024];
   dt_accel_path_iop(build_path, sizeof(build_path), module->op, path);
+  GSList *l = module->accel_closures;
   while(l)
   {
-    accel = (dt_accel_t *)l->data;
+    dt_accel_t *accel = (dt_accel_t *)l->data;
     if(accel && !strncmp(accel->path, build_path, 1024))
     {
       GtkAccelKey tmp_key
@@ -984,13 +1351,12 @@ void dt_accel_rename_preset_iop(dt_iop_module_t *module, const gchar *path, cons
 
 void dt_accel_rename_preset_lib(dt_lib_module_t *module, const gchar *path, const gchar *new_path)
 {
-  dt_accel_t *accel;
-  GSList *l = module->accel_closures;
   char build_path[1024];
   dt_accel_path_lib(build_path, sizeof(build_path), module->plugin_name, path);
+  GSList *l = module->accel_closures;
   while(l)
   {
-    accel = (dt_accel_t *)l->data;
+    dt_accel_t *accel = (dt_accel_t *)l->data;
     if(accel && !strncmp(accel->path, build_path, 1024))
     {
       GtkAccelKey tmp_key
@@ -1010,13 +1376,12 @@ void dt_accel_rename_preset_lib(dt_lib_module_t *module, const gchar *path, cons
 
 void dt_accel_rename_global(const gchar *path, const gchar *new_path)
 {
-  dt_accel_t *accel;
-  GSList *l = darktable.control->accelerator_list;
   char build_path[1024];
   dt_accel_path_global(build_path, sizeof(build_path), path);
+  GSList *l = darktable.control->accelerator_list;
   while(l)
   {
-    accel = (dt_accel_t *)l->data;
+    dt_accel_t *accel = (dt_accel_t *)l->data;
     if(accel && !strncmp(accel->path, build_path, 1024))
     {
       GtkAccelKey tmp_key
@@ -1037,13 +1402,12 @@ void dt_accel_rename_global(const gchar *path, const gchar *new_path)
 
 void dt_accel_rename_lua(const gchar *path, const gchar *new_path)
 {
-  dt_accel_t *accel;
-  GSList *l = darktable.control->accelerator_list;
   char build_path[1024];
   dt_accel_path_lua(build_path, sizeof(build_path), path);
+  GSList *l = darktable.control->accelerator_list;
   while(l)
   {
-    accel = (dt_accel_t *)l->data;
+    dt_accel_t *accel = (dt_accel_t *)l->data;
     if(accel && !strncmp(accel->path, build_path, 1024))
     {
       GtkAccelKey tmp_key
@@ -1074,7 +1438,7 @@ static gint _dynamic_accel_find(gconstpointer a, gconstpointer b)
 dt_accel_dynamic_t *dt_dynamic_accel_find_by_key(guint accel_key, GdkModifierType mods)
 {
   GtkAccelKey ak = { 0 };
-  ak.accel_key = accel_key;
+  ak.accel_key = gdk_keyval_to_lower(accel_key);
   ak.accel_mods = mods;
   GSList *da = g_slist_find_custom(darktable.control->dynamic_accelerator_valid, &ak, _dynamic_accel_find);
   if(da && da->data) return (dt_accel_dynamic_t *)da->data;
