@@ -1,6 +1,6 @@
 /*
     This file is part of darktable,
-    copyright (c) 2009--2010 johannes hanika.
+    Copyright (C) 2009-2020 darktable developers.
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -27,6 +27,8 @@
 #include "common/curve_tools.h"
 #include "common/darktable.h"
 #include "common/splines.h"
+#include "control/conf.h"
+#include "develop/develop.h"
 #include <cairo.h>
 #include <glib.h>
 #include <math.h>
@@ -44,6 +46,38 @@ typedef struct dt_draw_curve_t
   CurveSample csample;
 } dt_draw_curve_t;
 
+/** set color based on gui overlay preference */
+static inline void dt_draw_set_color_overlay(cairo_t *cr, double amt, double alpha)
+{
+  const int overlay_color = dt_conf_get_int("darkroom/ui/overlay_color");
+
+  if(overlay_color == DT_DEV_OVERLAY_GRAY)
+  {
+    cairo_set_source_rgba(cr, 1.0 * amt, 1.0 * amt, 1.0 * amt, alpha);
+  }
+  else if(overlay_color == DT_DEV_OVERLAY_RED)
+  {
+    cairo_set_source_rgba(cr, 1.0 * amt, 0.0, 0.0, alpha);
+  }
+  else if(overlay_color == DT_DEV_OVERLAY_GREEN)
+  {
+    cairo_set_source_rgba(cr, 0.0, 1.0 * amt, 0.0, alpha);
+  }
+  else if(overlay_color == DT_DEV_OVERLAY_YELLOW)
+  {
+    cairo_set_source_rgba(cr, 1.0 * amt, 1.0 * amt, 0.0, alpha);
+  }
+  else if(overlay_color == DT_DEV_OVERLAY_CYAN)
+  {
+    cairo_set_source_rgba(cr, 0.0, 1.0 * amt, 1.0 * amt, alpha);
+  }
+  else if(overlay_color == DT_DEV_OVERLAY_MAGENTA)
+  {
+    cairo_set_source_rgba(cr, 1.0 * amt, 0.0, 1.0 * amt, alpha);
+  }
+
+}
+
 /** draws a rating star
  */
 static inline void dt_draw_star(cairo_t *cr, float x, float y, float r1, float r2)
