@@ -1,6 +1,6 @@
 /*
     This file is part of darktable,
-    copyright (c) 2009--2010 henrik andersson.
+    Copyright (C) 2010-2020 darktable developers.
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -74,22 +74,50 @@ static int _single_selected_imgid()
   return imgid;
 }
 
+static gboolean _gui_styles_is_copy_module_order_set(dt_gui_styles_dialog_t *d)
+{
+  /* first item is the copy-module */
+  GtkTreeIter iter;
+  GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(d->items));
+
+  gboolean active = FALSE;
+  gint num = 0;
+  if(gtk_tree_model_get_iter_first(model, &iter))
+    gtk_tree_model_get(model, &iter, DT_STYLE_ITEMS_COL_ENABLED, &active, DT_STYLE_ITEMS_COL_NUM, &num, -1);
+  return active && (num == -1);
+}
+
+static gboolean _gui_styles_is_update_module_order_set(dt_gui_styles_dialog_t *d)
+{
+  /* first item is the copy-module */
+  GtkTreeIter iter;
+  GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(d->items));
+
+  gboolean active = FALSE;
+  gint num = 0;
+  if(gtk_tree_model_get_iter_first(model, &iter))
+    gtk_tree_model_get(model, &iter, DT_STYLE_ITEMS_COL_UPDATE, &active, DT_STYLE_ITEMS_COL_NUM, &num, -1);
+  return active && (num == -1);
+}
+
 void _gui_styles_get_active_items(dt_gui_styles_dialog_t *sd, GList **enabled, GList **update)
 {
   /* run through all items and add active ones to result */
   GtkTreeIter iter;
   GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(sd->items));
-  int num = 0, update_num = 0;
+  gint num = 0, update_num = 0;
   gboolean active, uactive;
 
   if(gtk_tree_model_get_iter_first(model, &iter))
   {
     do
     {
-      gtk_tree_model_get(model, &iter, DT_STYLE_ITEMS_COL_ENABLED, &active, DT_STYLE_ITEMS_COL_UPDATE,
-                         &uactive, DT_STYLE_ITEMS_COL_NUM, &num, DT_STYLE_ITEMS_COL_UPDATE_NUM, &update_num,
+      gtk_tree_model_get(model, &iter, DT_STYLE_ITEMS_COL_ENABLED, &active,
+                         DT_STYLE_ITEMS_COL_UPDATE, &uactive,
+                         DT_STYLE_ITEMS_COL_NUM, &num,
+                         DT_STYLE_ITEMS_COL_UPDATE_NUM, &update_num,
                          -1);
-      if(active || uactive)
+      if((active || uactive) && num >= 0)
       {
         *enabled = g_list_append(*enabled, GINT_TO_POINTER(num));
         if(update != NULL)
@@ -109,8 +137,11 @@ void _gui_styles_get_active_items(dt_gui_styles_dialog_t *sd, GList **enabled, G
   {
     do
     {
-      gtk_tree_model_get(model, &iter, DT_STYLE_ITEMS_COL_ENABLED, &active, DT_STYLE_ITEMS_COL_NUM, &num,
-                         DT_STYLE_ITEMS_COL_UPDATE_NUM, &update_num, -1);
+      gtk_tree_model_get(model, &iter,
+                         DT_STYLE_ITEMS_COL_ENABLED, &active,
+                         DT_STYLE_ITEMS_COL_NUM, &num,
+                         DT_STYLE_ITEMS_COL_UPDATE_NUM, &update_num,
+                         -1);
       if(active)
       {
         if(update_num == -1) // item from style
@@ -128,9 +159,34 @@ void _gui_styles_get_active_items(dt_gui_styles_dialog_t *sd, GList **enabled, G
   }
 }
 
+static void _gui_styles_select_all_items(dt_gui_styles_dialog_t *d, gboolean active)
+{
+  /* run through all items and set active status */
+  GtkTreeView *items = (d->duplicate) ? d->items_new : d->items;
+  GtkTreeIter iter;
+  GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(items));
+  if(gtk_tree_model_get_iter_first(model, &iter))
+  {
+    do
+    {
+      gtk_list_store_set(GTK_LIST_STORE(model), &iter, DT_STYLE_ITEMS_COL_ENABLED, active, -1);
+    } while(gtk_tree_model_iter_next(model, &iter));
+  }
+}
+
 static void _gui_styles_new_style_response(GtkDialog *dialog, gint response_id, dt_gui_styles_dialog_t *g)
 {
-  if(response_id == GTK_RESPONSE_ACCEPT)
+  if(response_id == GTK_RESPONSE_YES)
+  {
+    _gui_styles_select_all_items(g, TRUE);
+    return;
+  }
+  else if(response_id == GTK_RESPONSE_NONE)
+  {
+    _gui_styles_select_all_items(g, FALSE);
+    return;
+  }
+  else if(response_id == GTK_RESPONSE_ACCEPT)
   {
     /* get the filtered list from dialog */
     GList *result = NULL;
@@ -139,7 +195,8 @@ static void _gui_styles_new_style_response(GtkDialog *dialog, gint response_id,
     /* create the style from imageid */
     const gchar *name = gtk_entry_get_text(GTK_ENTRY(g->name));
     if(name && *name)
-      if(dt_styles_create_from_image(name, gtk_entry_get_text(GTK_ENTRY(g->description)), g->imgid, result))
+      if(dt_styles_create_from_image(name, gtk_entry_get_text(GTK_ENTRY(g->description)),
+                                     g->imgid, result, _gui_styles_is_copy_module_order_set(g)))
       {
         dt_control_log(_("style named '%s' successfully created"), name);
       };
@@ -151,7 +208,17 @@ static void _gui_styles_new_style_response(GtkDialog *dialog, gint response_id,
 
 static void _gui_styles_edit_style_response(GtkDialog *dialog, gint response_id, dt_gui_styles_dialog_t *g)
 {
-  if(response_id == GTK_RESPONSE_ACCEPT)
+  if(response_id == GTK_RESPONSE_YES)
+  {
+    _gui_styles_select_all_items(g, TRUE);
+    return;
+  }
+  else if(response_id == GTK_RESPONSE_NONE)
+  {
+    _gui_styles_select_all_items(g, FALSE);
+    return;
+  }
+  else if(response_id == GTK_RESPONSE_ACCEPT)
   {
     /* get the filtered list from dialog */
     GList *result = NULL, *update = NULL;
@@ -163,16 +230,19 @@ static void _gui_styles_edit_style_response(GtkDialog *dialog, gint response_id,
     {
       if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(g->duplicate)))
       {
-        dt_styles_create_from_style(g->nameorig, name, gtk_entry_get_text(GTK_ENTRY(g->description)), result,
-                                    g->imgid, update);
-        dt_control_log(_("style %s was successfully saved"), name);
+        dt_styles_create_from_style(g->nameorig, name, gtk_entry_get_text(GTK_ENTRY(g->description)),
+                                    result, g->imgid, update,
+                                    _gui_styles_is_copy_module_order_set(g),
+                                    _gui_styles_is_update_module_order_set(g));
       }
       else
       {
-        dt_styles_update(g->nameorig, name, gtk_entry_get_text(GTK_ENTRY(g->description)), result, g->imgid,
-                         update);
-        dt_control_log(_("style %s was successfully saved"), name);
+        dt_styles_update(g->nameorig, name, gtk_entry_get_text(GTK_ENTRY(g->description)),
+                         result, g->imgid, update,
+                         _gui_styles_is_copy_module_order_set(g),
+                         _gui_styles_is_update_module_order_set(g));
       }
+      dt_control_log(_("style %s was successfully saved"), name);
     }
   }
   gtk_widget_destroy(GTK_WIDGET(dialog));
@@ -188,11 +258,14 @@ static void _gui_styles_item_toggled(GtkCellRendererToggle *cell, gchar *path_st
   GtkTreePath *path = gtk_tree_path_new_from_string(path_str);
   GtkTreeIter iter;
   gboolean toggle_item;
-  int num, update_num;
+  gint num, update_num;
 
   gtk_tree_model_get_iter(model, &iter, path);
-  gtk_tree_model_get(model, &iter, DT_STYLE_ITEMS_COL_ENABLED, &toggle_item, DT_STYLE_ITEMS_COL_NUM, &num,
-                     DT_STYLE_ITEMS_COL_UPDATE_NUM, &update_num, -1);
+  gtk_tree_model_get(model, &iter,
+                     DT_STYLE_ITEMS_COL_ENABLED, &toggle_item,
+                     DT_STYLE_ITEMS_COL_NUM, &num,
+                     DT_STYLE_ITEMS_COL_UPDATE_NUM, &update_num,
+                     -1);
 
   toggle_item = (toggle_item == TRUE) ? FALSE : TRUE;
 
@@ -275,13 +348,16 @@ static void _gui_styles_dialog_run(gboolean edit, const char *name, int imgid)
   }
   else
   {
-    snprintf(title, sizeof(title), "%s", _("create new style"));
+    g_strlcpy(title, _("create new style"), sizeof(title));
     sd->duplicate = NULL;
   }
   GtkWidget *window = dt_ui_main_window(darktable.gui->ui);
   GtkDialog *dialog = GTK_DIALOG(
-      gtk_dialog_new_with_buttons(title, GTK_WINDOW(window), GTK_DIALOG_DESTROY_WITH_PARENT, _("_cancel"),
-                                  GTK_RESPONSE_REJECT, _("_save"), GTK_RESPONSE_ACCEPT, NULL));
+      gtk_dialog_new_with_buttons(title, GTK_WINDOW(window), GTK_DIALOG_DESTROY_WITH_PARENT,
+                                  _("_cancel"), GTK_RESPONSE_REJECT,
+                                  _("select _all"),  GTK_RESPONSE_YES,
+                                  _("select _none"), GTK_RESPONSE_NONE,
+                                  _("_save"), GTK_RESPONSE_ACCEPT, NULL));
 #ifdef GDK_WINDOWING_QUARTZ
   dt_osx_disallow_fullscreen(GTK_WIDGET(dialog));
 #endif
@@ -393,6 +469,12 @@ static void _gui_styles_dialog_run(gboolean edit, const char *name, int imgid)
   GtkTreeIter iter;
   if(edit)
   {
+    gtk_list_store_append(GTK_LIST_STORE(liststore), &iter);
+    gtk_list_store_set(GTK_LIST_STORE(liststore), &iter,
+                       DT_STYLE_ITEMS_COL_ENABLED, dt_styles_has_module_order(name),
+                       DT_STYLE_ITEMS_COL_NAME, _("modules order"),
+                       DT_STYLE_ITEMS_COL_NUM, -1,
+                       -1);
     /* get history items for named style and populate the items list */
     GList *items = dt_styles_get_item_list(name, FALSE, imgid);
     if(items)
@@ -404,9 +486,12 @@ static void _gui_styles_dialog_run(gboolean edit, const char *name, int imgid)
         if(item->num != -1 && item->selimg_num != -1) // defined in style and image
         {
           gtk_list_store_append(GTK_LIST_STORE(liststore), &iter);
-          gtk_list_store_set(GTK_LIST_STORE(liststore), &iter, DT_STYLE_ITEMS_COL_ENABLED, TRUE,
-                             DT_STYLE_ITEMS_COL_UPDATE, FALSE, DT_STYLE_ITEMS_COL_NAME, item->name,
-                             DT_STYLE_ITEMS_COL_NUM, item->num, DT_STYLE_ITEMS_COL_UPDATE_NUM,
+          gtk_list_store_set(GTK_LIST_STORE(liststore), &iter,
+                             DT_STYLE_ITEMS_COL_ENABLED, TRUE,
+                             DT_STYLE_ITEMS_COL_UPDATE, FALSE,
+                             DT_STYLE_ITEMS_COL_NAME, item->name,
+                             DT_STYLE_ITEMS_COL_NUM, item->num,
+                             DT_STYLE_ITEMS_COL_UPDATE_NUM,
                              item->selimg_num, -1);
           has_item = TRUE;
         }
@@ -414,10 +499,12 @@ static void _gui_styles_dialog_run(gboolean edit, const char *name, int imgid)
                 || item->selimg_num != -1) // defined in one or the other, let a way to select it or not
         {
           gtk_list_store_append(GTK_LIST_STORE(liststore_new), &iter);
-          gtk_list_store_set(GTK_LIST_STORE(liststore_new), &iter, DT_STYLE_ITEMS_COL_ENABLED,
-                             item->num != -1 ? TRUE : FALSE, DT_STYLE_ITEMS_COL_NAME, item->name,
-                             DT_STYLE_ITEMS_COL_NUM, item->num, DT_STYLE_ITEMS_COL_UPDATE_NUM,
-                             item->selimg_num, -1);
+          gtk_list_store_set(GTK_LIST_STORE(liststore_new), &iter,
+                             DT_STYLE_ITEMS_COL_ENABLED, item->num != -1 ? TRUE : FALSE,
+                             DT_STYLE_ITEMS_COL_NAME, item->name,
+                             DT_STYLE_ITEMS_COL_NUM, item->num,
+                             DT_STYLE_ITEMS_COL_UPDATE_NUM, item->selimg_num,
+                             -1);
           has_new_item = TRUE;
         }
       } while((items = g_list_next(items)));
@@ -426,6 +513,16 @@ static void _gui_styles_dialog_run(gboolean edit, const char *name, int imgid)
   }
   else
   {
+    const dt_iop_order_t order = dt_ioppr_get_iop_order_version(imgid);
+    char *label = g_strdup_printf("%s (%s)", _("modules order"), dt_iop_order_string(order));
+    gtk_list_store_append(GTK_LIST_STORE(liststore), &iter);
+    gtk_list_store_set(GTK_LIST_STORE(liststore), &iter,
+                       DT_STYLE_ITEMS_COL_ENABLED, TRUE,
+                       DT_STYLE_ITEMS_COL_NAME, label,
+                       DT_STYLE_ITEMS_COL_NUM, -1,
+                       -1);
+    g_free(label);
+
     GList *items = dt_history_get_items(imgid, FALSE);
     if(items)
     {
@@ -449,11 +546,14 @@ static void _gui_styles_dialog_run(gboolean edit, const char *name, int imgid)
         }
 
         gchar iname[256] = { 0 };
-        g_snprintf(iname, sizeof(iname), "%s", item->name);
+        g_strlcpy(iname, item->name, sizeof(iname));
 
         gtk_list_store_append(GTK_LIST_STORE(liststore), &iter);
-        gtk_list_store_set(GTK_LIST_STORE(liststore), &iter, DT_STYLE_ITEMS_COL_ENABLED, enabled,
-                           DT_STYLE_ITEMS_COL_NAME, iname, DT_STYLE_ITEMS_COL_NUM, item->num, -1);
+        gtk_list_store_set(GTK_LIST_STORE(liststore), &iter,
+                           DT_STYLE_ITEMS_COL_ENABLED, enabled,
+                           DT_STYLE_ITEMS_COL_NAME, iname,
+                           DT_STYLE_ITEMS_COL_NUM, item->num,
+                           -1);
 
         has_item = TRUE;
 
