@@ -1,6 +1,6 @@
 /*
     This file is part of darktable,
-    copyright (c) 2013 Google Inc.
+    Copyright (C) 2013-2020 darktable developers.
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -123,7 +123,8 @@ static int FileWriter(const uint8_t *data, size_t data_size, const WebPPicture *
 
 int write_image(dt_imageio_module_data_t *webp, const char *filename, const void *in_tmp,
                 dt_colorspaces_color_profile_type_t over_type, const char *over_filename,
-                void *exif, int exif_len, int imgid, int num, int total, struct dt_dev_pixelpipe_t *pipe)
+                void *exif, int exif_len, int imgid, int num, int total, struct dt_dev_pixelpipe_t *pipe,
+                const gboolean export_masks)
 {
   FILE *out = NULL;
   WebPPicture pic;
@@ -182,9 +183,7 @@ int write_image(dt_imageio_module_data_t *webp, const char *filename, const void
   }
 
   WebPPictureFree(&pic);
-  pic_init = 0;
   fclose(out);
-  out = NULL;
 
   dt_exif_write_blob(exif, exif_len, filename, 1);
 
