@@ -1,7 +1,6 @@
 /*
     This file is part of darktable,
-    copyright (c) 2013 jeremy Rosen
-    copyright (c) 2016 Roman Lebedev.
+    Copyright (C) 2016-2020 darktable developers.
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -67,9 +66,10 @@ int initialize_store(struct dt_imageio_module_storage_t *self, struct dt_imageio
                      GList **images, const gboolean high_quality, const gboolean upscale);
 /* this actually does the work */
 int store(struct dt_imageio_module_storage_t *self, struct dt_imageio_module_data_t *self_data, const int imgid,
-          struct dt_imageio_module_format_t *format, struct dt_imageio_module_data_t *fdata, const int num, const int total,
-          const gboolean high_quality, const gboolean upscale, enum dt_colorspaces_color_profile_type_t icc_type,
-          const gchar *icc_filename, enum dt_iop_color_intent_t icc_intent, struct dt_export_metadata_t *metadata);
+          struct dt_imageio_module_format_t *format, struct dt_imageio_module_data_t *fdata, const int num,
+          const int total, const gboolean high_quality, const gboolean upscale, const gboolean export_masks,
+          const enum dt_colorspaces_color_profile_type_t icc_type, const gchar *icc_filename,
+          enum dt_iop_color_intent_t icc_intent, struct dt_export_metadata_t *metadata);
 /* called once at the end (after exporting all images), if implemented. */
 void finalize_store(struct dt_imageio_module_storage_t *self, struct dt_imageio_module_data_t *data);
 
