@@ -1,6 +1,6 @@
 /*
     This file is part of darktable,
-    copyright (c) 2009--2011 johannes hanika.
+    Copyright (C) 2010-2020 darktable developers.
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -27,11 +27,13 @@
 #include "develop/develop.h"
 #include "develop/imageop.h"
 #include "develop/imageop_math.h"
+#include "develop/imageop_gui.h"
 #include "develop/tiling.h"
 #include "dtgtk/drawingarea.h"
 #include "gui/draw.h"
 #include "gui/gtk.h"
 #include "gui/presets.h"
+#include "gui/accelerators.h"
 #include "iop/iop_api.h"
 
 #include <assert.h>
@@ -51,8 +53,8 @@ DT_MODULE_INTROSPECTION(6, dt_iop_basecurve_params_t)
 
 typedef struct dt_iop_basecurve_node_t
 {
-  float x;
-  float y;
+  float x; // $MIN: 0.0 $MAX: 1.0
+  float y; // $MIN: 0.0 $MAX: 1.0
 } dt_iop_basecurve_node_t;
 
 typedef struct dt_iop_basecurve_params_t
@@ -60,12 +62,15 @@ typedef struct dt_iop_basecurve_params_t
   // three curves (c, ., .) with max number of nodes
   // the other two are reserved, maybe we'll have cam rgb at some point.
   dt_iop_basecurve_node_t basecurve[3][MAXNODES];
-  int basecurve_nodes[3];
-  int basecurve_type[3];
-  int exposure_fusion;    // number of exposure fusion steps
-  float exposure_stops;   // number of stops between fusion images
-  float exposure_bias;    // whether to do exposure-fusion with over or under-exposure
-  int preserve_colors;
+  int basecurve_nodes[3]; // $MIN: 0 $MAX: MAXNODES $DEFAULT: 0
+  int basecurve_type[3];  // $MIN: 0 $MAX: MONOTONE_HERMITE $DEFAULT: MONOTONE_HERMITE
+  int exposure_fusion;    /* number of exposure fusion steps
+                             $DEFAULT: 0 $DESCRIPTION: "fusion" */
+  float exposure_stops;   /* number of stops between fusion images
+                             $MIN: 0.01 $MAX: 4.0 $DEFAULT: 1.0 $DESCRIPTION: "exposure shift" */
+  float exposure_bias;    /* whether to do exposure-fusion with over or under-exposure
+                             $MIN: -1.0 $MAX: 1.0 $DEFAULT: 1.0 $DESCRIPTION: "exposure bias" */
+  dt_iop_rgb_norms_t preserve_colors; /* $DEFAULT: DT_RGB_NORM_LUMINANCE $DESCRIPTION: "preserve colors" */
 } dt_iop_basecurve_params_t;
 
 typedef struct dt_iop_basecurve_params5_t
@@ -175,6 +180,40 @@ int legacy_params(dt_iop_module_t *self, const void *const old_params, const int
   return 1;
 }
 
+typedef struct dt_iop_basecurve_gui_data_t
+{
+  dt_draw_curve_t *minmax_curve; // curve for gui to draw
+  int minmax_curve_type, minmax_curve_nodes;
+  GtkBox *hbox;
+  GtkDrawingArea *area;
+  GtkWidget *scale, *fusion, *exposure_step, *exposure_bias;
+  GtkWidget *cmb_preserve_colors;
+  double mouse_x, mouse_y;
+  int selected;
+  int timeout_handle;
+  double selected_offset, selected_y, selected_min, selected_max;
+  float draw_xs[DT_IOP_TONECURVE_RES], draw_ys[DT_IOP_TONECURVE_RES];
+  float draw_min_xs[DT_IOP_TONECURVE_RES], draw_min_ys[DT_IOP_TONECURVE_RES];
+  float draw_max_xs[DT_IOP_TONECURVE_RES], draw_max_ys[DT_IOP_TONECURVE_RES];
+  int loglogscale;
+} dt_iop_basecurve_gui_data_t;
+
+void init_key_accels(dt_iop_module_so_t *self)
+{
+  dt_accel_register_combobox_iop(self, FALSE, NC_("accel", "scale"));
+  dt_accel_register_combobox_iop(self, FALSE, NC_("accel", "preserve colors"));
+  dt_accel_register_combobox_iop(self, FALSE, NC_("accel", "exposure fusion"));
+}
+
+void connect_key_accels(dt_iop_module_t *self)
+{
+  dt_iop_basecurve_gui_data_t *g = (dt_iop_basecurve_gui_data_t *)self->gui_data;
+
+  dt_accel_connect_combobox_iop(self, "scale", GTK_WIDGET(g->scale));
+  dt_accel_connect_combobox_iop(self, "preserve colors", GTK_WIDGET(g->cmb_preserve_colors));
+  dt_accel_connect_combobox_iop(self, "exposure fusion", GTK_WIDGET(g->fusion));
+}
+
 static const char neutral[] = N_("neutral");
 static const char canon_eos[] = N_("canon eos like");
 static const char canon_eos_alt[] = N_("canon eos like alternate");
@@ -270,24 +309,6 @@ static const basecurve_preset_t basecurve_presets[] = {
 #undef m
 static const int basecurve_presets_cnt = sizeof(basecurve_presets) / sizeof(basecurve_preset_t);
 
-typedef struct dt_iop_basecurve_gui_data_t
-{
-  dt_draw_curve_t *minmax_curve; // curve for gui to draw
-  int minmax_curve_type, minmax_curve_nodes;
-  GtkBox *hbox;
-  GtkDrawingArea *area;
-  GtkWidget *scale, *fusion, *exposure_step, *exposure_bias;
-  GtkWidget *cmb_preserve_colors;
-  double mouse_x, mouse_y;
-  int selected;
-  int timeout_handle;
-  double selected_offset, selected_y, selected_min, selected_max;
-  float draw_xs[DT_IOP_TONECURVE_RES], draw_ys[DT_IOP_TONECURVE_RES];
-  float draw_min_xs[DT_IOP_TONECURVE_RES], draw_min_ys[DT_IOP_TONECURVE_RES];
-  float draw_max_xs[DT_IOP_TONECURVE_RES], draw_max_ys[DT_IOP_TONECURVE_RES];
-  int loglogscale;
-} dt_iop_basecurve_gui_data_t;
-
 typedef struct dt_iop_basecurve_data_t
 {
   dt_draw_curve_t *curve; // curve for pixelpipe piece and pixel processing
@@ -327,6 +348,15 @@ const char *name()
   return _("base curve");
 }
 
+const char *description()
+{
+  return _("apply a view transform based on camera manufacturer look,\n"
+           "for corrective purposes, to prepare images for display.\n"
+           "works in RGB,\n"
+           "takes preferably a linear RGB input,\n"
+           "outputs non-linear RGB.");
+}
+
 int default_group()
 {
   return IOP_GROUP_BASIC;
@@ -344,7 +374,9 @@ int default_colorspace(dt_iop_module_t *self, dt_dev_pixelpipe_t *pipe, dt_dev_p
 
 static void set_presets(dt_iop_module_so_t *self, const basecurve_preset_t *presets, int count, gboolean camera)
 {
-  const gboolean autoapply = dt_conf_get_bool("plugins/darkroom/basecurve/auto_apply");
+  char *workflow = dt_conf_get_string("plugins/darkroom/workflow");
+  const gboolean autoapply = strcmp(workflow, "display-referred") == 0;
+  g_free(workflow);
   const gboolean autoapply_percamera = dt_conf_get_bool("plugins/darkroom/basecurve/auto_apply_percamera_presets");
 
   const gboolean force_autoapply = autoapply && (autoapply_percamera || !camera);
@@ -1411,16 +1443,9 @@ void gui_update(struct dt_iop_module_t *self)
   dt_iop_basecurve_gui_data_t *g = (dt_iop_basecurve_gui_data_t *)self->gui_data;
   dt_bauhaus_combobox_set(g->cmb_preserve_colors, p->preserve_colors);
   dt_bauhaus_combobox_set(g->fusion, p->exposure_fusion);
-  if(p->exposure_fusion != 0)
-  {
-    gtk_widget_set_visible(g->exposure_step, TRUE);
-    gtk_widget_set_visible(g->exposure_bias, TRUE);
-  }
-  if(p->exposure_fusion == 0)
-  {
-    gtk_widget_set_visible(g->exposure_step, FALSE);
-    gtk_widget_set_visible(g->exposure_bias, FALSE);
-  }
+
+  gtk_widget_set_visible(g->exposure_step, p->exposure_fusion != 0);
+  gtk_widget_set_visible(g->exposure_bias, p->exposure_fusion != 0);
 
   if(g->timeout_handle)
   {
@@ -1428,7 +1453,6 @@ void gui_update(struct dt_iop_module_t *self)
     g->timeout_handle = 0;
   }
 
-
   dt_bauhaus_slider_set(g->exposure_step, p->exposure_stops);
   dt_bauhaus_slider_set(g->exposure_bias, p->exposure_bias);
   // gui curve is read directly from params during expose event.
@@ -1437,33 +1461,14 @@ void gui_update(struct dt_iop_module_t *self)
 
 void init(dt_iop_module_t *module)
 {
-  module->params = calloc(1, sizeof(dt_iop_basecurve_params_t));
-  module->default_params = calloc(1, sizeof(dt_iop_basecurve_params_t));
-  module->default_enabled = 0;
-  module->params_size = sizeof(dt_iop_basecurve_params_t);
-  module->gui_data = NULL;
-  dt_iop_basecurve_params_t tmp = (dt_iop_basecurve_params_t){
-    {
-      { // three curves (L, a, b) with a number of nodes
-        { 0.0, 0.0 },
-        { 1.0, 1.0 }
-      },
-    },
-    { 2, 0, 0 }, // number of nodes per curve
-    { MONOTONE_HERMITE, MONOTONE_HERMITE, MONOTONE_HERMITE },
-    0, 1.0f, 1.0f, // no exposure fusion, but if we would, add one stop
-    DT_RGB_NORM_LUMINANCE
-  };
-  memcpy(module->params, &tmp, sizeof(dt_iop_basecurve_params_t));
-  memcpy(module->default_params, &tmp, sizeof(dt_iop_basecurve_params_t));
-}
+  dt_iop_default_init(module);
 
-void cleanup(dt_iop_module_t *module)
-{
-  free(module->params);
-  module->params = NULL;
-  free(module->default_params);
-  module->default_params = NULL;
+  dt_iop_basecurve_params_t *d = module->default_params;
+
+  d->basecurve[0][1].x = d->basecurve[0][1].y = 1.0;
+  d->basecurve_nodes[0] = 2;
+
+  memcpy(module->params, module->default_params, sizeof(dt_iop_basecurve_params_t));
 }
 
 void init_global(dt_iop_module_so_t *module)
@@ -2069,43 +2074,25 @@ static gboolean dt_iop_basecurve_key_press(GtkWidget *widget, GdkEventKey *event
 
 #undef BASECURVE_DEFAULT_STEP
 
-static void fusion_callback(GtkWidget *widget, gpointer user_data)
+void gui_changed(dt_iop_module_t *self, GtkWidget *w, void *previous)
 {
-  if(darktable.gui->reset) return;
-  dt_iop_module_t *self = (dt_iop_module_t *)user_data;
   dt_iop_basecurve_params_t *p = (dt_iop_basecurve_params_t *)self->params;
-  int fuse = dt_bauhaus_combobox_get(widget);
   dt_iop_basecurve_gui_data_t *g = (dt_iop_basecurve_gui_data_t *)self->gui_data;
-  if(p->exposure_fusion == 0 && fuse != 0)
-  {
-    gtk_widget_set_visible(g->exposure_step, TRUE);
-    gtk_widget_set_visible(g->exposure_bias, TRUE);
-  }
-  if(p->exposure_fusion != 0 && fuse == 0)
-  {
-    gtk_widget_set_visible(g->exposure_step, FALSE);
-    gtk_widget_set_visible(g->exposure_bias, FALSE);
-  }
-  p->exposure_fusion = fuse;
-  dt_dev_add_history_item(darktable.develop, self, TRUE);
-}
 
-static void exposure_step_callback(GtkWidget *widget, gpointer user_data)
-{
-  dt_iop_module_t *self = (dt_iop_module_t *)user_data;
-  dt_iop_basecurve_params_t *p = (dt_iop_basecurve_params_t *)self->params;
-  float stops = dt_bauhaus_slider_get(widget);
-  p->exposure_stops = stops;
-  dt_dev_add_history_item(darktable.develop, self, TRUE);
-}
-
-static void exposure_bias_callback(GtkWidget *widget, gpointer user_data)
-{
-  dt_iop_module_t *self = (dt_iop_module_t *)user_data;
-  dt_iop_basecurve_params_t *p = (dt_iop_basecurve_params_t *)self->params;
-  float bias = dt_bauhaus_slider_get(widget);
-  p->exposure_bias = bias;
-  dt_dev_add_history_item(darktable.develop, self, TRUE);
+  if(w == g->fusion)
+  {
+    int prev = *(int *)previous;
+    if(p->exposure_fusion != 0 && prev == 0)
+    {
+      gtk_widget_set_visible(g->exposure_step, TRUE);
+      gtk_widget_set_visible(g->exposure_bias, TRUE);
+    }
+    if(p->exposure_fusion == 0 && prev != 0)
+    {
+      gtk_widget_set_visible(g->exposure_step, FALSE);
+      gtk_widget_set_visible(g->exposure_bias, FALSE);
+    }
+  }
 }
 
 static void scale_callback(GtkWidget *widget, gpointer user_data)
@@ -2119,16 +2106,6 @@ static void scale_callback(GtkWidget *widget, gpointer user_data)
   gtk_widget_queue_draw(GTK_WIDGET(g->area));
 }
 
-static void preserve_colors_callback(GtkWidget *widget, dt_iop_module_t *self)
-{
-  if(darktable.gui->reset) return;
-  dt_iop_basecurve_params_t *p = (dt_iop_basecurve_params_t *)self->params;
-
-  p->preserve_colors = dt_bauhaus_combobox_get(widget);
-
-  dt_dev_add_history_item(darktable.develop, self, TRUE);
-}
-
 void gui_init(struct dt_iop_module_t *self)
 {
   self->gui_data = malloc(sizeof(dt_iop_basecurve_gui_data_t));
@@ -2146,7 +2123,7 @@ void gui_init(struct dt_iop_module_t *self)
   c->timeout_handle = 0;
 
   self->widget = gtk_box_new(GTK_ORIENTATION_VERTICAL, DT_BAUHAUS_SPACE);
-  dt_gui_add_help_link(self->widget, dt_get_help_url(self->op));
+
   c->area = GTK_DRAWING_AREA(dtgtk_drawing_area_new_with_aspect_ratio(1.0));
   gtk_widget_set_tooltip_text(GTK_WIDGET(c->area), _("abscissa: input, ordinate: output. works on RGB channels"));
 
@@ -2161,47 +2138,29 @@ void gui_init(struct dt_iop_module_t *self)
   gtk_box_pack_start(GTK_BOX(self->widget), c->scale, TRUE, TRUE, 0);
   g_signal_connect(G_OBJECT(c->scale), "value-changed", G_CALLBACK(scale_callback), self);
 
-  c->cmb_preserve_colors = dt_bauhaus_combobox_new(self);
-  dt_bauhaus_widget_set_label(c->cmb_preserve_colors, NULL, _("preserve colors"));
-  dt_bauhaus_combobox_add(c->cmb_preserve_colors, _("none"));
-  dt_bauhaus_combobox_add(c->cmb_preserve_colors, _("luminance"));
-  dt_bauhaus_combobox_add(c->cmb_preserve_colors, _("max RGB"));
-  dt_bauhaus_combobox_add(c->cmb_preserve_colors, _("average RGB"));
-  dt_bauhaus_combobox_add(c->cmb_preserve_colors, _("sum RGB"));
-  dt_bauhaus_combobox_add(c->cmb_preserve_colors, _("norm RGB"));
-  dt_bauhaus_combobox_add(c->cmb_preserve_colors, _("basic power"));
-  gtk_box_pack_start(GTK_BOX(self->widget), c->cmb_preserve_colors, TRUE, TRUE, 0);
+  c->cmb_preserve_colors = dt_bauhaus_combobox_from_params(self, "preserve_colors");
   gtk_widget_set_tooltip_text(c->cmb_preserve_colors, _("method to preserve colors when applying contrast"));
-  g_signal_connect(G_OBJECT(c->cmb_preserve_colors), "value-changed", G_CALLBACK(preserve_colors_callback), self);
 
-  c->fusion = dt_bauhaus_combobox_new(self);
-  dt_bauhaus_widget_set_label(c->fusion, NULL, _("fusion"));
+  c->fusion = dt_bauhaus_combobox_from_params(self, "exposure_fusion");
   dt_bauhaus_combobox_add(c->fusion, _("none"));
   dt_bauhaus_combobox_add(c->fusion, _("two exposures"));
   dt_bauhaus_combobox_add(c->fusion, _("three exposures"));
   gtk_widget_set_tooltip_text(c->fusion, _("fuse this image stopped up/down a couple of times with itself, to "
                                            "compress high dynamic range. expose for the highlights before use."));
-  gtk_box_pack_start(GTK_BOX(self->widget), c->fusion, TRUE, TRUE, 0);
-  g_signal_connect(G_OBJECT(c->fusion), "value-changed", G_CALLBACK(fusion_callback), self);
 
-  c->exposure_step = dt_bauhaus_slider_new_with_range(self, 0.01f, 4.0f, 0.100f, 1.0f, 3);
+  c->exposure_step = dt_bauhaus_slider_from_params(self, "exposure_stops");
+  dt_bauhaus_slider_set_digits(c->exposure_step, 3);
   gtk_widget_set_tooltip_text(c->exposure_step, _("how many stops to shift the individual exposures apart"));
-  dt_bauhaus_widget_set_label(c->exposure_step, NULL, _("exposure shift"));
-  gtk_box_pack_start(GTK_BOX(self->widget), c->exposure_step, TRUE, TRUE, 0);
-  g_signal_connect(G_OBJECT(c->exposure_step), "value-changed", G_CALLBACK(exposure_step_callback), self);
-  gtk_widget_show_all(c->exposure_step);
   gtk_widget_set_no_show_all(c->exposure_step, TRUE);
   gtk_widget_set_visible(c->exposure_step, p->exposure_fusion != 0 ? TRUE : FALSE);
 
   // initially set to 1 (consistency with previous versions), but double-click resets to 0
   // to get a quick way to reach 0 with the mouse.
-  c->exposure_bias = dt_bauhaus_slider_new_with_range(self, -1.0f, 1.0f, 0.100f, 0.0f, 3);
+  c->exposure_bias = dt_bauhaus_slider_from_params(self, "exposure_bias");
+  dt_bauhaus_slider_set_default(c->exposure_bias, 0.0f); 
+  dt_bauhaus_slider_set_digits(c->exposure_bias, 3); 
   gtk_widget_set_tooltip_text(c->exposure_bias, _("whether to shift exposure up or down "
                                                   "(-1: reduce highlight, +1: reduce shadows)"));
-  dt_bauhaus_widget_set_label(c->exposure_bias, NULL, _("exposure bias"));
-  gtk_box_pack_start(GTK_BOX(self->widget), c->exposure_bias, TRUE, TRUE, 0);
-  g_signal_connect(G_OBJECT(c->exposure_bias), "value-changed", G_CALLBACK(exposure_bias_callback), self);
-  gtk_widget_show_all(c->exposure_bias);
   gtk_widget_set_no_show_all(c->exposure_bias, TRUE);
   gtk_widget_set_visible(c->exposure_bias, p->exposure_fusion != 0 ? TRUE : FALSE);
 
