@@ -1,6 +1,6 @@
 /*
     This file is part of darktable,
-    copyright (c) 2010-2012 Henrik Andersson.
+    Copyright (C) 2010-2020 darktable developers.
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
 #include "develop/develop.h"
 #include "develop/imageop.h"
 #include "develop/imageop_math.h"
+#include "develop/imageop_gui.h"
 #include "develop/tiling.h"
 #include "gui/accelerators.h"
 #include "gui/gtk.h"
@@ -45,16 +46,14 @@ DT_MODULE_INTROSPECTION(1, dt_iop_bloom_params_t)
 
 typedef struct dt_iop_bloom_params_t
 {
-  float size;
-  float threshold;
-  float strength;
+  float size;       // $MIN: 0.0 $MAX: 100.0 $DEFAULT: 20.0
+  float threshold;  // $MIN: 0.0 $MAX: 100.0 $DEFAULT: 90.0
+  float strength;   // $MIN: 0.0 $MAX: 100.0 $DEFAULT: 25.0
 } dt_iop_bloom_params_t;
 
 typedef struct dt_iop_bloom_gui_data_t
 {
-  GtkBox *vbox;
-  GtkWidget *label1, *label2, *label3; // size,threshold,strength
-  GtkWidget *scale1, *scale2, *scale3; // size,threshold,strength
+  GtkWidget *size, *threshold, *strength; // size,threshold,strength
 } dt_iop_bloom_gui_data_t;
 
 typedef struct dt_iop_bloom_data_t
@@ -77,6 +76,15 @@ const char *name()
   return _("bloom");
 }
 
+const char *description()
+{
+  return _("apply Orton effect for a dreamy aetherical look,\n"
+           "for creative purposes.\n"
+           "works in Lab,\n"
+           "takes preferably a linear RGB input,\n"
+           "outputs non-linear RGB.");
+}
+
 int flags()
 {
   return IOP_FLAGS_INCLUDE_IN_STYLES | IOP_FLAGS_SUPPORTS_BLENDING;
@@ -102,9 +110,9 @@ void init_key_accels(dt_iop_module_so_t *self)
 void connect_key_accels(dt_iop_module_t *self)
 {
   const dt_iop_bloom_gui_data_t *g = (dt_iop_bloom_gui_data_t *)self->gui_data;
-  dt_accel_connect_slider_iop(self, "size", GTK_WIDGET(g->scale1));
-  dt_accel_connect_slider_iop(self, "threshold", GTK_WIDGET(g->scale2));
-  dt_accel_connect_slider_iop(self, "strength", GTK_WIDGET(g->scale3));
+  dt_accel_connect_slider_iop(self, "size", GTK_WIDGET(g->size));
+  dt_accel_connect_slider_iop(self, "threshold", GTK_WIDGET(g->threshold));
+  dt_accel_connect_slider_iop(self, "strength", GTK_WIDGET(g->strength));
 }
 
 #define GAUSS(a, b, c, x) (a * pow(2.718281828, (-pow((x - b), 2) / (pow(c, 2)))))
@@ -437,33 +445,6 @@ void cleanup_global(dt_iop_module_so_t *module)
   module->data = NULL;
 }
 
-static void strength_callback(GtkWidget *slider, gpointer user_data)
-{
-  dt_iop_module_t *self = (dt_iop_module_t *)user_data;
-  if(self->dt->gui->reset) return;
-  dt_iop_bloom_params_t *p = (dt_iop_bloom_params_t *)self->params;
-  p->strength = dt_bauhaus_slider_get(slider);
-  dt_dev_add_history_item(darktable.develop, self, TRUE);
-}
-
-static void threshold_callback(GtkWidget *slider, gpointer user_data)
-{
-  dt_iop_module_t *self = (dt_iop_module_t *)user_data;
-  if(self->dt->gui->reset) return;
-  dt_iop_bloom_params_t *p = (dt_iop_bloom_params_t *)self->params;
-  p->threshold = dt_bauhaus_slider_get(slider);
-  dt_dev_add_history_item(darktable.develop, self, TRUE);
-}
-
-static void size_callback(GtkWidget *slider, gpointer user_data)
-{
-  dt_iop_module_t *self = (dt_iop_module_t *)user_data;
-  if(self->dt->gui->reset) return;
-  dt_iop_bloom_params_t *p = (dt_iop_bloom_params_t *)self->params;
-  p->size = dt_bauhaus_slider_get(slider);
-  dt_dev_add_history_item(darktable.develop, self, TRUE);
-}
-
 void commit_params(struct dt_iop_module_t *self, dt_iop_params_t *p1, dt_dev_pixelpipe_t *pipe,
                    dt_dev_pixelpipe_iop_t *piece)
 {
@@ -489,74 +470,29 @@ void cleanup_pipe(struct dt_iop_module_t *self, dt_dev_pixelpipe_t *pipe, dt_dev
 
 void gui_update(struct dt_iop_module_t *self)
 {
-  const dt_iop_module_t *module = (dt_iop_module_t *)self;
-  const dt_iop_bloom_params_t *p = (dt_iop_bloom_params_t *)module->params;
+  const dt_iop_bloom_params_t *p = (dt_iop_bloom_params_t *)self->params;
   dt_iop_bloom_gui_data_t *g = (dt_iop_bloom_gui_data_t *)self->gui_data;
-  dt_bauhaus_slider_set(g->scale1, p->size);
-  dt_bauhaus_slider_set(g->scale2, p->threshold);
-  dt_bauhaus_slider_set(g->scale3, p->strength);
-}
-
-void init(dt_iop_module_t *module)
-{
-  module->params = calloc(1, sizeof(dt_iop_bloom_params_t));
-  module->default_params = calloc(1, sizeof(dt_iop_bloom_params_t));
-  module->default_enabled = 0;
-  module->params_size = sizeof(dt_iop_bloom_params_t);
-  module->gui_data = NULL;
-  dt_iop_bloom_params_t tmp = (dt_iop_bloom_params_t){ 20, 90, 25 };
-  memcpy(module->params, &tmp, sizeof(dt_iop_bloom_params_t));
-  memcpy(module->default_params, &tmp, sizeof(dt_iop_bloom_params_t));
-}
-
-void cleanup(dt_iop_module_t *module)
-{
-  free(module->params);
-  module->params = NULL;
-  free(module->default_params);
-  module->default_params = NULL;
+  dt_bauhaus_slider_set(g->size, p->size);
+  dt_bauhaus_slider_set(g->threshold, p->threshold);
+  dt_bauhaus_slider_set(g->strength, p->strength);
 }
 
 void gui_init(struct dt_iop_module_t *self)
 {
   self->gui_data = malloc(sizeof(dt_iop_bloom_gui_data_t));
   dt_iop_bloom_gui_data_t *g = (dt_iop_bloom_gui_data_t *)self->gui_data;
-  const dt_iop_bloom_params_t *p = (dt_iop_bloom_params_t *)self->params;
 
-  self->widget = gtk_box_new(GTK_ORIENTATION_VERTICAL, DT_BAUHAUS_SPACE);
-  dt_gui_add_help_link(self->widget, dt_get_help_url(self->op));
+  g->size = dt_bauhaus_slider_from_params(self, N_("size"));
+  dt_bauhaus_slider_set_format(g->size, "%.0f%%");
+  gtk_widget_set_tooltip_text(g->size, _("the size of bloom"));
 
-  /* size */
-  g->scale1 = dt_bauhaus_slider_new_with_range(self, 0.0, 100.0, 1.0, p->size, 0);
-  dt_bauhaus_slider_set_format(g->scale1, "%.0f%%");
-  dt_bauhaus_widget_set_label(g->scale1, NULL, _("size"));
-  gtk_widget_set_tooltip_text(g->scale1, _("the size of bloom"));
+  g->threshold = dt_bauhaus_slider_from_params(self, N_("threshold"));
+  dt_bauhaus_slider_set_format(g->threshold, "%.0f%%");
+  gtk_widget_set_tooltip_text(g->threshold, _("the threshold of light"));
 
-  /* threshold */
-  g->scale2 = dt_bauhaus_slider_new_with_range(self, 0.0, 100.0, 1.0, p->threshold, 0);
-  dt_bauhaus_slider_set_format(g->scale2, "%.0f%%");
-  dt_bauhaus_widget_set_label(g->scale2, NULL, _("threshold"));
-  gtk_widget_set_tooltip_text(g->scale2, _("the threshold of light"));
-
-  /* strength */
-  g->scale3 = dt_bauhaus_slider_new_with_range(self, 0.0, 100.0, 1.0, p->strength, 0);
-  dt_bauhaus_slider_set_format(g->scale3, "%.0f%%");
-  dt_bauhaus_widget_set_label(g->scale3, NULL, _("strength"));
-  gtk_widget_set_tooltip_text(g->scale3, _("the strength of bloom"));
-
-  gtk_box_pack_start(GTK_BOX(self->widget), GTK_WIDGET(g->scale1), TRUE, TRUE, 0);
-  gtk_box_pack_start(GTK_BOX(self->widget), GTK_WIDGET(g->scale2), TRUE, TRUE, 0);
-  gtk_box_pack_start(GTK_BOX(self->widget), GTK_WIDGET(g->scale3), TRUE, TRUE, 0);
-
-  g_signal_connect(G_OBJECT(g->scale1), "value-changed", G_CALLBACK(size_callback), self);
-  g_signal_connect(G_OBJECT(g->scale2), "value-changed", G_CALLBACK(threshold_callback), self);
-  g_signal_connect(G_OBJECT(g->scale3), "value-changed", G_CALLBACK(strength_callback), self);
-}
-
-void gui_cleanup(struct dt_iop_module_t *self)
-{
-  free(self->gui_data);
-  self->gui_data = NULL;
+  g->strength = dt_bauhaus_slider_from_params(self, N_("strength"));
+  dt_bauhaus_slider_set_format(g->strength, "%.0f%%");
+  gtk_widget_set_tooltip_text(g->strength, _("the strength of bloom"));
 }
 
 // modelines: These editor modelines have been set for all relevant files by tools/update_modelines.sh
