@@ -1,6 +1,6 @@
 /*
     This file is part of darktable,
-    copyright (c) 2011 johannes hanika.
+    Copyright (C) 2011-2020 darktable developers.
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -25,6 +25,7 @@
 #include "control/control.h"
 #include "develop/develop.h"
 #include "develop/imageop.h"
+#include "develop/imageop_gui.h"
 #include "dtgtk/button.h"
 #include "dtgtk/resetlabel.h"
 #include "dtgtk/togglebutton.h"
@@ -56,28 +57,32 @@ DT_MODULE_INTROSPECTION(3, dt_iop_borders_params_t)
 #define DT_IOP_BORDERS_POSITION_H_COUNT 5
 #define DT_IOP_BORDERS_POSITION_V_COUNT 5
 
-typedef enum dt_iop_borders_pickcolor_type_t
-{
-  DT_BORDERS_NONE = 0,
-  DT_BORDERS_FRAME,
-  DT_BORDERS_BORDER
-} dt_iop_borders_pickcolor_type_t;
-
 typedef struct dt_iop_borders_params_t
 {
-  float color[3];           // border color
-  float aspect;             // aspect ratio of the outer frame w/h
-  char aspect_text[20];     // aspect ratio of the outer frame w/h (user string version)
-  int aspect_orient;        // aspect ratio orientation
-  float size;               // border width relative to overal frame width
-  float pos_h;              // picture horizontal position ratio into the final image
-  char pos_h_text[20];      // picture horizontal position ratio into the final image (user string version)
-  float pos_v;              // picture vertical position ratio into the final image
-  char pos_v_text[20];      // picture vertical position ratio into the final image (user string version)
-  float frame_size;         // frame line width relative to border width
-  float frame_offset;       // frame offset from picture size relative to [border width - frame width]
-  float frame_color[3];     // frame line color
-  gboolean max_border_size; // the way border size is computed
+  float color[3];           // border color $DEFAULT: 1.0
+  float aspect;             /* aspect ratio of the outer frame w/h
+                               $MIN: 1.0 $MAX: 3.0 $DEFAULT: DT_IOP_BORDERS_ASPECT_CONSTANT_VALUE $DESCRIPTION: "aspect ratio" */
+  char aspect_text[20];     /* aspect ratio of the outer frame w/h (user string version)
+                               DEFAULT: "constant border" */
+  int aspect_orient;        /* aspect ratio orientation
+                               $DEFAULT: 0 $DESCRIPTION: "orientation" */
+  float size;               /* border width relative to overal frame width
+                               $MIN: 0.0 $MAX: 0.5 $DEFAULT: 0.1 $DESCRIPTION: "border size" */
+  float pos_h;              /* picture horizontal position ratio into the final image
+                               $MIN: 0.0 $MAX: 1.0 $DEFAULT: 0.5 $DESCRIPTION: "horizontal position" */
+  char pos_h_text[20];      /* picture horizontal position ratio into the final image (user string version)
+                               DEFAULT: "1/2" */
+  float pos_v;              /* picture vertical position ratio into the final image
+                               $MIN: 0.0 $MAX: 1.0 $DEFAULT: 0.5 $DESCRIPTION: "vertical position"*/
+  char pos_v_text[20];      /* picture vertical position ratio into the final image (user string version)
+                               DEFAULT: "1/2" */
+  float frame_size;         /* frame line width relative to border width
+                               $MIN: 0.0 $MAX: 1.0 $DEFAULT: 0.0 $DESCRIPTION: "frame line size" */
+  float frame_offset;       /* frame offset from picture size relative to [border width - frame width]
+                               $MIN: 0.0 $MAX: 1.0 $DEFAULT: 0.5 $DESCRIPTION: "frame line offset" */
+  float frame_color[3];     // frame line color $DEFAULT: 0.0
+  gboolean max_border_size; /* the way border size is computed
+                               $DEFAULT: TRUE */
 } dt_iop_borders_params_t;
 
 typedef struct dt_iop_borders_gui_data_t
@@ -91,16 +96,14 @@ typedef struct dt_iop_borders_gui_data_t
   GtkWidget *pos_v;
   GtkWidget *pos_v_slider;
   GtkWidget *colorpick;
-  GtkToggleButton *border_picker; // the 1st button
+  GtkWidget *border_picker; // the 1st button
   float aspect_ratios[DT_IOP_BORDERS_ASPECT_COUNT];
   float pos_h_ratios[DT_IOP_BORDERS_POSITION_H_COUNT];
   float pos_v_ratios[DT_IOP_BORDERS_POSITION_V_COUNT];
-  int combo_changed;
   GtkWidget *frame_size;
   GtkWidget *frame_offset;
   GtkWidget *frame_colorpick;
-  GtkToggleButton *frame_picker; // the 2nd button
-  dt_iop_color_picker_t color_picker;
+  GtkWidget *frame_picker; // the 2nd button
 } dt_iop_borders_gui_data_t;
 
 // ******* Check and update legacy params...(esp. ver 4)
@@ -201,6 +204,10 @@ void init_key_accels(dt_iop_module_so_t *self)
   dt_accel_register_slider_iop(self, FALSE, NC_("accel", "frame line size"));
   dt_accel_register_slider_iop(self, FALSE, NC_("accel", "frame line offset"));
   dt_accel_register_iop(self, FALSE, NC_("accel", "pick frame line color from image"), 0, 0);
+  dt_accel_register_combobox_iop(self, FALSE, NC_("accel", "aspect"));
+  dt_accel_register_combobox_iop(self, FALSE, NC_("accel", "orientation"));
+  dt_accel_register_combobox_iop(self, FALSE, NC_("accel", "horizontal position"));
+  dt_accel_register_combobox_iop(self, FALSE, NC_("accel", "vertical position"));
 }
 
 void connect_key_accels(dt_iop_module_t *self)
@@ -211,6 +218,10 @@ void connect_key_accels(dt_iop_module_t *self)
   dt_accel_connect_button_iop(self, "pick frame line color from image", GTK_WIDGET(g->frame_colorpick));
   dt_accel_connect_slider_iop(self, "frame line size", GTK_WIDGET(g->frame_size));
   dt_accel_connect_slider_iop(self, "frame line offset", GTK_WIDGET(g->frame_offset));
+  dt_accel_connect_combobox_iop(self, "aspect", GTK_WIDGET(g->aspect));
+  dt_accel_connect_combobox_iop(self, "orientation", GTK_WIDGET(g->aspect_orient));
+  dt_accel_connect_combobox_iop(self, "horizontal position", GTK_WIDGET(g->pos_h));
+  dt_accel_connect_combobox_iop(self, "vertical position", GTK_WIDGET(g->pos_v));
 }
 
 int distort_transform(dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, float *points, size_t points_count)
@@ -601,26 +612,7 @@ void init_presets(dt_iop_module_so_t *self)
   dt_gui_presets_add_generic(_("15:10 postcard black"), self->op, self->version(), &p, sizeof(p), 1);
 }
 
-static int _iop_color_picker_get_set(dt_iop_module_t *self, GtkWidget *button)
-{
-  dt_iop_borders_gui_data_t *g = (dt_iop_borders_gui_data_t *)self->gui_data;
-
-  const dt_iop_borders_pickcolor_type_t current_picker = g->color_picker.current_picker;
-
-  g->color_picker.current_picker = DT_BORDERS_NONE;
-
-  if(button == GTK_WIDGET(g->frame_picker))
-    g->color_picker.current_picker = DT_BORDERS_FRAME;
-  else if(button == GTK_WIDGET(g->border_picker))
-    g->color_picker.current_picker = DT_BORDERS_BORDER;
-
-  if (current_picker == g->color_picker.current_picker)
-    return DT_COLOR_PICKER_ALREADY_SELECTED;
-  else
-    return g->color_picker.current_picker;
-}
-
-static void _iop_color_picker_apply(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece)
+void color_picker_apply(dt_iop_module_t *self, GtkWidget *picker, dt_dev_pixelpipe_iop_t *piece)
 {
   dt_iop_borders_gui_data_t *g = (dt_iop_borders_gui_data_t *)self->gui_data;
   dt_iop_borders_params_t *p = (dt_iop_borders_params_t *)self->params;
@@ -646,14 +638,14 @@ static void _iop_color_picker_apply(struct dt_iop_module_t *self, dt_dev_pixelpi
                         .blue = self->picked_color[2],
                         .alpha = 1.0 };
 
-  if(g->color_picker.current_picker == DT_BORDERS_FRAME)
+  if(picker == g->frame_picker)
   {
     p->frame_color[0] = self->picked_color[0];
     p->frame_color[1] = self->picked_color[1];
     p->frame_color[2] = self->picked_color[2];
     gtk_color_chooser_set_rgba(GTK_COLOR_CHOOSER(g->frame_colorpick), &c);
   }
-  else if(g->color_picker.current_picker == DT_BORDERS_BORDER)
+  else if(picker == g->border_picker)
   {
     p->color[0] = self->picked_color[0];
     p->color[1] = self->picked_color[1];
@@ -664,17 +656,6 @@ static void _iop_color_picker_apply(struct dt_iop_module_t *self, dt_dev_pixelpi
   dt_dev_add_history_item(darktable.develop, self, TRUE);
 }
 
-static void _iop_color_picker_update(dt_iop_module_t *self)
-{
-  dt_iop_borders_gui_data_t *g = (dt_iop_borders_gui_data_t *)self->gui_data;
-  const dt_iop_borders_pickcolor_type_t which_colorpicker = g->color_picker.current_picker;
-  const int reset = darktable.gui->reset;
-  darktable.gui->reset = 1;
-  gtk_toggle_button_set_active(g->frame_picker, which_colorpicker == DT_BORDERS_FRAME);
-  gtk_toggle_button_set_active(g->border_picker, which_colorpicker == DT_BORDERS_BORDER);
-  darktable.gui->reset = reset;
-}
-
 static void aspect_changed(GtkWidget *combo, dt_iop_module_t *self)
 {
   dt_iop_borders_gui_data_t *g = (dt_iop_borders_gui_data_t *)self->gui_data;
@@ -689,34 +670,14 @@ static void aspect_changed(GtkWidget *combo, dt_iop_module_t *self)
   {
     g_strlcpy(p->aspect_text, text, sizeof(p->aspect_text));
     p->aspect = g->aspect_ratios[which];
-    g->combo_changed = 1;
+    ++darktable.gui->reset;
     dt_bauhaus_slider_set(g->aspect_slider,p->aspect);
+    --darktable.gui->reset;
   }
   dt_iop_color_picker_reset(self, TRUE);
   dt_dev_add_history_item(darktable.develop, self, TRUE);
 }
 
-static void aspect_slider_callback(GtkWidget *slider, dt_iop_module_t *self)
-{
-    if(self->dt->gui->reset) return;
-    dt_iop_borders_params_t *p = (dt_iop_borders_params_t *)self->params;
-    dt_iop_borders_gui_data_t *g = (dt_iop_borders_gui_data_t *)self->gui_data;
-    if(g->combo_changed){g->combo_changed=0;return;}
-    p->aspect = dt_bauhaus_slider_get(slider);
-    dt_bauhaus_combobox_set(g->aspect, DT_IOP_BORDERS_ASPECT_COUNT);
-    dt_iop_color_picker_reset(self, TRUE);
-    dt_dev_add_history_item(darktable.develop, self, TRUE);
-}
-
-static void aspect_orient_changed(GtkWidget *widget, dt_iop_module_t *self)
-{
-  if(darktable.gui->reset) return;
-  dt_iop_borders_params_t *p = (dt_iop_borders_params_t *)self->params;
-  p->aspect_orient = dt_bauhaus_combobox_get(widget);
-  dt_iop_color_picker_reset(self, TRUE);
-  dt_dev_add_history_item(darktable.develop, self, TRUE);
-}
-
 static void position_h_changed(GtkWidget *combo, dt_iop_module_t *self)
 {
   dt_iop_borders_gui_data_t *g = (dt_iop_borders_gui_data_t *)self->gui_data;
@@ -731,25 +692,14 @@ static void position_h_changed(GtkWidget *combo, dt_iop_module_t *self)
   {
     g_strlcpy(p->pos_h_text, text, sizeof(p->pos_h_text));
     p->pos_h = g->pos_h_ratios[which];
-    g->combo_changed = 1;
+    ++darktable.gui->reset;
     dt_bauhaus_slider_set(g->pos_h_slider,p->pos_h);
+    --darktable.gui->reset;
   }
   dt_iop_color_picker_reset(self, TRUE);
   dt_dev_add_history_item(darktable.develop, self, TRUE);
 }
 
-static void pos_h_slider_callback(GtkWidget *slider, dt_iop_module_t *self)
-{
-  if(self->dt->gui->reset) return;
-  dt_iop_borders_params_t *p = (dt_iop_borders_params_t *)self->params;
-  dt_iop_borders_gui_data_t *g = (dt_iop_borders_gui_data_t *)self->gui_data;
-  if(g->combo_changed){g->combo_changed=0;return;}
-  p->pos_h = dt_bauhaus_slider_get(slider);
-  dt_bauhaus_combobox_set(g->pos_h, DT_IOP_BORDERS_POSITION_H_COUNT);
-  dt_iop_color_picker_reset(self, TRUE);
-  dt_dev_add_history_item(darktable.develop, self, TRUE);
-}
-
 static void position_v_changed(GtkWidget *combo, dt_iop_module_t *self)
 {
   dt_iop_borders_gui_data_t *g = (dt_iop_borders_gui_data_t *)self->gui_data;
@@ -760,59 +710,39 @@ static void position_v_changed(GtkWidget *combo, dt_iop_module_t *self)
   {
     g_strlcpy(p->aspect_text, text, sizeof(p->aspect_text));
   }
-  else if(which < DT_IOP_BORDERS_POSITION_H_COUNT)
+  else if(which < DT_IOP_BORDERS_POSITION_V_COUNT)
   {
     g_strlcpy(p->pos_v_text, text, sizeof(p->pos_v_text));
-    p->pos_v = g->pos_h_ratios[which];
-    g->combo_changed = 1;
+    p->pos_v = g->pos_v_ratios[which];
+    ++darktable.gui->reset;
     dt_bauhaus_slider_set(g->pos_v_slider,p->pos_v);
+    --darktable.gui->reset;
   }
   dt_iop_color_picker_reset(self, TRUE);
   dt_dev_add_history_item(darktable.develop, self, TRUE);
 }
 
-static void pos_v_slider_callback(GtkWidget *slider, dt_iop_module_t *self)
+void gui_changed(dt_iop_module_t *self, GtkWidget *w, void *previous)
 {
-    if(self->dt->gui->reset) return;
-    dt_iop_borders_params_t *p = (dt_iop_borders_params_t *)self->params;
-    dt_iop_borders_gui_data_t *g = (dt_iop_borders_gui_data_t *)self->gui_data;
-    if(g->combo_changed){g->combo_changed=0;return;}
-    p->pos_v = dt_bauhaus_slider_get(slider);
+  dt_iop_borders_gui_data_t *g = (dt_iop_borders_gui_data_t *)self->gui_data;
+
+  if (w == g->aspect_slider)
+  {
+    dt_bauhaus_combobox_set(g->aspect, DT_IOP_BORDERS_ASPECT_COUNT);
+  }
+  else if(w == g->pos_h_slider)
+  {
+    dt_bauhaus_combobox_set(g->pos_h, DT_IOP_BORDERS_POSITION_H_COUNT);
+  }
+  else if(w == g->pos_v_slider)
+  {
     dt_bauhaus_combobox_set(g->pos_v, DT_IOP_BORDERS_POSITION_V_COUNT);
-    dt_iop_color_picker_reset(self, TRUE);
-    dt_dev_add_history_item(darktable.develop, self, TRUE);
-}
-
-static void size_callback(GtkWidget *slider, dt_iop_module_t *self)
-{
-  if(self->dt->gui->reset) return;
-  dt_iop_borders_params_t *p = (dt_iop_borders_params_t *)self->params;
-  p->size = dt_bauhaus_slider_get(slider) / 100.0f;
-  dt_iop_color_picker_reset(self, TRUE);
-  dt_dev_add_history_item(darktable.develop, self, TRUE);
-}
-
-static void frame_size_callback(GtkWidget *slider, dt_iop_module_t *self)
-{
-  if(self->dt->gui->reset) return;
-  dt_iop_borders_params_t *p = (dt_iop_borders_params_t *)self->params;
-  p->frame_size = dt_bauhaus_slider_get(slider) / 100.0f;
-  dt_iop_color_picker_reset(self, TRUE);
-  dt_dev_add_history_item(darktable.develop, self, TRUE);
-}
-
-static void frame_offset_callback(GtkWidget *slider, dt_iop_module_t *self)
-{
-  if(self->dt->gui->reset) return;
-  dt_iop_borders_params_t *p = (dt_iop_borders_params_t *)self->params;
-  p->frame_offset = dt_bauhaus_slider_get(slider) / 100.0f;
-  dt_iop_color_picker_reset(self, TRUE);
-  dt_dev_add_history_item(darktable.develop, self, TRUE);
+  }
 }
 
 static void colorpick_color_set(GtkColorButton *widget, dt_iop_module_t *self)
 {
-  if(self->dt->gui->reset) return;
+  if(darktable.gui->reset) return;
   dt_iop_borders_params_t *p = (dt_iop_borders_params_t *)self->params;
 
   // turn off the other color picker so that this tool actually works ...
@@ -830,7 +760,7 @@ static void colorpick_color_set(GtkColorButton *widget, dt_iop_module_t *self)
 
 static void frame_colorpick_color_set(GtkColorButton *widget, dt_iop_module_t *self)
 {
-  if(self->dt->gui->reset) return;
+  if(darktable.gui->reset) return;
   dt_iop_borders_params_t *p = (dt_iop_borders_params_t *)self->params;
 
   // turn off the other color picker so that this tool actually works ...
@@ -849,7 +779,7 @@ void gui_update(struct dt_iop_module_t *self)
 {
   dt_iop_borders_gui_data_t *g = (dt_iop_borders_gui_data_t *)self->gui_data;
   dt_iop_borders_params_t *p = (dt_iop_borders_params_t *)self->params;
-  dt_bauhaus_slider_set(g->size, p->size * 100.0f);
+  dt_bauhaus_slider_set(g->size, p->size);
 
   // ----- Aspect
   int k = 0;
@@ -899,9 +829,8 @@ void gui_update(struct dt_iop_module_t *self)
   dt_bauhaus_slider_set(g->aspect_slider, p->aspect);
   dt_bauhaus_slider_set(g->pos_h_slider, p->pos_h);
   dt_bauhaus_slider_set(g->pos_v_slider, p->pos_v);
-  dt_bauhaus_slider_set(g->frame_size, p->frame_size * 100.0f);
-  dt_bauhaus_slider_set(g->frame_offset, p->frame_offset * 100.0f);
-  g->combo_changed = 0;
+  dt_bauhaus_slider_set(g->frame_size, p->frame_size);
+  dt_bauhaus_slider_set(g->frame_offset, p->frame_offset);
 
   // ----- Border Color
   GdkRGBA c = (GdkRGBA){.red = p->color[0], .green = p->color[1], .blue = p->color[2], .alpha = 1.0 };
@@ -914,22 +843,6 @@ void gui_update(struct dt_iop_module_t *self)
   gtk_color_chooser_set_rgba(GTK_COLOR_CHOOSER(g->frame_colorpick), &fc);
 }
 
-void init(dt_iop_module_t *module)
-{
-  // module->data = malloc(sizeof(dt_iop_borders_data_t));
-  module->params = calloc(1, sizeof(dt_iop_borders_params_t));
-  module->default_params = calloc(1, sizeof(dt_iop_borders_params_t));
-  module->default_enabled = 0;
-  module->params_size = sizeof(dt_iop_borders_params_t);
-  module->gui_data = NULL;
-}
-
-void cleanup(dt_iop_module_t *module)
-{
-  free(module->params);
-  module->params = NULL;
-}
-
 static void gui_init_aspect(struct dt_iop_module_t *self)
 {
   dt_iop_borders_gui_data_t *g = (dt_iop_borders_gui_data_t *)self->gui_data;
@@ -1000,15 +913,11 @@ void gui_init(struct dt_iop_module_t *self)
   dt_iop_borders_gui_data_t *g = (dt_iop_borders_gui_data_t *)self->gui_data;
   dt_iop_borders_params_t *p = (dt_iop_borders_params_t *)self->params;
 
-  self->widget = gtk_box_new(GTK_ORIENTATION_VERTICAL, DT_BAUHAUS_SPACE);
-  dt_gui_add_help_link(self->widget, dt_get_help_url(self->op));
-
-  g->size = dt_bauhaus_slider_new_with_range(self, 0.0, 50.0, 0.5, p->size * 100.0, 2);
-  dt_bauhaus_widget_set_label(g->size, NULL, _("border size"));
+  g->size = dt_bauhaus_slider_from_params(self, "size");
+  dt_bauhaus_slider_set_factor(g->size, 100);
+  dt_bauhaus_slider_set_digits(g->size, 4);
   dt_bauhaus_slider_set_format(g->size, "%.2f %%");
-  g_signal_connect(G_OBJECT(g->size), "value-changed", G_CALLBACK(size_callback), self);
   gtk_widget_set_tooltip_text(g->size, _("size of the border in percent of the full image"));
-  gtk_box_pack_start(GTK_BOX(self->widget), g->size, TRUE, TRUE, 0);
 
   g->aspect = dt_bauhaus_combobox_new(self);
   dt_bauhaus_combobox_set_editable(g->aspect, 1);
@@ -1017,22 +926,14 @@ void gui_init(struct dt_iop_module_t *self)
   gui_init_aspect(self);
   g_signal_connect(G_OBJECT(g->aspect), "value-changed", G_CALLBACK(aspect_changed), self);
   gtk_widget_set_tooltip_text(g->aspect, _("select the aspect ratio or right click and type your own (w:h)"));
-
-  g->aspect_slider = dt_bauhaus_slider_new_with_range(self, 1, 3, 0.1, p->aspect, 2);
-  dt_bauhaus_widget_set_label(g->aspect_slider, NULL, _("aspect ratio"));
-  dt_bauhaus_slider_set_format(g->aspect_slider, "%.2f");
-  g_signal_connect(G_OBJECT(g->aspect_slider), "value-changed", G_CALLBACK(aspect_slider_callback), self);
+  g->aspect_slider = dt_bauhaus_slider_from_params(self, "aspect");
   gtk_widget_set_tooltip_text(g->aspect_slider, _("set the custom aspect ratio"));
-  gtk_box_pack_start(GTK_BOX(self->widget), g->aspect_slider, TRUE, TRUE, 0);
 
-  g->aspect_orient = dt_bauhaus_combobox_new(self);
-  dt_bauhaus_widget_set_label(g->aspect_orient, NULL, _("orientation"));
+  g->aspect_orient = dt_bauhaus_combobox_from_params(self, "aspect_orient");
   dt_bauhaus_combobox_add(g->aspect_orient, _("auto"));
   dt_bauhaus_combobox_add(g->aspect_orient, _("portrait"));
   dt_bauhaus_combobox_add(g->aspect_orient, _("landscape"));
   gtk_widget_set_tooltip_text(g->aspect_orient, _("aspect ratio orientation of the image with border"));
-  g_signal_connect(G_OBJECT(g->aspect_orient), "value-changed", G_CALLBACK(aspect_orient_changed), self);
-  gtk_box_pack_start(GTK_BOX(self->widget), g->aspect_orient, TRUE, TRUE, 0);
 
   g->pos_h = dt_bauhaus_combobox_new(self);
   dt_bauhaus_combobox_set_editable(g->pos_h, 1);
@@ -1041,12 +942,9 @@ void gui_init(struct dt_iop_module_t *self)
   g_signal_connect(G_OBJECT(g->pos_h), "value-changed", G_CALLBACK(position_h_changed), self);
   gtk_widget_set_tooltip_text(g->pos_h, _("select the horizontal position ratio relative to top "
                                           "or right click and type your own (y:h)"));
-  g->pos_h_slider = dt_bauhaus_slider_new_with_range(self, 0.0, 1.0, 0.1, p->pos_h, 2);
-  dt_bauhaus_widget_set_label(g->pos_h_slider, NULL, _("horizontal position"));
-  dt_bauhaus_slider_set_format(g->pos_h_slider, "%.2f");
-  g_signal_connect(G_OBJECT(g->pos_h_slider), "value-changed", G_CALLBACK(pos_h_slider_callback), self);
+  g->pos_h_slider = dt_bauhaus_slider_from_params(self, "pos_h");
+  dt_bauhaus_slider_set_step(g->pos_h_slider, 0.1);
   gtk_widget_set_tooltip_text(g->pos_h_slider, _("custom horizontal position"));
-  gtk_box_pack_start(GTK_BOX(self->widget), g->pos_h_slider, TRUE, TRUE, 0);
 
   g->pos_v = dt_bauhaus_combobox_new(self);
   dt_bauhaus_combobox_set_editable(g->pos_v, 1);
@@ -1055,96 +953,67 @@ void gui_init(struct dt_iop_module_t *self)
   g_signal_connect(G_OBJECT(g->pos_v), "value-changed", G_CALLBACK(position_v_changed), self);
   gtk_widget_set_tooltip_text(g->pos_v, _("select the vertical position ratio relative to left "
                                           "or right click and type your own (x:w)"));
-  g->pos_v_slider = dt_bauhaus_slider_new_with_range(self, 0.0, 1.0, 0.1, p->pos_v, 2);
-  dt_bauhaus_widget_set_label(g->pos_v_slider, NULL, _("vertical position"));
-  dt_bauhaus_slider_set_format(g->pos_v_slider, "%.2f");
-  g_signal_connect(G_OBJECT(g->pos_v_slider), "value-changed", G_CALLBACK(pos_v_slider_callback), self);
+  g->pos_v_slider = dt_bauhaus_slider_from_params(self, "pos_v");
+  dt_bauhaus_slider_set_step(g->pos_v_slider, 0.1);
   gtk_widget_set_tooltip_text(g->pos_v_slider, _("custom vertical position"));
-  gtk_box_pack_start(GTK_BOX(self->widget), g->pos_v_slider, TRUE, TRUE, 0);
 
   gui_init_positions(self);
 
-  g->frame_size = dt_bauhaus_slider_new_with_range(self, 0.0, 100.0, 0.5, p->frame_size * 100.0, 2);
-  dt_bauhaus_widget_set_label(g->frame_size, NULL, _("frame line size"));
+  g->frame_size = dt_bauhaus_slider_from_params(self, "frame_size");
+  dt_bauhaus_slider_set_factor(g->frame_size, 100);
+  dt_bauhaus_slider_set_step(g->frame_size, 0.005);
+  dt_bauhaus_slider_set_digits(g->frame_size, 4);
   dt_bauhaus_slider_set_format(g->frame_size, "%.2f %%");
-  g_signal_connect(G_OBJECT(g->frame_size), "value-changed", G_CALLBACK(frame_size_callback), self);
   gtk_widget_set_tooltip_text(g->frame_size, _("size of the frame line in percent of min border width"));
-  gtk_box_pack_start(GTK_BOX(self->widget), g->frame_size, TRUE, TRUE, 0);
 
-  g->frame_offset = dt_bauhaus_slider_new_with_range(self, 0.0, 100.0, 0.5, p->frame_offset * 100.0, 2);
-  dt_bauhaus_widget_set_label(g->frame_offset, NULL, _("frame line offset"));
+  g->frame_offset = dt_bauhaus_slider_from_params(self, "frame_offset");
+  dt_bauhaus_slider_set_factor(g->frame_offset, 100);
+  dt_bauhaus_slider_set_step(g->frame_size, 0.005);
+  dt_bauhaus_slider_set_digits(g->frame_offset, 4);
   dt_bauhaus_slider_set_format(g->frame_offset, "%.2f %%");
-  g_signal_connect(G_OBJECT(g->frame_offset), "value-changed", G_CALLBACK(frame_offset_callback), self);
   gtk_widget_set_tooltip_text(g->frame_offset, _("offset of the frame line beginning on picture side"));
-  gtk_box_pack_start(GTK_BOX(self->widget), g->frame_offset, TRUE, TRUE, 0);
 
   GdkRGBA color = (GdkRGBA){.red = p->color[0], .green = p->color[1], .blue = p->color[2], .alpha = 1.0 };
 
-  GtkWidget *box = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
+  GtkWidget *label, *box;
+
+  box = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
+  label = dtgtk_reset_label_new(_("border color"), self, &p->color, 3 * sizeof(float));
+  gtk_box_pack_start(GTK_BOX(box), label, TRUE, TRUE, 0);
   g->colorpick = gtk_color_button_new_with_rgba(&color);
   gtk_color_chooser_set_use_alpha(GTK_COLOR_CHOOSER(g->colorpick), FALSE);
   gtk_color_button_set_title(GTK_COLOR_BUTTON(g->colorpick), _("select border color"));
-  GtkWidget *label = dtgtk_reset_label_new(_("border color"), self, &p->color, 3 * sizeof(float));
   g_signal_connect(G_OBJECT(g->colorpick), "color-set", G_CALLBACK(colorpick_color_set), self);
-
-  g->border_picker = GTK_TOGGLE_BUTTON(dtgtk_togglebutton_new(dtgtk_cairo_paint_colorpicker, CPF_STYLE_FLAT, NULL));
-  gtk_widget_set_tooltip_text(GTK_WIDGET(g->border_picker), _("pick border color from image"));
-  g_signal_connect(G_OBJECT(g->border_picker), "toggled", G_CALLBACK(dt_iop_color_picker_callback), &g->color_picker);
-
-  gtk_box_pack_start(GTK_BOX(box), label, TRUE, TRUE, 0);
   gtk_box_pack_start(GTK_BOX(box), GTK_WIDGET(g->colorpick), FALSE, TRUE, 0);
-  gtk_box_pack_start(GTK_BOX(box), GTK_WIDGET(g->border_picker), FALSE, FALSE, 0);
+  g->border_picker = dt_color_picker_new(self, DT_COLOR_PICKER_POINT, box);
+  gtk_widget_set_tooltip_text(GTK_WIDGET(g->border_picker), _("pick border color from image"));
   gtk_box_pack_start(GTK_BOX(self->widget), box, TRUE, TRUE, 0);
 
   box = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
+  label = dtgtk_reset_label_new(_("frame line color"), self, &p->color, 3 * sizeof(float));
+  gtk_box_pack_start(GTK_BOX(box), label, TRUE, TRUE, 0);
   g->frame_colorpick = gtk_color_button_new_with_rgba(&color);
   gtk_color_chooser_set_use_alpha(GTK_COLOR_CHOOSER(g->frame_colorpick), FALSE);
   gtk_color_button_set_title(GTK_COLOR_BUTTON(g->frame_colorpick), _("select frame line color"));
-  label = dtgtk_reset_label_new(_("frame line color"), self, &p->color, 3 * sizeof(float));
   g_signal_connect(G_OBJECT(g->frame_colorpick), "color-set", G_CALLBACK(frame_colorpick_color_set), self);
-
-  g->frame_picker = GTK_TOGGLE_BUTTON(dtgtk_togglebutton_new(dtgtk_cairo_paint_colorpicker, CPF_STYLE_FLAT, NULL));
-  gtk_widget_set_tooltip_text(GTK_WIDGET(g->frame_picker), _("pick frame line color from image"));
-  g_signal_connect(G_OBJECT(g->frame_picker), "toggled", G_CALLBACK(dt_iop_color_picker_callback), &g->color_picker);
-
-  gtk_box_pack_start(GTK_BOX(box), label, TRUE, TRUE, 0);
   gtk_box_pack_start(GTK_BOX(box), GTK_WIDGET(g->frame_colorpick), FALSE, TRUE, 0);
-  gtk_box_pack_start(GTK_BOX(box), GTK_WIDGET(g->frame_picker), FALSE, FALSE, 0);
+  g->frame_picker = dt_color_picker_new(self, DT_COLOR_PICKER_POINT, box);
+  gtk_widget_set_tooltip_text(GTK_WIDGET(g->frame_picker), _("pick frame line color from image"));
   gtk_box_pack_start(GTK_BOX(self->widget), box, TRUE, TRUE, 0);
-
-  dt_iop_init_picker(&g->color_picker,
-              self,
-              DT_COLOR_PICKER_POINT,
-              _iop_color_picker_get_set,
-              _iop_color_picker_apply,
-              _iop_color_picker_update);
 }
 
 
-void reload_defaults(dt_iop_module_t *self)
+void init(dt_iop_module_t *self)
 {
-  dt_iop_borders_params_t tmp = (dt_iop_borders_params_t){ { 1.0f, 1.0f, 1.0f },
-                                                           DT_IOP_BORDERS_ASPECT_CONSTANT_VALUE,
-                                                           "constant border",
-                                                           0,
-                                                           0.1f,
-                                                           0.5f,
-                                                           "1/2",
-                                                           0.5f,
-                                                           "1/2",
-                                                           0.0f,
-                                                           0.5f,
-                                                           { 0.0f, 0.0f, 0.0f },
-                                                           TRUE };
-  memcpy(self->params, &tmp, sizeof(dt_iop_borders_params_t));
-  memcpy(self->default_params, &tmp, sizeof(dt_iop_borders_params_t));
-  self->default_enabled = 0;
-}
+  dt_iop_default_init(self);
 
-void gui_cleanup(struct dt_iop_module_t *self)
-{
-  free(self->gui_data);
-  self->gui_data = NULL;
+  dt_iop_borders_params_t *defaults = self->default_params;
+
+  g_strlcpy(defaults->aspect_text, "constant border", sizeof(defaults->aspect_text));
+  g_strlcpy(defaults->pos_h_text, "1/2", sizeof(defaults->pos_h_text));
+  g_strlcpy(defaults->pos_v_text, "1/2", sizeof(defaults->pos_v_text));
+
+  memcpy(self->params, self->default_params, sizeof(dt_iop_borders_params_t));
 }
 
 // modelines: These editor modelines have been set for all relevant files by tools/update_modelines.sh
