@@ -1,6 +1,6 @@
 /*
     This file is part of darktable,
-    copyright (c) 2009--2011 johannes hanika.
+    Copyright (C) 2010-2020 darktable developers.
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -39,7 +39,7 @@ DT_MODULE_INTROSPECTION(1, dt_iop_cacorrect_params_t)
 
 typedef struct dt_iop_cacorrect_params_t
 {
-  int keep;
+  int keep; // $DEFAULT: 50
 } dt_iop_cacorrect_params_t;
 
 typedef struct dt_iop_cacorrect_gui_data_t
@@ -426,7 +426,7 @@ static void CA_correct(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *pie
     {
 // Main algorithm: Tile loop calculating correction parameters per tile
 #ifdef _OPENMP
-#pragma omp for collapse(2) schedule(dynamic) nowait
+#pragma omp for collapse(2) schedule(static) nowait
 #endif
       for(int top = -border; top < height; top += ts - border2)
         for(int left = -border; left < width; left += ts - border2)
@@ -471,7 +471,7 @@ static void CA_correct(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *pie
 
           if(rrmax < rr1)
           {
-            for(int rr = 0; rr < border; rr++)
+            for(int rr = 0; rr < MIN(border, rr1 - rrmax); rr++)
               for(int cc = ccmin; cc < ccmax; cc++)
               {
                 int c = FC(rr, cc, filters);
@@ -492,7 +492,7 @@ static void CA_correct(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *pie
           if(ccmax < cc1)
           {
             for(int rr = rrmin; rr < rrmax; rr++)
-              for(int cc = 0; cc < border; cc++)
+              for(int cc = 0; cc < MIN(border, cc1 - ccmax); cc++)
               {
                 int c = FC(rr, cc, filters);
                 rgb[c][rr * ts + ccmax + cc] = (in[(top + rr) * width + (width - cc - 2)]);
@@ -512,8 +512,8 @@ static void CA_correct(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *pie
 
           if(rrmax < rr1 && ccmax < cc1)
           {
-            for(int rr = 0; rr < border; rr++)
-              for(int cc = 0; cc < border; cc++)
+            for(int rr = 0; rr < MIN(border, rr1 - rrmax); rr++)
+              for(int cc = 0; cc < MIN(border, cc1 - ccmax); cc++)
               {
                 int c = FC(rr, cc, filters);
                 rgb[c][(rrmax + rr) * ts + ccmax + cc] = (in[(height - rr - 2) * width + (width - cc - 2)]);
@@ -523,7 +523,7 @@ static void CA_correct(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *pie
           if(rrmin > 0 && ccmax < cc1)
           {
             for(int rr = 0; rr < border; rr++)
-              for(int cc = 0; cc < border; cc++)
+              for(int cc = 0; cc < MIN(border, cc1 - ccmax); cc++)
               {
                 int c = FC(rr, cc, filters);
                 rgb[c][(rr)*ts + ccmax + cc] = (in[(border2 - rr) * width + (width - cc - 2)]);
@@ -532,7 +532,7 @@ static void CA_correct(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *pie
 
           if(rrmax < rr1 && ccmin > 0)
           {
-            for(int rr = 0; rr < border; rr++)
+            for(int rr = 0; rr < MIN(border, rr1 - rrmax); rr++)
               for(int cc = 0; cc < border; cc++)
               {
                 int c = FC(rr, cc, filters);
@@ -1074,7 +1074,7 @@ static void CA_correct(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *pie
     if(processpasstwo)
     {
 #ifdef _OPENMP
-#pragma omp for schedule(dynamic) collapse(2) nowait
+#pragma omp for schedule(static) collapse(2) nowait
 #endif
 
       for(int top = -border; top < height; top += ts - border2)
@@ -1128,7 +1128,7 @@ static void CA_correct(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *pie
 
           if(rrmax < rr1)
           {
-            for(int rr = 0; rr < border; rr++)
+            for(int rr = 0; rr < MIN(border, rr1 - rrmax); rr++)
               for(int cc = ccmin; cc < ccmax; cc++)
               {
                 int c = FC(rr, cc, filters);
@@ -1151,7 +1151,7 @@ static void CA_correct(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *pie
           if(ccmax < cc1)
           {
             for(int rr = rrmin; rr < rrmax; rr++)
-              for(int cc = 0; cc < border; cc++)
+              for(int cc = 0; cc < MIN(border, cc1 - ccmax); cc++)
               {
                 int c = FC(rr, cc, filters);
                 rgb[c][rr * ts + ccmax + cc] = (in[(top + rr) * width + (width - cc - 2)]);
@@ -1173,8 +1173,8 @@ static void CA_correct(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *pie
 
           if(rrmax < rr1 && ccmax < cc1)
           {
-            for(int rr = 0; rr < border; rr++)
-              for(int cc = 0; cc < border; cc++)
+            for(int rr = 0; rr < MIN(border, rr1 - rrmax); rr++)
+              for(int cc = 0; cc < MIN(border, cc1 - ccmax); cc++)
               {
                 int c = FC(rr, cc, filters);
                 rgb[c][(rrmax + rr) * ts + ccmax + cc] = (in[(height - rr - 2) * width + (width - cc - 2)]);
@@ -1185,7 +1185,7 @@ static void CA_correct(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *pie
           if(rrmin > 0 && ccmax < cc1)
           {
             for(int rr = 0; rr < border; rr++)
-              for(int cc = 0; cc < border; cc++)
+              for(int cc = 0; cc < MIN(border, cc1 - ccmax); cc++)
               {
                 int c = FC(rr, cc, filters);
                 rgb[c][(rr)*ts + ccmax + cc] = (in[(border2 - rr) * width + (width - cc - 2)]);
@@ -1195,7 +1195,7 @@ static void CA_correct(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *pie
 
           if(rrmax < rr1 && ccmin > 0)
           {
-            for(int rr = 0; rr < border; rr++)
+            for(int rr = 0; rr < MIN(border, rr1 - rrmax); rr++)
               for(int cc = 0; cc < border; cc++)
               {
                 int c = FC(rr, cc, filters);
@@ -1495,11 +1495,8 @@ void process(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, const
 
 void reload_defaults(dt_iop_module_t *module)
 {
-  // init defaults:
-  dt_iop_cacorrect_params_t tmp = (dt_iop_cacorrect_params_t){.keep = 50 };
-
   // we might be called from presets update infrastructure => there is no image
-  if(!module->dev) goto end;
+  if(!module->dev) return;
 
   dt_image_t *img = &module->dev->image_storage;
   // can't be switched on for non-raw or x-trans images:
@@ -1507,37 +1504,6 @@ void reload_defaults(dt_iop_module_t *module)
     module->hide_enable_button = 0;
   else
     module->hide_enable_button = 1;
-  module->default_enabled = 0;
-
-end:
-  memcpy(module->params, &tmp, sizeof(dt_iop_cacorrect_params_t));
-  memcpy(module->default_params, &tmp, sizeof(dt_iop_cacorrect_params_t));
-}
-
-/** init, cleanup, commit to pipeline */
-void init(dt_iop_module_t *module)
-{
-  // we don't need global data:
-  module->global_data = NULL; // malloc(sizeof(dt_iop_cacorrect_global_data_t));
-  module->params = calloc(1, sizeof(dt_iop_cacorrect_params_t));
-  module->default_params = calloc(1, sizeof(dt_iop_cacorrect_params_t));
-  // our module is disabled by default
-  // by default:
-  module->default_enabled = 0;
-
-  // we come just before demosaicing.
-  module->params_size = sizeof(dt_iop_cacorrect_params_t);
-  module->gui_data = NULL;
-}
-
-void cleanup(dt_iop_module_t *module)
-{
-  free(module->params);
-  module->params = NULL;
-  free(module->default_params);
-  module->default_params = NULL;
-  free(module->global_data); // just to be sure
-  module->global_data = NULL;
 }
 
 /** commit is the synch point between core and gui, so it copies params to pipe data. */
@@ -1576,7 +1542,6 @@ void gui_init(dt_iop_module_t *self)
   self->widget = gtk_label_new("");
   gtk_widget_set_halign(self->widget, GTK_ALIGN_START);
   self->gui_data = &dummy;
-  dt_gui_add_help_link(self->widget, dt_get_help_url(self->op));
 }
 
 void gui_cleanup(dt_iop_module_t *self)
