@@ -1,6 +1,6 @@
 /*
   This file is part of darktable,
-  copyright (c) 2010-2012 Henrik Andersson.
+  Copyright (C) 2010-2020 darktable developers.
 
   darktable is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -78,12 +78,12 @@ typedef enum _channelmixer_output_t
 typedef struct dt_iop_channelmixer_params_t
 {
   //_channelmixer_output_t output_channel;
-  /** amount of red to mix value -1.0 - 1.0 */
-  float red[CHANNEL_SIZE];
-  /** amount of green to mix value -1.0 - 1.0 */
-  float green[CHANNEL_SIZE];
-  /** amount of blue to mix value -1.0 - 1.0 */
-  float blue[CHANNEL_SIZE];
+  /** amount of red to mix value */
+  float red[CHANNEL_SIZE];   // $MIN: -1.0 $MAX: 1.0 
+  /** amount of green to mix value */
+  float green[CHANNEL_SIZE]; // $MIN: -1.0 $MAX: 1.0 
+  /** amount of blue to mix value */
+  float blue[CHANNEL_SIZE];  // $MIN: -1.0 $MAX: 1.0 
 } dt_iop_channelmixer_params_t;
 
 typedef struct dt_iop_channelmixer_gui_data_t
@@ -132,6 +132,7 @@ void init_key_accels(dt_iop_module_so_t *self)
   dt_accel_register_slider_iop(self, FALSE, NC_("accel", "red"));
   dt_accel_register_slider_iop(self, FALSE, NC_("accel", "green"));
   dt_accel_register_slider_iop(self, FALSE, NC_("accel", "blue"));
+  dt_accel_register_combobox_iop(self, FALSE, NC_("accel", "destination"));
 }
 
 void connect_key_accels(dt_iop_module_t *self)
@@ -142,6 +143,7 @@ void connect_key_accels(dt_iop_module_t *self)
   dt_accel_connect_slider_iop(self, "red", GTK_WIDGET(g->scale_red));
   dt_accel_connect_slider_iop(self, "green", GTK_WIDGET(g->scale_green));
   dt_accel_connect_slider_iop(self, "blue", GTK_WIDGET(g->scale_blue));
+  dt_accel_connect_combobox_iop(self, "destination", GTK_WIDGET(g->output_channel));
 }
 
 void process(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, const void *const ivoid,
@@ -313,7 +315,7 @@ void cleanup_global(dt_iop_module_so_t *module)
 static void red_callback(GtkWidget *slider, gpointer user_data)
 {
   dt_iop_module_t *self = (dt_iop_module_t *)user_data;
-  if(self->dt->gui->reset) return;
+  if(darktable.gui->reset) return;
   dt_iop_channelmixer_params_t *p = (dt_iop_channelmixer_params_t *)self->params;
   dt_iop_channelmixer_gui_data_t *g = (dt_iop_channelmixer_gui_data_t *)self->gui_data;
   const int output_channel_index = dt_bauhaus_combobox_get(g->output_channel);
@@ -327,7 +329,7 @@ static void red_callback(GtkWidget *slider, gpointer user_data)
 static void green_callback(GtkWidget *slider, gpointer user_data)
 {
   dt_iop_module_t *self = (dt_iop_module_t *)user_data;
-  if(self->dt->gui->reset) return;
+  if(darktable.gui->reset) return;
   dt_iop_channelmixer_params_t *p = (dt_iop_channelmixer_params_t *)self->params;
   dt_iop_channelmixer_gui_data_t *g = (dt_iop_channelmixer_gui_data_t *)self->gui_data;
   const int output_channel_index = dt_bauhaus_combobox_get(g->output_channel);
@@ -342,7 +344,7 @@ static void green_callback(GtkWidget *slider, gpointer user_data)
 static void blue_callback(GtkWidget *slider, gpointer user_data)
 {
   dt_iop_module_t *self = (dt_iop_module_t *)user_data;
-  if(self->dt->gui->reset) return;
+  if(darktable.gui->reset) return;
   dt_iop_channelmixer_params_t *p = (dt_iop_channelmixer_params_t *)self->params;
   dt_iop_channelmixer_gui_data_t *g = (dt_iop_channelmixer_gui_data_t *)self->gui_data;
   const int output_channel_index = dt_bauhaus_combobox_get(g->output_channel);
@@ -356,7 +358,7 @@ static void blue_callback(GtkWidget *slider, gpointer user_data)
 static void output_callback(GtkComboBox *combo, gpointer user_data)
 {
   dt_iop_module_t *self = (dt_iop_module_t *)user_data;
-  if(self->dt->gui->reset) return;
+  if(darktable.gui->reset) return;
   dt_iop_channelmixer_params_t *p = (dt_iop_channelmixer_params_t *)self->params;
   dt_iop_channelmixer_gui_data_t *g = (dt_iop_channelmixer_gui_data_t *)self->gui_data;
 
@@ -418,24 +420,13 @@ void gui_update(struct dt_iop_module_t *self)
 
 void init(dt_iop_module_t *module)
 {
-  module->params = calloc(1, sizeof(dt_iop_channelmixer_params_t));
-  module->default_params = calloc(1, sizeof(dt_iop_channelmixer_params_t));
-  module->default_enabled = 0;
-  module->params_size = sizeof(dt_iop_channelmixer_params_t);
-  module->gui_data = NULL;
-  dt_iop_channelmixer_params_t tmp = (dt_iop_channelmixer_params_t){ { 0, 0, 0, 1, 0, 0, 0 },
-                                                                     { 0, 0, 0, 0, 1, 0, 0 },
-                                                                     { 0, 0, 0, 0, 0, 1, 0 } };
-  memcpy(module->params, &tmp, sizeof(dt_iop_channelmixer_params_t));
-  memcpy(module->default_params, &tmp, sizeof(dt_iop_channelmixer_params_t));
-}
+  dt_iop_default_init(module);
 
-void cleanup(dt_iop_module_t *module)
-{
-  free(module->params);
-  module->params = NULL;
-  free(module->default_params);
-  module->default_params = NULL;
+  dt_iop_channelmixer_params_t *d = module->default_params;
+
+  d->red[CHANNEL_RED] = d->green[CHANNEL_GREEN] = d->blue[CHANNEL_BLUE] = 1.0;
+
+  memcpy(module->params, module->default_params, sizeof(dt_iop_channelmixer_params_t));
 }
 
 void gui_init(struct dt_iop_module_t *self)
@@ -445,7 +436,6 @@ void gui_init(struct dt_iop_module_t *self)
   dt_iop_channelmixer_params_t *p = (dt_iop_channelmixer_params_t *)self->params;
 
   self->widget = gtk_box_new(GTK_ORIENTATION_VERTICAL, DT_BAUHAUS_SPACE);
-  dt_gui_add_help_link(self->widget, dt_get_help_url(self->op));
 
   /* output */
   g->output_channel = dt_bauhaus_combobox_new(self);
@@ -589,12 +579,6 @@ void init_presets(dt_iop_module_so_t *self)
   DT_DEBUG_SQLITE3_EXEC(dt_database_get(darktable.db), "COMMIT", NULL, NULL, NULL);
 }
 
-void gui_cleanup(struct dt_iop_module_t *self)
-{
-  free(self->gui_data);
-  self->gui_data = NULL;
-}
-
 // modelines: These editor modelines have been set for all relevant files by tools/update_modelines.sh
 // vim: shiftwidth=2 expandtab tabstop=2 cindent
 // kate: tab-indents: off; indent-width 2; replace-tabs on; indent-mode cstyle; remove-trailing-spaces modified;
