@@ -1,10 +1,7 @@
 /* -*- Mode: c; c-basic-offset: 2; -*- */
 /*
     This file is part of darktable,
-    copyright (c) 2009--2010 johannes hanika
-    copyright (c) 2011 Sergey Astanin
-    copyright (c) 2012 Henrik Andersson
-    copyright (c) 2014 Ulrich Pegelow
+    Copyright (C) 2011-2020 darktable developers.
 
 
     darktable is free software: you can redistribute it and/or modify
@@ -27,6 +24,7 @@
 #include "common/opencl.h"
 #include "control/control.h"
 #include "develop/imageop.h"
+#include "develop/imageop_gui.h"
 #include "gui/accelerators.h"
 #include "gui/gtk.h"
 #include "iop/iop_api.h"
@@ -51,11 +49,11 @@ typedef struct dt_iop_colorcontrast_params1_t
 
 typedef struct dt_iop_colorcontrast_params_t
 {
-  float a_steepness;
+  float a_steepness; // $MIN: 0.0 $MAX: 5.0 $DEFAULT: 1.0 $DESCRIPTION: "green vs magenta"
   float a_offset;
-  float b_steepness;
+  float b_steepness; // $MIN: 0.0 $MAX: 5.0 $DEFAULT: 1.0 $DESCRIPTION: "blue vs yellow"
   float b_offset;
-  int unbound;
+  int unbound;       // $DEFAULT: 1
 } dt_iop_colorcontrast_params_t;
 
 typedef struct dt_iop_colorcontrast_gui_data_t
@@ -299,40 +297,6 @@ void cleanup_global(dt_iop_module_so_t *module)
 }
 
 
-void reload_defaults(dt_iop_module_t *module)
-{
-  // change default_enabled depending on type of image, or set new default_params even.
-  // if this callback exists, it has to write default_params and default_enabled.
-  dt_iop_colorcontrast_params_t tmp = (dt_iop_colorcontrast_params_t){ 1.0, 0.0, 1.0, 0.0, 1 };
-  memcpy(module->params, &tmp, sizeof(dt_iop_colorcontrast_params_t));
-  memcpy(module->default_params, &tmp, sizeof(dt_iop_colorcontrast_params_t));
-  module->default_enabled = 0;
-}
-
-/** init, cleanup, commit to pipeline */
-void init(dt_iop_module_t *module)
-{
-  module->params = calloc(1, sizeof(dt_iop_colorcontrast_params_t));
-  module->default_params = calloc(1, sizeof(dt_iop_colorcontrast_params_t));
-  // our module is disabled by default
-  module->default_enabled = 0;
-  // we are pretty late in the pipe:
-  module->params_size = sizeof(dt_iop_colorcontrast_params_t);
-  module->gui_data = NULL;
-  // init defaults:
-  dt_iop_colorcontrast_params_t tmp = (dt_iop_colorcontrast_params_t){ 1.0, 0.0, 1.0, 0.0, 1 };
-  memcpy(module->params, &tmp, sizeof(dt_iop_colorcontrast_params_t));
-  memcpy(module->default_params, &tmp, sizeof(dt_iop_colorcontrast_params_t));
-}
-
-void cleanup(dt_iop_module_t *module)
-{
-  free(module->params);
-  module->params = NULL;
-  free(module->default_params);
-  module->default_params = NULL;
-}
-
 /** commit is the synch point between core and gui, so it copies params to pipe data. */
 void commit_params(struct dt_iop_module_t *self, dt_iop_params_t *params, dt_dev_pixelpipe_t *pipe,
                    dt_dev_pixelpipe_iop_t *piece)
@@ -358,33 +322,8 @@ void cleanup_pipe(struct dt_iop_module_t *self, dt_dev_pixelpipe_t *pipe, dt_dev
   piece->data = NULL;
 }
 
-/** put your local callbacks here, be sure to make them static so they won't be visible outside this file! */
-static void a_slider_callback(GtkRange *range, dt_iop_module_t *self)
-{
-  // this is important to avoid cycles!
-  if(darktable.gui->reset) return;
-  dt_iop_colorcontrast_gui_data_t *g = (dt_iop_colorcontrast_gui_data_t *)self->gui_data;
-  dt_iop_colorcontrast_params_t *p = (dt_iop_colorcontrast_params_t *)self->params;
-  p->a_steepness = dt_bauhaus_slider_get(g->a_scale);
-  // let core know of the changes
-  dt_dev_add_history_item(darktable.develop, self, TRUE);
-}
-
-static void b_slider_callback(GtkRange *range, dt_iop_module_t *self)
-{
-  // this is important to avoid cycles!
-  if(darktable.gui->reset) return;
-  dt_iop_colorcontrast_gui_data_t *g = (dt_iop_colorcontrast_gui_data_t *)self->gui_data;
-  dt_iop_colorcontrast_params_t *p = (dt_iop_colorcontrast_params_t *)self->params;
-  p->b_steepness = dt_bauhaus_slider_get(g->b_scale);
-  // let core know of the changes
-  dt_dev_add_history_item(darktable.develop, self, TRUE);
-}
-
-/** gui callbacks, these are needed. */
 void gui_update(dt_iop_module_t *self)
 {
-  // let gui slider match current parameters:
   dt_iop_colorcontrast_gui_data_t *g = (dt_iop_colorcontrast_gui_data_t *)self->gui_data;
   dt_iop_colorcontrast_params_t *p = (dt_iop_colorcontrast_params_t *)self->params;
   dt_bauhaus_slider_set(g->a_scale, p->a_steepness);
@@ -393,38 +332,16 @@ void gui_update(dt_iop_module_t *self)
 
 void gui_init(dt_iop_module_t *self)
 {
-  // init the slider (more sophisticated layouts are possible with gtk tables and boxes):
   self->gui_data = malloc(sizeof(dt_iop_colorcontrast_gui_data_t));
   dt_iop_colorcontrast_gui_data_t *g = (dt_iop_colorcontrast_gui_data_t *)self->gui_data;
-  dt_iop_colorcontrast_params_t *p = (dt_iop_colorcontrast_params_t *)self->params;
 
-  self->widget = gtk_box_new(GTK_ORIENTATION_VERTICAL, DT_BAUHAUS_SPACE);
-  dt_gui_add_help_link(self->widget, dt_get_help_url(self->op));
-
-  /* a scale */
-  g->a_scale = dt_bauhaus_slider_new_with_range(self, 0.0, 5.0, 0.01, p->a_steepness, 2);
-  dt_bauhaus_widget_set_label(g->a_scale, NULL, _("green vs magenta"));
-  gtk_box_pack_start(GTK_BOX(self->widget), GTK_WIDGET(g->a_scale), TRUE, TRUE, 0);
+  g->a_scale = dt_bauhaus_slider_from_params(self, "a_steepness");
   gtk_widget_set_tooltip_text(g->a_scale, _("steepness of the a* curve in Lab"));
-  g_signal_connect(G_OBJECT(g->a_scale), "value-changed", G_CALLBACK(a_slider_callback), self);
 
-
-  /* b scale */
-  g->b_scale = dt_bauhaus_slider_new_with_range(self, 0.0, 5.0, 0.01, p->b_steepness, 2);
-  dt_bauhaus_widget_set_label(g->b_scale, NULL, _("blue vs yellow"));
-  gtk_box_pack_start(GTK_BOX(self->widget), GTK_WIDGET(g->b_scale), TRUE, TRUE, 0);
+  g->b_scale = dt_bauhaus_slider_from_params(self, "b_steepness");
   gtk_widget_set_tooltip_text(g->b_scale, _("steepness of the b* curve in Lab"));
-  g_signal_connect(G_OBJECT(g->b_scale), "value-changed", G_CALLBACK(b_slider_callback), self);
 }
 
-void gui_cleanup(dt_iop_module_t *self)
-{
-  // nothing else necessary, gtk will clean up the sliders.
-  free(self->gui_data);
-  self->gui_data = NULL;
-}
-
-
 // modelines: These editor modelines have been set for all relevant files by tools/update_modelines.sh
 // vim: shiftwidth=2 expandtab tabstop=2 cindent
 // kate: tab-indents: off; indent-width 2; replace-tabs on; indent-mode cstyle; remove-trailing-spaces modified;
