@@ -1,7 +1,6 @@
 /*
     This file is part of darktable,
-    copyright (c) 2009--2010 johannes hanika.
-    copyright (c) 2014-2016 Roman Lebedev.
+    Copyright (C) 2009-2020 darktable developers.
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -38,6 +37,7 @@
 #include "develop/develop.h"
 #include "develop/imageop.h"
 #include "develop/imageop_math.h"
+#include "develop/imageop_gui.h"
 #include "develop/pixelpipe.h"
 #include "dtgtk/paint.h"
 #include "dtgtk/resetlabel.h"
@@ -50,12 +50,12 @@
 #define exposure2white(x) exp2f(-(x))
 #define white2exposure(x) -dt_log2f(fmaxf(1e-20f, x))
 
-DT_MODULE_INTROSPECTION(5, dt_iop_exposure_params_t)
+DT_MODULE_INTROSPECTION(6, dt_iop_exposure_params_t)
 
 typedef enum dt_iop_exposure_mode_t
 {
-  EXPOSURE_MODE_MANUAL,
-  EXPOSURE_MODE_DEFLICKER
+  EXPOSURE_MODE_MANUAL,   // $DESCRIPTION: "manual"
+  EXPOSURE_MODE_DEFLICKER // $DESCRIPTION: "automatic"
 } dt_iop_exposure_mode_t;
 
 // uint16_t pixel can have any value in range [0, 65535], thus, there is
@@ -64,18 +64,19 @@ typedef enum dt_iop_exposure_mode_t
 
 typedef struct dt_iop_exposure_params_t
 {
-  dt_iop_exposure_mode_t mode;
-  float black;
-  float exposure;
-  float deflicker_percentile, deflicker_target_level;
+  dt_iop_exposure_mode_t mode; // $DEFAULT: EXPOSURE_MODE_MANUAL
+  float black;    // $MIN: -1.0 $MAX: 1.0 $DEFAULT: 0.0 $DESCRIPTION: "black level correction"
+  float exposure; // $MIN: -18.0 $MAX: 18.0 $DEFAULT: 0.0
+  float deflicker_percentile;   // $MIN: 0.0 $MAX: 100.0 $DEFAULT: 50.0 $DESCRIPTION: "percentile"
+  float deflicker_target_level; // $MIN: -18.0 $MAX: 18.0 $DEFAULT: -4.0 $DESCRIPTION: "target level"
+  int compensate_exposure_bias; // $DEFAULT: FALSE
 } dt_iop_exposure_params_t;
 
 typedef struct dt_iop_exposure_gui_data_t
 {
-  GList *modes;
   GtkWidget *mode;
   GtkWidget *black;
-  GtkWidget *mode_stack;
+  GtkStack *mode_stack;
   GtkWidget *exposure;
   GtkWidget *autoexpp;
   GtkWidget *deflicker_percentile;
@@ -83,9 +84,9 @@ typedef struct dt_iop_exposure_gui_data_t
   uint32_t *deflicker_histogram; // used to cache histogram of source file
   dt_dev_histogram_stats_t deflicker_histogram_stats;
   GtkLabel *deflicker_used_EC;
+  GtkWidget *compensate_exposure_bias;
   float deflicker_computed_exposure;
   dt_pthread_mutex_t lock;
-  dt_iop_color_picker_t color_picker;
 } dt_iop_exposure_gui_data_t;
 
 typedef struct dt_iop_exposure_data_t
@@ -122,32 +123,49 @@ int default_colorspace(dt_iop_module_t *self, dt_dev_pixelpipe_t *pipe, dt_dev_p
   return iop_cs_rgb;
 }
 
+static void dt_iop_exposure_set_exposure(struct dt_iop_module_t *self, const float exposure);
+static float dt_iop_exposure_get_exposure(struct dt_iop_module_t *self);
+static void dt_iop_exposure_set_black(struct dt_iop_module_t *self, const float black);
+static float dt_iop_exposure_get_black(struct dt_iop_module_t *self);
+
 void init_key_accels(dt_iop_module_so_t *self)
 {
-  dt_accel_register_slider_iop(self, FALSE, NC_("accel", "mode"));
   dt_accel_register_slider_iop(self, FALSE, NC_("accel", "black"));
   dt_accel_register_slider_iop(self, FALSE, NC_("accel", "exposure"));
   dt_accel_register_slider_iop(self, FALSE, NC_("accel", "auto-exposure"));
   dt_accel_register_slider_iop(self, FALSE, NC_("accel", "percentile"));
   dt_accel_register_slider_iop(self, FALSE, NC_("accel", "target level"));
+  dt_accel_register_combobox_iop(self, FALSE, NC_("accel", "mode"));
 }
 
 void connect_key_accels(dt_iop_module_t *self)
 {
   dt_iop_exposure_gui_data_t *g = (dt_iop_exposure_gui_data_t *)self->gui_data;
 
-  dt_accel_connect_slider_iop(self, "mode", GTK_WIDGET(g->mode));
   dt_accel_connect_slider_iop(self, "black", GTK_WIDGET(g->black));
   dt_accel_connect_slider_iop(self, "exposure", GTK_WIDGET(g->exposure));
   dt_accel_connect_slider_iop(self, "auto-exposure", GTK_WIDGET(g->autoexpp));
   dt_accel_connect_slider_iop(self, "percentile", GTK_WIDGET(g->deflicker_percentile));
   dt_accel_connect_slider_iop(self, "target level", GTK_WIDGET(g->deflicker_target_level));
+  dt_accel_connect_combobox_iop(self, "mode", GTK_WIDGET(g->mode));
+
+  /* register hooks with current dev so that  histogram
+     can interact with this module.
+  */
+  dt_dev_proxy_exposure_t *instance = g_malloc0(sizeof(dt_dev_proxy_exposure_t));
+  instance->module = self;
+  instance->set_exposure = dt_iop_exposure_set_exposure;
+  instance->get_exposure = dt_iop_exposure_get_exposure;
+  instance->set_black = dt_iop_exposure_set_black;
+  instance->get_black = dt_iop_exposure_get_black;
+  darktable.develop->proxy.exposure
+      = g_list_prepend(darktable.develop->proxy.exposure, instance);
 }
 
 int legacy_params(dt_iop_module_t *self, const void *const old_params, const int old_version,
                   void *new_params, const int new_version)
 {
-  if(old_version == 2 && new_version == 5)
+  if(old_version == 2 && new_version == 6)
   {
     typedef struct dt_iop_exposure_params_v2_t
     {
@@ -162,9 +180,10 @@ int legacy_params(dt_iop_module_t *self, const void *const old_params, const int
 
     n->black = o->black;
     n->exposure = o->exposure;
+    n->compensate_exposure_bias = FALSE;
     return 0;
   }
-  if(old_version == 3 && new_version == 5)
+  if(old_version == 3 && new_version == 6)
   {
     typedef struct dt_iop_exposure_params_v3_t
     {
@@ -184,9 +203,10 @@ int legacy_params(dt_iop_module_t *self, const void *const old_params, const int
     n->exposure = o->exposure;
     n->deflicker_percentile = o->deflicker_percentile;
     n->deflicker_target_level = o->deflicker_target_level;
+    n->compensate_exposure_bias = FALSE;
     return 0;
   }
-  if(old_version == 4 && new_version == 5)
+  if(old_version == 4 && new_version == 6)
   {
     typedef enum dt_iop_exposure_deflicker_histogram_source_t {
       DEFLICKER_HISTOGRAM_SOURCE_THUMBNAIL,
@@ -215,6 +235,31 @@ int legacy_params(dt_iop_module_t *self, const void *const old_params, const int
     n->deflicker_target_level = o->deflicker_target_level;
     // deflicker_histogram_source is dropped. this does change output,
     // but deflicker still was not publicly released at that point
+    n->compensate_exposure_bias = FALSE;
+    return 0;
+  }
+  if(old_version == 5 && new_version == 6)
+  {
+    typedef struct dt_iop_exposure_params_v5_t
+    {
+      dt_iop_exposure_mode_t mode;
+      float black;
+      float exposure;
+      float deflicker_percentile, deflicker_target_level;
+    } dt_iop_exposure_params_v5_t;
+
+    dt_iop_exposure_params_v5_t *o = (dt_iop_exposure_params_v5_t *)old_params;
+    dt_iop_exposure_params_t *n = (dt_iop_exposure_params_t *)new_params;
+    dt_iop_exposure_params_t *d = (dt_iop_exposure_params_t *)self->default_params;
+
+    *n = *d; // start with a fresh copy of default parameters
+
+    n->mode = o->mode;
+    n->black = o->black;
+    n->exposure = o->exposure;
+    n->deflicker_percentile = o->deflicker_percentile;
+    n->deflicker_target_level = o->deflicker_target_level;
+    n->compensate_exposure_bias = FALSE;
     return 0;
   }
   return 1;
@@ -222,17 +267,31 @@ int legacy_params(dt_iop_module_t *self, const void *const old_params, const int
 
 void init_presets (dt_iop_module_so_t *self)
 {
-  DT_DEBUG_SQLITE3_EXEC(dt_database_get(darktable.db), "BEGIN", NULL, NULL, NULL);
-
   dt_gui_presets_add_generic(_("magic lantern defaults"), self->op, self->version(),
                              &(dt_iop_exposure_params_t){.mode = EXPOSURE_MODE_DEFLICKER,
                                                          .black = 0.0f,
                                                          .exposure = 0.0f,
                                                          .deflicker_percentile = 50.0f,
-                                                         .deflicker_target_level = -4.0f },
+                                                         .deflicker_target_level = -4.0f,
+                                                         .compensate_exposure_bias = FALSE},
                              sizeof(dt_iop_exposure_params_t), 1);
 
-  DT_DEBUG_SQLITE3_EXEC(dt_database_get(darktable.db), "COMMIT", NULL, NULL, NULL);
+
+  // For scene-referred workflow, since filmic doesn't brighten as base curve does,
+  // we need an initial exposure boost. This might be too much in some cases but…
+  dt_gui_presets_add_generic(_("scene-referred default"), self->op, self->version(),
+                             &(dt_iop_exposure_params_t){.mode = EXPOSURE_MODE_MANUAL,
+                                                         .black = -0.000244140625f,
+                                                         .exposure = 0.5f,
+                                                         .deflicker_percentile = 50.0f,
+                                                         .deflicker_target_level = -4.0f,
+                                                         .compensate_exposure_bias = TRUE},
+                             sizeof(dt_iop_exposure_params_t), 1);
+
+  dt_gui_presets_update_ldr(_("scene-referred default"), self->op, self->version(), FOR_RAW);
+
+  dt_gui_presets_update_autoapply(_("scene-referred default"), self->op, self->version(),
+     (strcmp(dt_conf_get_string("plugins/darkroom/workflow"), "scene-referred") == 0));
 }
 
 static void deflicker_prepare_histogram(dt_iop_module_t *self, uint32_t **histogram,
@@ -351,7 +410,7 @@ static void process_common_setup(dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *
     }
 
     // second, show computed correction in UI.
-    if(g && piece->pipe->type == DT_DEV_PIXELPIPE_PREVIEW)
+    if(g && (piece->pipe->type & DT_DEV_PIXELPIPE_PREVIEW) == DT_DEV_PIXELPIPE_PREVIEW)
     {
       dt_pthread_mutex_lock(&g->lock);
       g->deflicker_computed_exposure = exposure;
@@ -451,13 +510,37 @@ void process_sse2(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, c
 }
 #endif
 
+
+static float get_exposure_bias(const struct dt_iop_module_t *self)
+{
+  float bias = 0.0f;
+
+  // just check that pointers exist and are initialized
+  if(&(self->dev->image_storage) && &(self->dev->image_storage.exif_exposure_bias))
+    bias = self->dev->image_storage.exif_exposure_bias;
+
+  // sanity checks because I don't trust exif tags too much
+  if(!isnan(bias))
+    return CLAMP(bias, -5.0f, 5.0f);
+  else
+    return 0.0f;
+}
+
+
 void commit_params(struct dt_iop_module_t *self, dt_iop_params_t *p1, dt_dev_pixelpipe_t *pipe,
                    dt_dev_pixelpipe_iop_t *piece)
 {
   dt_iop_exposure_params_t *p = (dt_iop_exposure_params_t *)p1;
   dt_iop_exposure_data_t *d = (dt_iop_exposure_data_t *)piece->data;
 
-  d->params = *p;
+  d->params.black = p->black;
+  d->params.exposure = p->exposure;
+  d->params.deflicker_percentile = p->deflicker_percentile;
+  d->params.deflicker_target_level = p->deflicker_target_level;
+
+  // If exposure bias compensation has been required, add it on top of user exposure correction
+  if(p->compensate_exposure_bias)
+    d->params.exposure -= get_exposure_bias(self);
 
   d->deflicker = 0;
 
@@ -504,8 +587,13 @@ void gui_update(struct dt_iop_module_t *self)
 
   dt_iop_color_picker_reset(self, TRUE);
 
-  dt_bauhaus_combobox_set(g->mode, g_list_index(g->modes, GUINT_TO_POINTER(p->mode)));
+  dt_bauhaus_combobox_set(g->mode, p->mode);
 
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(g->compensate_exposure_bias), p->compensate_exposure_bias);
+  /* xgettext:no-c-format */
+  gtk_button_set_label(GTK_BUTTON(g->compensate_exposure_bias), g_strdup_printf(_("compensate camera exposure (%+.1f EV)"),
+                                                                                  get_exposure_bias(self)));
+  gtk_label_set_ellipsize(GTK_LABEL(gtk_bin_get_child(GTK_BIN(g->compensate_exposure_bias))), PANGO_ELLIPSIZE_MIDDLE);
   dt_bauhaus_slider_set_soft(g->black, p->black);
   dt_bauhaus_slider_set_soft(g->exposure, p->exposure);
 
@@ -540,34 +628,11 @@ void gui_update(struct dt_iop_module_t *self)
 void gui_focus(struct dt_iop_module_t *self, gboolean in)
 {
   // switch off auto exposure when we lose focus (switching images etc)
+
   dt_iop_exposure_gui_data_t *g = (dt_iop_exposure_gui_data_t *)self->gui_data;
-
-  const int reset = darktable.gui->reset;
-  darktable.gui->reset = 1;
+  ++darktable.gui->reset;
   dt_bauhaus_slider_set(g->autoexpp, 0.01);
-  darktable.gui->reset = reset;
-}
-
-void init(dt_iop_module_t *module)
-{
-  module->params = calloc(1, sizeof(dt_iop_exposure_params_t));
-  module->default_params = calloc(1, sizeof(dt_iop_exposure_params_t));
-  module->default_enabled = 0;
-  module->params_size = sizeof(dt_iop_exposure_params_t);
-  module->gui_data = NULL;
-}
-
-void reload_defaults(dt_iop_module_t *module)
-{
-  dt_iop_exposure_params_t tmp = (dt_iop_exposure_params_t){.mode = EXPOSURE_MODE_MANUAL,
-                                                            .black = 0.0f,
-                                                            .exposure = 0.0f,
-                                                            .deflicker_percentile = 50.0f,
-                                                            .deflicker_target_level = -4.0f
-  };
-
-  memcpy(module->params, &tmp, sizeof(dt_iop_exposure_params_t));
-  memcpy(module->default_params, &tmp, sizeof(dt_iop_exposure_params_t));
+  --darktable.gui->reset;
 }
 
 void init_global(dt_iop_module_so_t *module)
@@ -579,14 +644,6 @@ void init_global(dt_iop_module_so_t *module)
   gd->kernel_exposure = dt_opencl_create_kernel(program, "exposure");
 }
 
-void cleanup(dt_iop_module_t *module)
-{
-  free(module->params);
-  module->params = NULL;
-  free(module->default_params);
-  module->default_params = NULL;
-}
-
 void cleanup_global(dt_iop_module_so_t *module)
 {
   dt_iop_exposure_global_data_t *gd = (dt_iop_exposure_global_data_t *)module->data;
@@ -595,55 +652,13 @@ void cleanup_global(dt_iop_module_so_t *module)
   module->data = NULL;
 }
 
-static void mode_callback(GtkWidget *combo, gpointer user_data)
-{
-  dt_iop_module_t *self = (dt_iop_module_t *)user_data;
-
-  if(darktable.gui->reset) return;
-
-  dt_iop_exposure_gui_data_t *g = (dt_iop_exposure_gui_data_t *)self->gui_data;
-  dt_iop_exposure_params_t *p = (dt_iop_exposure_params_t *)self->params;
-
-  dt_iop_color_picker_reset(self, TRUE);
-
-  const dt_iop_exposure_mode_t new_mode
-      = GPOINTER_TO_UINT(g_list_nth_data(g->modes, dt_bauhaus_combobox_get(combo)));
-
-  free(g->deflicker_histogram);
-  g->deflicker_histogram = NULL;
-
-  switch(new_mode)
-  {
-    case EXPOSURE_MODE_DEFLICKER:
-      autoexp_disable(self);
-      if(!dt_image_is_raw(&self->dev->image_storage) || self->dev->image_storage.buf_dsc.channels != 1
-         || self->dev->image_storage.buf_dsc.datatype != TYPE_UINT16)
-      {
-        dt_bauhaus_combobox_set(g->mode, g_list_index(g->modes, GUINT_TO_POINTER(EXPOSURE_MODE_MANUAL)));
-        gtk_widget_set_sensitive(GTK_WIDGET(g->mode), FALSE);
-        break;
-      }
-      p->mode = EXPOSURE_MODE_DEFLICKER;
-      gtk_stack_set_visible_child_name(GTK_STACK(g->mode_stack), "deflicker");
-      deflicker_prepare_histogram(self, &g->deflicker_histogram, &g->deflicker_histogram_stats);
-      break;
-    case EXPOSURE_MODE_MANUAL:
-    default:
-      p->mode = EXPOSURE_MODE_MANUAL;
-      gtk_stack_set_visible_child_name(GTK_STACK(g->mode_stack), "manual");
-      break;
-  }
-
-  dt_dev_add_history_item(darktable.develop, self, TRUE);
-}
-
 static void exposure_set_black(struct dt_iop_module_t *self, const float black);
 
 static void exposure_set_white(struct dt_iop_module_t *self, const float white)
 {
   dt_iop_exposure_params_t *p = (dt_iop_exposure_params_t *)self->params;
 
-  float exposure = white2exposure(white);
+  const float exposure = white2exposure(white);
   if(p->exposure == exposure) return;
 
   p->exposure = exposure;
@@ -651,10 +666,16 @@ static void exposure_set_white(struct dt_iop_module_t *self, const float white)
 
   dt_iop_exposure_gui_data_t *g = (dt_iop_exposure_gui_data_t *)self->gui_data;
 
-  const int reset = darktable.gui->reset;
-  darktable.gui->reset = 1;
+  ++darktable.gui->reset;
   dt_bauhaus_slider_set_soft(g->exposure, p->exposure);
-  darktable.gui->reset = reset;
+  --darktable.gui->reset;
+  dt_dev_add_history_item(darktable.develop, self, TRUE);
+}
+
+static void compensate_exposure_bias_callback(GtkWidget *widget, dt_iop_module_t *self)
+{
+  dt_iop_exposure_params_t *p = (dt_iop_exposure_params_t *)self->params;
+  p->compensate_exposure_bias = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
   dt_dev_add_history_item(darktable.develop, self, TRUE);
 }
 
@@ -668,16 +689,15 @@ static void dt_iop_exposure_set_exposure(struct dt_iop_module_t *self, const flo
 
     p->deflicker_target_level = exposure;
 
-    const int reset = darktable.gui->reset;
-    darktable.gui->reset = 1;
+    ++darktable.gui->reset;
     dt_bauhaus_slider_set(g->deflicker_target_level, p->deflicker_target_level);
-    darktable.gui->reset = reset;
+    --darktable.gui->reset;
 
     dt_dev_add_history_item(darktable.develop, self, TRUE);
   }
   else
   {
-    float white = exposure2white(exposure);
+    const float white = exposure2white(exposure);
     exposure_set_white(self, white);
     autoexp_disable(self);
   }
@@ -710,10 +730,9 @@ static void exposure_set_black(struct dt_iop_module_t *self, const float black)
   }
 
   dt_iop_exposure_gui_data_t *g = (dt_iop_exposure_gui_data_t *)self->gui_data;
-  const int reset = darktable.gui->reset;
-  darktable.gui->reset = 1;
+  ++darktable.gui->reset;
   dt_bauhaus_slider_set_soft(g->black, p->black);
-  darktable.gui->reset = reset;
+  --darktable.gui->reset;
   dt_dev_add_history_item(darktable.develop, self, TRUE);
 }
 
@@ -729,9 +748,9 @@ static float dt_iop_exposure_get_black(struct dt_iop_module_t *self)
   return p->black;
 }
 
-static void _iop_color_picker_apply(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece)
+void color_picker_apply(dt_iop_module_t *self, GtkWidget *picker, dt_dev_pixelpipe_iop_t *piece)
 {
-  if(self->dt->gui->reset) return;
+  if(darktable.gui->reset) return;
 
   dt_iop_exposure_gui_data_t *g = (dt_iop_exposure_gui_data_t *)self->gui_data;
   const float white = fmaxf(fmaxf(self->picked_color_max[0], self->picked_color_max[1]),
@@ -742,58 +761,61 @@ static void _iop_color_picker_apply(struct dt_iop_module_t *self, dt_dev_pixelpi
 static void autoexpp_callback(GtkWidget *slider, gpointer user_data)
 {
   dt_iop_module_t *self = (dt_iop_module_t *)user_data;
-  if(self->dt->gui->reset) return;
-  if(self->request_color_pick != DT_REQUEST_COLORPICK_MODULE || self->picked_color_max[0] < 0.0f) return;
-
   dt_iop_exposure_gui_data_t *g = (dt_iop_exposure_gui_data_t *)self->gui_data;
+
+  if(self->dt->gui->reset) return;
+  if(self->request_color_pick != DT_REQUEST_COLORPICK_MODULE || 
+     !dt_bauhaus_widget_get_quad_active(g->autoexpp) ||
+     self->picked_color_max[0] < 0.0f) return;
+
   const float white = fmaxf(fmaxf(self->picked_color_max[0], self->picked_color_max[1]),
                             self->picked_color_max[2]) * (1.0 - dt_bauhaus_slider_get(g->autoexpp));
   exposure_set_white(self, white);
 }
 
-static void deflicker_params_callback(GtkWidget *slider, gpointer user_data)
+void gui_changed(dt_iop_module_t *self, GtkWidget *w, void *previous)
 {
-  dt_iop_module_t *self = (dt_iop_module_t *)user_data;
-  if(self->dt->gui->reset) return;
-
-  if(!dt_image_is_raw(&self->dev->image_storage) || self->dev->image_storage.buf_dsc.channels != 1
-     || self->dev->image_storage.buf_dsc.datatype != TYPE_UINT16)
-    return;
-
   dt_iop_exposure_gui_data_t *g = (dt_iop_exposure_gui_data_t *)self->gui_data;
   dt_iop_exposure_params_t *p = (dt_iop_exposure_params_t *)self->params;
 
-  if(p->mode != EXPOSURE_MODE_DEFLICKER) return;
+  if(w == g->mode)
+  {
+    free(g->deflicker_histogram);
+    g->deflicker_histogram = NULL;
 
-  dt_iop_color_picker_reset(self, TRUE);
-
-  p->deflicker_percentile = dt_bauhaus_slider_get(g->deflicker_percentile);
-  p->deflicker_target_level = dt_bauhaus_slider_get(g->deflicker_target_level);
-
-  dt_dev_add_history_item(darktable.develop, self, TRUE);
-}
-
-static void exposure_callback(GtkWidget *slider, gpointer user_data)
-{
-  dt_iop_module_t *self = (dt_iop_module_t *)user_data;
-  if(self->dt->gui->reset) return;
-
-  dt_iop_color_picker_reset(self, TRUE);
-  autoexp_disable(self);
-
-  const float exposure = dt_bauhaus_slider_get(slider);
-  dt_iop_exposure_set_exposure(self, exposure);
-}
-
-static void black_callback(GtkWidget *slider, gpointer user_data)
-{
-  dt_iop_module_t *self = (dt_iop_module_t *)user_data;
-  if(self->dt->gui->reset) return;
-
-  dt_iop_color_picker_reset(self, TRUE);
-
-  const float black = dt_bauhaus_slider_get(slider);
-  dt_iop_exposure_set_black(self, black);
+    switch(p->mode)
+    {
+      case EXPOSURE_MODE_DEFLICKER:
+        autoexp_disable(self);
+        if(!dt_image_is_raw(&self->dev->image_storage) || self->dev->image_storage.buf_dsc.channels != 1
+          || self->dev->image_storage.buf_dsc.datatype != TYPE_UINT16)
+        {
+          p->mode = EXPOSURE_MODE_MANUAL;
+          dt_bauhaus_combobox_set(g->mode, p->mode);
+          gtk_widget_set_sensitive(GTK_WIDGET(g->mode), FALSE);
+          break;
+        }
+        gtk_stack_set_visible_child_name(GTK_STACK(g->mode_stack), "deflicker");
+        deflicker_prepare_histogram(self, &g->deflicker_histogram, &g->deflicker_histogram_stats);
+        break;
+      case EXPOSURE_MODE_MANUAL:
+      default:
+        gtk_stack_set_visible_child_name(GTK_STACK(g->mode_stack), "manual");
+        break;
+    }
+  }
+  else if(w == g->exposure)
+  {
+    const float white = exposure2white(p->exposure);
+    if(p->black >= white)
+      exposure_set_black(self, white - 0.01);
+  }
+  else if(w == g->black)
+  {
+    const float white = exposure2white(p->exposure);
+    if(p->black >= white)
+      exposure_set_white(self, p->black + 0.01);
+  }
 }
 
 static gboolean draw(GtkWidget *widget, cairo_t *cr, dt_iop_module_t *self)
@@ -807,18 +829,19 @@ static gboolean draw(GtkWidget *widget, cairo_t *cr, dt_iop_module_t *self)
   {
     gchar *str = g_strdup_printf(_("%.2f EV"), g->deflicker_computed_exposure);
 
-    const int reset = darktable.gui->reset;
-    darktable.gui->reset = 1;
+    ++darktable.gui->reset;
     gtk_label_set_text(g->deflicker_used_EC, str);
-    darktable.gui->reset = reset;
+    --darktable.gui->reset;
 
     g_free(str);
   }
   dt_pthread_mutex_unlock(&g->lock);
 
-  if(self->request_color_pick != DT_REQUEST_COLORPICK_MODULE) return FALSE;
+  // if color-picker active and is the one in the main module (not blending ones)
 
-  if(self->picked_color_max[0] < 0.0f) return FALSE;
+  if(self->request_color_pick != DT_REQUEST_COLORPICK_MODULE ||
+     !dt_bauhaus_widget_get_quad_active(g->autoexpp) ||
+     self->picked_color_max[0] < 0.0f) return FALSE;
 
   const float white = fmaxf(fmaxf(self->picked_color_max[0], self->picked_color_max[1]),
                             self->picked_color_max[2]) * (1.0 - dt_bauhaus_slider_get(g->autoexpp));
@@ -841,97 +864,57 @@ void gui_init(struct dt_iop_module_t *self)
   dt_iop_exposure_gui_data_t *g = (dt_iop_exposure_gui_data_t *)self->gui_data;
   dt_iop_exposure_params_t *p = (dt_iop_exposure_params_t *)self->params;
 
-  g->modes = NULL;
-
   g->deflicker_histogram = NULL;
 
   dt_pthread_mutex_init(&g->lock, NULL);
 
-  /* register hooks with current dev so that  histogram
-     can interact with this module.
-   */
-  dt_dev_proxy_exposure_t *instance = g_malloc0(sizeof(dt_dev_proxy_exposure_t));
-  instance->module = self;
-  instance->set_exposure = dt_iop_exposure_set_exposure;
-  instance->get_exposure = dt_iop_exposure_get_exposure;
-  instance->set_black = dt_iop_exposure_set_black;
-  instance->get_black = dt_iop_exposure_get_black;
-  darktable.develop->proxy.exposure
-      = g_list_insert_sorted(darktable.develop->proxy.exposure, instance, dt_dev_exposure_hooks_sort);
-
-  self->widget = GTK_WIDGET(gtk_box_new(GTK_ORIENTATION_VERTICAL, DT_BAUHAUS_SPACE));
-  dt_gui_add_help_link(self->widget, dt_get_help_url(self->op));
-
-  g->mode = dt_bauhaus_combobox_new(self);
-  dt_bauhaus_widget_set_label(g->mode, NULL, _("mode"));
-
-  dt_bauhaus_combobox_add(g->mode, C_("mode", "manual"));
-  g->modes = g_list_append(g->modes, GUINT_TO_POINTER(EXPOSURE_MODE_MANUAL));
-
-  dt_bauhaus_combobox_add(g->mode, _("automatic"));
-  g->modes = g_list_append(g->modes, GUINT_TO_POINTER(EXPOSURE_MODE_DEFLICKER));
-
-  dt_bauhaus_combobox_set_default(g->mode, 0);
-  dt_bauhaus_combobox_set(g->mode, g_list_index(g->modes, GUINT_TO_POINTER(p->mode)));
-
-  gtk_box_pack_start(GTK_BOX(self->widget), GTK_WIDGET(g->mode), TRUE, TRUE, 0);
-
-  g->mode_stack = gtk_stack_new();
+  g->mode_stack = GTK_STACK(gtk_stack_new());
   gtk_stack_set_homogeneous(GTK_STACK(g->mode_stack),FALSE);
-  gtk_box_pack_start(GTK_BOX(self->widget), g->mode_stack, TRUE, TRUE, 0);
 
-  GtkWidget *vbox_manual = GTK_WIDGET(gtk_box_new(GTK_ORIENTATION_VERTICAL, DT_BAUHAUS_SPACE));
+  GtkWidget *vbox_manual = self->widget = gtk_box_new(GTK_ORIENTATION_VERTICAL, DT_BAUHAUS_SPACE);
+  gtk_stack_add_named(GTK_STACK(g->mode_stack), vbox_manual, "manual");
 
-  g->exposure = dt_bauhaus_slider_new_with_range(self, -3.0, 3.0, .02, p->exposure, 3);
+  /* xgettext:no-c-format */
+  g->compensate_exposure_bias = gtk_check_button_new_with_label(g_strdup_printf(_("compensate camera exposure (%+.1f EV)"),
+                                                                                  get_exposure_bias(self)));
+  gtk_label_set_ellipsize(GTK_LABEL(gtk_bin_get_child(GTK_BIN(g->compensate_exposure_bias))), PANGO_ELLIPSIZE_MIDDLE);
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(g->compensate_exposure_bias), p->compensate_exposure_bias);
+  gtk_widget_set_tooltip_text(g->compensate_exposure_bias, _("automatically remove the camera exposure bias\n"
+                                                             "this is useful if you exposed the image to the right."));
+  gtk_box_pack_start(GTK_BOX(vbox_manual), g->compensate_exposure_bias, TRUE, TRUE, 0);
+
+  g->exposure = dt_bauhaus_slider_from_params(self, N_("exposure"));
   gtk_widget_set_tooltip_text(g->exposure, _("adjust the exposure correction"));
+  dt_bauhaus_slider_set_step(g->exposure, 0.02);
+  dt_bauhaus_slider_set_digits(g->exposure, 3);
   dt_bauhaus_slider_set_format(g->exposure, _("%.2f EV"));
-  dt_bauhaus_widget_set_label(g->exposure, NULL, _("exposure"));
-  dt_bauhaus_slider_enable_soft_boundaries(g->exposure, -18.0, 18.0);
-  gtk_box_pack_start(GTK_BOX(vbox_manual), GTK_WIDGET(g->exposure), TRUE, TRUE, 0);
+  dt_bauhaus_slider_set_soft_range(g->exposure, -3.0, 3.0);
 
-  g->black = dt_bauhaus_slider_new_with_range(self, -0.1, 0.1, .001, p->black, 4);
-  gtk_widget_set_tooltip_text(g->black, _("adjust the black level to unclip negative RGB values.\n"
-                                          "you should never use it to add more density in blacks!\n"
-                                          "if poorly set, it will clip near-black colors out of gamut\n"
-                                          "by pushing RGB values into negatives."));
-  dt_bauhaus_slider_set_format(g->black, "%.4f");
-  dt_bauhaus_widget_set_label(g->black, NULL, _("black level correction"));
-  dt_bauhaus_slider_enable_soft_boundaries(g->black, -1.0, 1.0);
-  gtk_box_pack_start(GTK_BOX(self->widget), GTK_WIDGET(g->black), TRUE, TRUE, 0);
-
-  g->autoexpp = dt_bauhaus_slider_new_with_range(self, 0.0, 0.2, .001, 0.01, 3);
+  g->autoexpp = dt_color_picker_new(self, DT_COLOR_PICKER_AREA,
+                dt_bauhaus_slider_new_with_range(self, 0.0, 0.2, .001, 0.01, 3));
   gtk_widget_set_tooltip_text(g->autoexpp, _("percentage of bright values clipped out, toggle color picker to activate"));
   dt_bauhaus_slider_set_format(g->autoexpp, "%.3f%%");
   dt_bauhaus_widget_set_label(g->autoexpp, NULL, _("clipping threshold"));
-  dt_bauhaus_widget_set_quad_paint(g->autoexpp, dtgtk_cairo_paint_colorpicker, CPF_STYLE_FLAT | CPF_DO_NOT_USE_BORDER, NULL);
-  dt_bauhaus_widget_set_quad_toggle(g->autoexpp, TRUE);
+  g_signal_connect(G_OBJECT(g->autoexpp), "value-changed", G_CALLBACK(autoexpp_callback), self);
   gtk_box_pack_start(GTK_BOX(vbox_manual), GTK_WIDGET(g->autoexpp), TRUE, TRUE, 0);
 
-  gtk_widget_show_all(vbox_manual);
-  gtk_stack_add_named(GTK_STACK(g->mode_stack), vbox_manual, "manual");
+  GtkWidget *vbox_deflicker = self->widget = gtk_box_new(GTK_ORIENTATION_VERTICAL, DT_BAUHAUS_SPACE);
+  gtk_stack_add_named(GTK_STACK(g->mode_stack), vbox_deflicker, "deflicker");
 
-  GtkWidget *vbox_deflicker = GTK_WIDGET(gtk_box_new(GTK_ORIENTATION_VERTICAL, DT_BAUHAUS_SPACE));
-
-  g->deflicker_percentile = dt_bauhaus_slider_new_with_range(self, 0, 100, 1.0, p->deflicker_percentile, 3);
-  dt_bauhaus_widget_set_label(g->deflicker_percentile, NULL, _("percentile"));
+  g->deflicker_percentile = dt_bauhaus_slider_from_params(self, "deflicker_percentile");
   dt_bauhaus_slider_set_format(g->deflicker_percentile, "%.2f%%");
   gtk_widget_set_tooltip_text(g->deflicker_percentile,
                               // xgettext:no-c-format
                               _("where in the histogram to meter for deflicking. E.g. 50% is median"));
-  gtk_box_pack_start(GTK_BOX(vbox_deflicker), GTK_WIDGET(g->deflicker_percentile), TRUE, TRUE, 0);
 
-  g->deflicker_target_level
-      = dt_bauhaus_slider_new_with_range(self, -18.0, 18.0, .01, p->deflicker_target_level, 3);
-  dt_bauhaus_widget_set_label(g->deflicker_target_level, NULL, _("target level"));
+  g->deflicker_target_level = dt_bauhaus_slider_from_params(self, "deflicker_target_level");
+  dt_bauhaus_slider_set_step(g->deflicker_target_level, 0.1);
   dt_bauhaus_slider_set_format(g->deflicker_target_level, _("%.2f EV"));
   gtk_widget_set_tooltip_text(g->deflicker_target_level,
                               _("where to place the exposure level for processed pics, EV below overexposure."));
-  gtk_box_pack_start(GTK_BOX(vbox_deflicker), GTK_WIDGET(g->deflicker_target_level), TRUE, TRUE, 0);
 
   GtkBox *hbox1 = GTK_BOX(gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0));
-  GtkLabel *label = GTK_LABEL(gtk_label_new(_("computed EC: ")));
-  gtk_box_pack_start(GTK_BOX(hbox1), GTK_WIDGET(label), FALSE, FALSE, 0);
-
+  gtk_box_pack_start(GTK_BOX(hbox1), GTK_WIDGET(gtk_label_new(_("computed EC: "))), FALSE, FALSE, 0);
   g->deflicker_used_EC = GTK_LABEL(gtk_label_new("")); // This gets filled in by process
   gtk_widget_set_tooltip_text(GTK_WIDGET(g->deflicker_used_EC), _("what exposure correction has actually been used"));
   gtk_box_pack_start(GTK_BOX(hbox1), GTK_WIDGET(g->deflicker_used_EC), FALSE, FALSE, 0);
@@ -942,25 +925,24 @@ void gui_init(struct dt_iop_module_t *self)
 
   gtk_box_pack_start(GTK_BOX(vbox_deflicker), GTK_WIDGET(hbox1), FALSE, FALSE, 0);
 
-  gtk_widget_show_all(vbox_deflicker);
-  gtk_stack_add_named(GTK_STACK(g->mode_stack), vbox_deflicker, "deflicker");
+  // Start building top level widget
+  self->widget = GTK_WIDGET(gtk_box_new(GTK_ORIENTATION_VERTICAL, DT_BAUHAUS_SPACE));
 
-  g_signal_connect(G_OBJECT(g->mode), "value-changed", G_CALLBACK(mode_callback), self);
-  g_signal_connect(G_OBJECT(g->black), "value-changed", G_CALLBACK(black_callback), self);
-  g_signal_connect(G_OBJECT(g->exposure), "value-changed", G_CALLBACK(exposure_callback), self);
-  g_signal_connect(G_OBJECT(g->autoexpp), "value-changed", G_CALLBACK(autoexpp_callback), self);
-  g_signal_connect(G_OBJECT(g->autoexpp), "quad-pressed", G_CALLBACK(dt_iop_color_picker_callback), &g->color_picker);
-  g_signal_connect(G_OBJECT(g->deflicker_percentile), "value-changed", G_CALLBACK(deflicker_params_callback),
-                   self);
-  g_signal_connect(G_OBJECT(g->deflicker_target_level), "value-changed",
-                   G_CALLBACK(deflicker_params_callback), self);
+  g->mode = dt_bauhaus_combobox_from_params(self, N_("mode"));
+
+  gtk_box_pack_start(GTK_BOX(self->widget), GTK_WIDGET(g->mode_stack), TRUE, TRUE, 0);
+
+  g->black = dt_bauhaus_slider_from_params(self, "black");
+  gtk_widget_set_tooltip_text(g->black, _("adjust the black level to unclip negative RGB values.\n"
+                                          "you should never use it to add more density in blacks!\n"
+                                          "if poorly set, it will clip near-black colors out of gamut\n"
+                                          "by pushing RGB values into negatives."));
+  dt_bauhaus_slider_set_step(g->black, 0.001);
+  dt_bauhaus_slider_set_digits(g->black, 4);
+  dt_bauhaus_slider_set_soft_range(g->black, -0.1, 0.1);
+
+  g_signal_connect(G_OBJECT(g->compensate_exposure_bias), "toggled", G_CALLBACK(compensate_exposure_bias_callback), self);
   g_signal_connect(G_OBJECT(self->widget), "draw", G_CALLBACK(draw), self);
-
-  dt_iop_init_single_picker(&g->color_picker,
-                     self,
-                     GTK_WIDGET(g->autoexpp),
-                     DT_COLOR_PICKER_AREA,
-                     _iop_color_picker_apply);
 }
 
 void gui_cleanup(struct dt_iop_module_t *self)
@@ -982,7 +964,6 @@ void gui_cleanup(struct dt_iop_module_t *self)
 
   free(g->deflicker_histogram);
   g->deflicker_histogram = NULL;
-  g_list_free(g->modes);
 
   dt_pthread_mutex_destroy(&g->lock);
 
