@@ -1,6 +1,6 @@
 /*
     This file is part of darktable,
-    copyright (c) 2010-2012 Henrik Andersson.
+    Copyright (C) 2010-2020 darktable developers.
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -31,6 +31,7 @@
 #include "develop/develop.h"
 #include "develop/imageop.h"
 #include "develop/imageop_math.h"
+#include "develop/imageop_gui.h"
 #include "develop/tiling.h"
 #include "dtgtk/gradientslider.h"
 #include "gui/accelerators.h"
@@ -49,12 +50,12 @@ DT_MODULE_INTROSPECTION(1, dt_iop_graduatednd_params_t)
 
 typedef struct dt_iop_graduatednd_params_t
 {
-  float density;     // The density of filter 0-8 EV
-  float hardness; // Default 0% = soft and 100% = hard
-  float rotation;    // 2*PI -180 - +180
-  float offset;      // Default 50%, centered, can be offsetted...
-  float hue;         // the hue
-  float saturation;  // the saturation
+  float density;     // $MIN: -8.0 $MAX: 8.0 $DEFAULT: 1.0 $DESCRIPTION: "density" The density of filter 0-8 EV
+  float hardness;    // $MIN: 0.0 $MAX: 100.0 $DEFAULT: 0.0 $DESCRIPTION: "hardness" 0% = soft and 100% = hard
+  float rotation;    // $MIN: -180.0 $MAX: 180.0 $DEFAULT: 0.0 $DESCRIPTION: "rotation" 2*PI -180 - +180
+  float offset;      // $DEFAULT: 50.0 $DESCRIPTION: "offset" centered, can be offsetted...
+  float hue;         // $MIN: 0.0 $MAX: 1.0 $DEFAULT: 0.0 $DESCRIPTION: "hue"
+  float saturation;  // $MIN: 0.0 $MAX: 1.0 $DEFAULT: 0.0 $DESCRIPTION: "saturation"
 } dt_iop_graduatednd_params_t;
 
 typedef struct dt_iop_graduatednd_global_data_t
@@ -113,12 +114,9 @@ void init_presets(dt_iop_module_so_t *self)
 
 typedef struct dt_iop_graduatednd_gui_data_t
 {
-  GtkBox *vbox;
-  GtkWidget *label1, *label2, *label3, *label5, *label6; // density, hardness, rotation, hue, saturation
-  GtkWidget *scale1, *scale2, *scale3;                   // density, hardness, rotation
-  GtkWidget *gslider1, *gslider2;                        // hue, saturation
-
-  dt_iop_color_picker_t color_picker;
+//  GtkBox *vbox;
+//  GtkWidget *label1, *label2, *label3, *label5, *label6; // density, hardness, rotation, hue, saturation
+  GtkWidget *density, *hardness, *rotation, *hue, *saturation;
 
   int selected;
   int dragging;
@@ -172,11 +170,11 @@ void connect_key_accels(dt_iop_module_t *self)
 {
   dt_iop_graduatednd_gui_data_t *g = (dt_iop_graduatednd_gui_data_t *)self->gui_data;
 
-  dt_accel_connect_slider_iop(self, "density", GTK_WIDGET(g->scale1));
-  dt_accel_connect_slider_iop(self, "hardness", GTK_WIDGET(g->scale2));
-  dt_accel_connect_slider_iop(self, "rotation", GTK_WIDGET(g->scale3));
-  dt_accel_connect_slider_iop(self, "hue", GTK_WIDGET(g->gslider1));
-  dt_accel_connect_slider_iop(self, "saturation", GTK_WIDGET(g->gslider2));
+  dt_accel_connect_slider_iop(self, "density", GTK_WIDGET(g->density));
+  dt_accel_connect_slider_iop(self, "hardness", GTK_WIDGET(g->hardness));
+  dt_accel_connect_slider_iop(self, "rotation", GTK_WIDGET(g->rotation));
+  dt_accel_connect_slider_iop(self, "hue", GTK_WIDGET(g->hue));
+  dt_accel_connect_slider_iop(self, "saturation", GTK_WIDGET(g->saturation));
 }
 
 static inline float f(const float t, const float c, const float x)
@@ -195,20 +193,20 @@ static float dist_seg(float xa, float ya, float xb, float yb, float xc, float yc
 {
   if(xa == xb && ya == yb) return (xc - xa) * (xc - xa) + (yc - ya) * (yc - ya);
 
-  float sx = xb - xa;
-  float sy = yb - ya;
+  const float sx = xb - xa;
+  const float sy = yb - ya;
 
-  float ux = xc - xa;
-  float uy = yc - ya;
+  const float ux = xc - xa;
+  const float uy = yc - ya;
 
-  float dp = sx * ux + sy * uy;
+  const float dp = sx * ux + sy * uy;
   if(dp < 0) return (xc - xa) * (xc - xa) + (yc - ya) * (yc - ya);
 
-  float sn2 = sx * sx + sy * sy;
+  const float sn2 = sx * sx + sy * sy;
   if(dp > sn2) return (xc - xb) * (xc - xb) + (yc - yb) * (yc - yb);
 
-  float ah2 = dp * dp / sn2;
-  float un2 = ux * ux + uy * uy;
+  const float ah2 = dp * dp / sn2;
+  const float un2 = ux * ux + uy * uy;
   return un2 - ah2;
 }
 
@@ -289,17 +287,19 @@ static int set_grad_from_points(struct dt_iop_module_t *self, float xa, float ya
   }
   else // let's pretend that we are at PI/2
   {
-    if(v <0) v = -MPI2;
-    else v = MPI2;
+    const float diff_y = pts[3] - pts[1];
+    if(diff_y <= 0.0f) v = -MPI2;
+    else               v = MPI2;
   }
 
-  *rotation = -v * 180.0 / M_PI;
+  *rotation = -v * 180.0f / M_PI;
 
   // and now we go for the offset (more easy)
   sinv = sinf(v);
   cosv = cosf(v);
-  float ofs = -2.0 * sinv * pts[0] + sinv - cosv + 1.0 + 2.0 * cosv * pts[1];
-  *offset = ofs * 50.0;
+  const float ofs = (-2.0f * sinv * pts[0]) + sinv - cosv + 1.0f + (2.0f * cosv * pts[1]);
+
+  *offset = ofs * 50.0f;
 
   return 1;
 }
@@ -317,31 +317,48 @@ static int set_points_from_grad(struct dt_iop_module_t *self, float *xa, float *
   float wp = piece->buf_out.width, hp = piece->buf_out.height;
 
   // if sinv=0 then this is just the offset
-  if(sinv == 0)
+
+  if(sinv == 0.0f) // horizontal
   {
-    if(v == 0)
+    if(rotation == 0.0f)
     {
-      pts[0] = wp * 0.1;
-      pts[2] = wp * 0.9;
-      pts[1] = pts[3] = hp * offset / 100.0;
+      pts[0] = wp * 0.1f;
+      pts[2] = wp * 0.9f;
+      pts[1] = pts[3] = hp * offset / 100.0f;
     }
     else
     {
-      pts[2] = wp * 0.1;
-      pts[0] = wp * 0.9;
-      pts[1] = pts[3] = hp * (1.0 - offset / 100.0);
+      pts[2] = wp * 0.1f;
+      pts[0] = wp * 0.9f;
+      pts[1] = pts[3] = hp * (1.0f - offset / 100.0f);
+    }
+  }
+  else if(fabsf(sinv) == 1) // vertical
+  {
+    if(rotation == 90)
+    {
+      pts[0] = pts[2] = wp * offset / 100.0f;
+      pts[3] = hp * 0.1f;
+      pts[1] = hp * 0.9f;
+    }
+    else
+    {
+      pts[0] = pts[2] = wp * (1.0 - offset / 100.0f);
+      pts[1] = hp * 0.1f;
+      pts[3] = hp * 0.9f;
     }
   }
   else
   {
     // otherwise we determine the extremities
     const float cosv = cos(v);
-    float xx1 = (sinv - cosv + 1.0 - offset / 50.0) * wp * 0.5 / sinv;
-    float xx2 = (sinv + cosv + 1.0 - offset / 50.0) * wp * 0.5 / sinv;
-    float yy1 = 0;
+    float xx1 = (sinv - cosv + 1.0f - offset / 50.0f) * wp * 0.5f / sinv;
+    float xx2 = (sinv + cosv + 1.0f - offset / 50.0f) * wp * 0.5f / sinv;
+    float yy1 = 0.0f;
     float yy2 = hp;
-    float a = hp / (xx2 - xx1);
-    float b = -xx1 * a;
+    const float a = hp / (xx2 - xx1);
+    const float b = -xx1 * a;
+
     // now ensure that the line isn't outside image borders
     if(xx2 > wp)
     {
@@ -370,12 +387,7 @@ static int set_points_from_grad(struct dt_iop_module_t *self, float *xa, float *
     yy2 -= (yy2 - yy1) * 0.1;
     yy1 += (yy2 - yy1) * 0.1;
 
-    // now we have to decide which point is where, depending of the angle
-    /*xx1 /= wd;
-    xx2 /= wd;
-    yy1 /= ht;
-    yy2 /= ht;*/
-    if(v < M_PI * 0.5 && v > -M_PI * 0.5)
+    if(rotation < 90.0f && rotation > -90.0f)
     {
       // we want xa < xb
       if(xx1 < xx2)
@@ -430,7 +442,7 @@ static inline void update_saturation_slider_end_color(GtkWidget *slider, float h
   dt_bauhaus_slider_set_stop(slider, 1.0, rgb[0], rgb[1], rgb[2]);
 }
 
-static void _iop_color_picker_apply(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece)
+void color_picker_apply(dt_iop_module_t *self, GtkWidget *picker, dt_dev_pixelpipe_iop_t *piece)
 {
   dt_iop_graduatednd_gui_data_t *g = (dt_iop_graduatednd_gui_data_t *)self->gui_data;
   dt_iop_graduatednd_params_t *p = (dt_iop_graduatednd_params_t *)self->params;
@@ -448,12 +460,11 @@ static void _iop_color_picker_apply(struct dt_iop_module_t *self, dt_dev_pixelpi
   p->hue        = H;
   p->saturation = S;
 
-  const int reset = darktable.gui->reset;
-  darktable.gui->reset = 1;
-  dt_bauhaus_slider_set(g->gslider1, p->hue);
-  dt_bauhaus_slider_set(g->gslider2, p->saturation);
-  update_saturation_slider_end_color(g->gslider2, p->hue);
-  darktable.gui->reset = reset;
+  ++darktable.gui->reset;
+  dt_bauhaus_slider_set(g->hue, p->hue);
+  dt_bauhaus_slider_set(g->saturation, p->saturation);
+  update_saturation_slider_end_color(g->saturation, p->hue);
+  --darktable.gui->reset;
 
   dt_dev_add_history_item(darktable.develop, self, TRUE);
 }
@@ -470,13 +481,13 @@ void gui_post_expose(struct dt_iop_module_t *self, cairo_t *cr, int32_t width, i
   dt_iop_graduatednd_gui_data_t *g = (dt_iop_graduatednd_gui_data_t *)self->gui_data;
   dt_iop_graduatednd_params_t *p = (dt_iop_graduatednd_params_t *)self->params;
 
-  float wd = dev->preview_pipe->backbuf_width;
-  float ht = dev->preview_pipe->backbuf_height;
-  float zoom_y = dt_control_get_dev_zoom_y();
-  float zoom_x = dt_control_get_dev_zoom_x();
-  dt_dev_zoom_t zoom = dt_control_get_dev_zoom();
-  int closeup = dt_control_get_dev_closeup();
-  float zoom_scale = dt_dev_get_zoom_scale(dev, zoom, 1<<closeup, 1);
+  const float wd = dev->preview_pipe->backbuf_width;
+  const float ht = dev->preview_pipe->backbuf_height;
+  const float zoom_y = dt_control_get_dev_zoom_y();
+  const float zoom_x = dt_control_get_dev_zoom_x();
+  const dt_dev_zoom_t zoom = dt_control_get_dev_zoom();
+  const int closeup = dt_control_get_dev_closeup();
+  const float zoom_scale = dt_dev_get_zoom_scale(dev, zoom, 1<<closeup, 1);
 
   cairo_translate(cr, width / 2.0, height / 2.0f);
   cairo_scale(cr, zoom_scale, zoom_scale);
@@ -489,14 +500,14 @@ void gui_post_expose(struct dt_iop_module_t *self, cairo_t *cr, int32_t width, i
     g->define = 1;
   }
 
-  float xa = g->xa * wd, xb = g->xb * wd, ya = g->ya * ht, yb = g->yb * ht;
+  const float xa = g->xa * wd, xb = g->xb * wd, ya = g->ya * ht, yb = g->yb * ht;
   // the lines
   cairo_set_line_cap(cr, CAIRO_LINE_CAP_ROUND);
   if(g->selected == 3 || g->dragging == 3)
     cairo_set_line_width(cr, DT_PIXEL_APPLY_DPI(5.0) / zoom_scale);
   else
     cairo_set_line_width(cr, DT_PIXEL_APPLY_DPI(3.0) / zoom_scale);
-  cairo_set_source_rgba(cr, .3, .3, .3, .8);
+  dt_draw_set_color_overlay(cr, 0.3, 0.8);
 
   cairo_move_to(cr, xa, ya);
   cairo_line_to(cr, xb, yb);
@@ -506,14 +517,14 @@ void gui_post_expose(struct dt_iop_module_t *self, cairo_t *cr, int32_t width, i
     cairo_set_line_width(cr, DT_PIXEL_APPLY_DPI(2.0) / zoom_scale);
   else
     cairo_set_line_width(cr, DT_PIXEL_APPLY_DPI(1.0) / zoom_scale);
-  cairo_set_source_rgba(cr, .8, .8, .8, .8);
+  dt_draw_set_color_overlay(cr, 0.8, 0.8);
   cairo_move_to(cr, xa, ya);
   cairo_line_to(cr, xb, yb);
   cairo_stroke(cr);
 
   // the extremities
   float x1, y1, x2, y2;
-  float l = sqrt((xb - xa) * (xb - xa) + (yb - ya) * (yb - ya));
+  const float l = sqrt((xb - xa) * (xb - xa) + (yb - ya) * (yb - ya));
   const float ext = wd * 0.01f / zoom_scale;
   x1 = xa + (xb - xa) * ext / l;
   y1 = ya + (yb - ya) * ext / l;
@@ -527,14 +538,14 @@ void gui_post_expose(struct dt_iop_module_t *self, cairo_t *cr, int32_t width, i
   cairo_close_path(cr);
   cairo_set_line_width(cr, DT_PIXEL_APPLY_DPI(1.0) / zoom_scale);
   if(g->selected == 1 || g->dragging == 1)
-    cairo_set_source_rgba(cr, .8, .8, .8, 1.0);
+    dt_draw_set_color_overlay(cr, 0.8, 1.0);
   else
-    cairo_set_source_rgba(cr, .8, .8, .8, .5);
+    dt_draw_set_color_overlay(cr, 0.8, 0.5);
   cairo_fill_preserve(cr);
   if(g->selected == 1 || g->dragging == 1)
-    cairo_set_source_rgba(cr, .3, .3, .3, 1.0);
+    dt_draw_set_color_overlay(cr, 0.3, 1.0);
   else
-    cairo_set_source_rgba(cr, .3, .3, .3, .5);
+    dt_draw_set_color_overlay(cr, 0.3, 0.5);
   cairo_stroke(cr);
 
   x1 = xb - (xb - xa) * ext / l;
@@ -549,14 +560,14 @@ void gui_post_expose(struct dt_iop_module_t *self, cairo_t *cr, int32_t width, i
   cairo_close_path(cr);
   cairo_set_line_width(cr, DT_PIXEL_APPLY_DPI(1.0) / zoom_scale);
   if(g->selected == 2 || g->dragging == 2)
-    cairo_set_source_rgba(cr, .8, .8, .8, 1.0);
+    dt_draw_set_color_overlay(cr, 0.8, 1.0);
   else
-    cairo_set_source_rgba(cr, .8, .8, .8, .5);
+    dt_draw_set_color_overlay(cr, 0.8, 0.5);
   cairo_fill_preserve(cr);
   if(g->selected == 2 || g->dragging == 2)
-    cairo_set_source_rgba(cr, .3, .3, .3, 1.0);
+    dt_draw_set_color_overlay(cr, 0.3, 1.0);
   else
-    cairo_set_source_rgba(cr, .3, .3, .3, .5);
+    dt_draw_set_color_overlay(cr, 0.3, 0.5);
   cairo_stroke(cr);
 }
 
@@ -629,6 +640,7 @@ int button_pressed(struct dt_iop_module_t *self, double x, double y, double pres
 
   if(which == 3)
   {
+    // creating a line with right click
     g->dragging = 2;
     g->xa = pzx;
     g->ya = pzy;
@@ -655,17 +667,12 @@ int button_released(struct dt_iop_module_t *self, double x, double y, int which,
   dt_iop_graduatednd_params_t *p = (dt_iop_graduatednd_params_t *)self->params;
   if(g->dragging > 0)
   {
-    // dt_iop_graduatednd_params_t *p   = (dt_iop_graduatednd_params_t *)self->params;
-    // float wd = self->dev->preview_pipe->backbuf_width;
-    // float ht = self->dev->preview_pipe->backbuf_height;
     float pzx, pzy;
     dt_dev_get_pointer_zoom_pos(self->dev, x, y, &pzx, &pzy);
     pzx += 0.5f;
     pzy += 0.5f;
 
     float r = 0.0, o = 0.0;
-    // float pts[4];
-    // dt_dev_distort_backtransform(self->dev,pts,2);
     set_grad_from_points(self, g->xa, g->ya, g->xb, g->yb, &r, &o);
 
     // if this is a "line dragging, we reset extremities, to be sure they are not outside the image
@@ -675,14 +682,13 @@ int button_released(struct dt_iop_module_t *self, double x, double y, int which,
        * whole line dragging should not change rotation, so we should reuse
        * old rotation to avoid rounding issues
        */
+
       r = p->rotation;
       set_points_from_grad(self, &g->xa, &g->ya, &g->xb, &g->yb, r, o);
     }
-    const int reset = darktable.gui->reset;
-    darktable.gui->reset = 1;
-    dt_bauhaus_slider_set(g->scale3, r);
-    // dt_bauhaus_slider_set(g->scale4,o);
-    darktable.gui->reset = reset;
+    ++darktable.gui->reset;
+    dt_bauhaus_slider_set(g->rotation, r);
+    --darktable.gui->reset;
     p->rotation = r;
     p->offset = o;
     g->dragging = 0;
@@ -706,7 +712,7 @@ int scrolled(dt_iop_module_t *self, double x, double y, int up, uint32_t state)
       dens = fmaxf(-8.0, p->density - 0.1);
     if(dens != p->density)
     {
-      dt_bauhaus_slider_set(g->scale1, dens);
+      dt_bauhaus_slider_set(g->density, dens);
     }
     return 1;
   }
@@ -719,7 +725,7 @@ int scrolled(dt_iop_module_t *self, double x, double y, int up, uint32_t state)
       comp = fmaxf(0.0, p->hardness - 1.0);
     if(comp != p->hardness)
     {
-      dt_bauhaus_slider_set(g->scale2, comp);
+      dt_bauhaus_slider_set(g->hardness, comp);
     }
     return 1;
   }
@@ -766,7 +772,7 @@ void process(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, const
 #endif
     for(int y = 0; y < roi_out->height; y++)
     {
-      size_t k = (size_t)roi_out->width * y * ch;
+      const size_t k = (size_t)roi_out->width * y * ch;
       const float *in = (float *)ivoid + k;
       float *out = (float *)ovoid + k;
 
@@ -782,10 +788,10 @@ void process(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, const
         // use exp2f approximation to calculate e^(ln2*x/8)
         // in worst case - density==8,CLIP(0.5-length) == 1.0 it gives 0.6% of error
         const float t = 0.693147181f /* ln2 */ * (data->density * CLIP(0.5f + length) / 8.0f);
-        float d1 = t * t * 0.5f;
-        float d2 = d1 * t * 0.333333333f;
-        float d3 = d2 * t * 0.25f;
-        float d = 1 + t + d1 + d2 + d3; /* taylor series for e^x till x^4 */
+        const float d1 = t * t * 0.5f;
+        const float d2 = d1 * t * 0.333333333f;
+        const float d3 = d2 * t * 0.25f;
+        const float d = 1 + t + d1 + d2 + d3; /* taylor series for e^x till x^4 */
         // printf("%d %d  %f\n",y,x,d);
         float density = d;
         density = density * density;
@@ -793,7 +799,7 @@ void process(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, const
         density = density * density;
 #else
         // use fair exp2f
-        float density = exp2f(data->density * CLIP(0.5f + length));
+        const float density = exp2f(data->density * CLIP(0.5f + length));
 #endif
 
         for(int l = 0; l < 3; l++)
@@ -815,7 +821,7 @@ void process(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, const
 #endif
     for(int y = 0; y < roi_out->height; y++)
     {
-      size_t k = (size_t)roi_out->width * y * ch;
+      const size_t k = (size_t)roi_out->width * y * ch;
       const float *in = (float *)ivoid + k;
       float *out = (float *)ovoid + k;
 
@@ -831,16 +837,16 @@ void process(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, const
         // use exp2f approximation to calculate e^(ln2*x/8)
         // in worst case - density==-8,CLIP(0.5-length) == 1.0 it gives 0.6% of error
         const float t = 0.693147181f /* ln2 */ * (-data->density * CLIP(0.5f - length) / 8.0f);
-        float d1 = t * t * 0.5f;
-        float d2 = d1 * t * 0.333333333f;
-        float d3 = d2 * t * 0.25f;
-        float d = 1 + t + d1 + d2 + d3; /* taylor series for e^x till x^4 */
+        const float d1 = t * t * 0.5f;
+        const float d2 = d1 * t * 0.333333333f;
+        const float d3 = d2 * t * 0.25f;
+        const float d = 1 + t + d1 + d2 + d3; /* taylor series for e^x till x^4 */
         float density = d;
         density = density * density;
         density = density * density;
         density = density * density;
 #else
-        float density = exp2f(-data->density * CLIP(0.5f - length));
+        const float density = exp2f(-data->density * CLIP(0.5f - length));
 #endif
 
         for(int l = 0; l < 3; l++)
@@ -897,7 +903,7 @@ void process_sse2(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, c
 #endif
     for(int y = 0; y < roi_out->height; y++)
     {
-      size_t k = (size_t)roi_out->width * y * ch;
+      const size_t k = (size_t)roi_out->width * y * ch;
       const float *in = (float *)ivoid + k;
       float *out = (float *)ovoid + k;
 
@@ -905,8 +911,8 @@ void process_sse2(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, c
                      * filter_hardness;
       const float length_inc = sinv * hw_inv * filter_hardness;
 
-      __m128 c = _mm_set_ps(0, data->color[2], data->color[1], data->color[0]);
-      __m128 c1 = _mm_sub_ps(_mm_set1_ps(1.0f), c);
+      const __m128 c = _mm_set_ps(0, data->color[2], data->color[1], data->color[0]);
+      const __m128 c1 = _mm_sub_ps(_mm_set1_ps(1.0f), c);
 
       for(int x = 0; x < roi_out->width; x++, in += ch, out += ch)
       {
@@ -916,10 +922,10 @@ void process_sse2(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, c
         // use exp2f approximation to calculate e^(ln2*x/8)
         // in worst case - density==8,CLIP(0.5-length) == 1.0 it gives 0.6% of error
         const float t = 0.693147181f /* ln2 */ * (data->density * CLIP(0.5f + length) / 8.0f);
-        float d1 = t * t * 0.5f;
-        float d2 = d1 * t * 0.333333333f;
-        float d3 = d2 * t * 0.25f;
-        float d = 1 + t + d1 + d2 + d3; /* taylor series for e^x till x^4 */
+        const float d1 = t * t * 0.5f;
+        const float d2 = d1 * t * 0.333333333f;
+        const float d3 = d2 * t * 0.25f;
+        const float d = 1 + t + d1 + d2 + d3; /* taylor series for e^x till x^4 */
         // printf("%d %d  %f\n",y,x,d);
         __m128 density = _mm_set1_ps(d);
         density = _mm_mul_ps(density, density);
@@ -927,7 +933,7 @@ void process_sse2(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, c
         density = _mm_mul_ps(density, density);
 #else
         // use fair exp2f
-        __m128 density = _mm_set1_ps(exp2f(data->density * CLIP(0.5f + length)));
+        const __m128 density = _mm_set1_ps(exp2f(data->density * CLIP(0.5f + length)));
 #endif
 
         /* max(0,in / (c + (1-c)*density)) */
@@ -948,16 +954,16 @@ void process_sse2(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, c
 #endif
     for(int y = 0; y < roi_out->height; y++)
     {
-      size_t k = (size_t)roi_out->width * y * ch;
+      const size_t k = (size_t)roi_out->width * y * ch;
       const float *in = (float *)ivoid + k;
       float *out = (float *)ovoid + k;
 
       float length = (sinv * (-1.0f + ix * hw_inv) - cosv * (-1.0f + (iy + y) * hh_inv) - 1.0f + offset)
-                     * filter_hardness;
+                      * filter_hardness;
       const float length_inc = sinv * hw_inv * filter_hardness;
 
-      __m128 c = _mm_set_ps(0, data->color[2], data->color[1], data->color[0]);
-      __m128 c1 = _mm_sub_ps(_mm_set1_ps(1.0f), c);
+      const __m128 c = _mm_set_ps(0, data->color[2], data->color[1], data->color[0]);
+      const __m128 c1 = _mm_sub_ps(_mm_set1_ps(1.0f), c);
 
       for(int x = 0; x < roi_out->width; x++, in += ch, out += ch)
       {
@@ -967,16 +973,16 @@ void process_sse2(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, c
         // use exp2f approximation to calculate e^(ln2*x/8)
         // in worst case - density==-8,CLIP(0.5-length) == 1.0 it gives 0.6% of error
         const float t = 0.693147181f /* ln2 */ * (-data->density * CLIP(0.5f - length) / 8.0f);
-        float d1 = t * t * 0.5f;
-        float d2 = d1 * t * 0.333333333f;
-        float d3 = d2 * t * 0.25f;
-        float d = 1 + t + d1 + d2 + d3; /* taylor series for e^x till x^4 */
+        const float d1 = t * t * 0.5f;
+        const float d2 = d1 * t * 0.333333333f;
+        const float d3 = d2 * t * 0.25f;
+        const float d = 1 + t + d1 + d2 + d3; /* taylor series for e^x till x^4 */
         __m128 density = _mm_set1_ps(d);
         density = _mm_mul_ps(density, density);
         density = _mm_mul_ps(density, density);
         density = _mm_mul_ps(density, density);
 #else
-        __m128 density = _mm_set1_ps(exp2f(-data->density * CLIP(0.5f - length)));
+        const __m128 density = _mm_set1_ps(exp2f(-data->density * CLIP(0.5f - length)));
 #endif
 
         /* max(0,in * (c + (1-c)*density)) */
@@ -1077,36 +1083,19 @@ void cleanup_global(dt_iop_module_so_t *module)
   module->data = NULL;
 }
 
-
-static void density_callback(GtkWidget *slider, gpointer user_data)
+void gui_changed(dt_iop_module_t *self, GtkWidget *w, void *previous)
 {
-  dt_iop_module_t *self = (dt_iop_module_t *)user_data;
-  if(self->dt->gui->reset) return;
-  dt_iop_graduatednd_params_t *p = (dt_iop_graduatednd_params_t *)self->params;
-  p->density = dt_bauhaus_slider_get(slider);
-  dt_dev_add_history_item(darktable.develop, self, TRUE);
-}
-
-static void hardness_callback(GtkWidget *slider, gpointer user_data)
-{
-  dt_iop_module_t *self = (dt_iop_module_t *)user_data;
-  if(self->dt->gui->reset) return;
-  dt_iop_graduatednd_params_t *p = (dt_iop_graduatednd_params_t *)self->params;
-  p->hardness = dt_bauhaus_slider_get(slider);
-  dt_dev_add_history_item(darktable.develop, self, TRUE);
-}
-
-static void rotation_callback(GtkWidget *slider, gpointer user_data)
-{
-  dt_iop_module_t *self = (dt_iop_module_t *)user_data;
-  if(self->dt->gui->reset) return;
   dt_iop_graduatednd_params_t *p = (dt_iop_graduatednd_params_t *)self->params;
   dt_iop_graduatednd_gui_data_t *g = (dt_iop_graduatednd_gui_data_t *)self->gui_data;
-  // float wd = self->dev->preview_pipe->backbuf_width;
-  // float ht = self->dev->preview_pipe->backbuf_height;
-  p->rotation = dt_bauhaus_slider_get(slider);
-  set_points_from_grad(self, &g->xa, &g->ya, &g->xb, &g->yb, p->rotation, p->offset);
-  dt_dev_add_history_item(darktable.develop, self, TRUE);
+  if(w == g->rotation)
+  {
+    set_points_from_grad(self, &g->xa, &g->ya, &g->xb, &g->yb, p->rotation, p->offset);
+  }
+  else if(w == g->hue)
+  {
+    update_saturation_slider_end_color(g->saturation, p->hue);
+    gtk_widget_queue_draw(g->saturation);
+  }
 }
 
 void commit_params(struct dt_iop_module_t *self, dt_iop_params_t *p1, dt_dev_pixelpipe_t *pipe,
@@ -1149,146 +1138,54 @@ void gui_update(struct dt_iop_module_t *self)
 
   dt_iop_color_picker_reset(self, TRUE);
 
-  dt_bauhaus_slider_set(g->scale1, p->density);
-  dt_bauhaus_slider_set(g->scale2, p->hardness);
-  dt_bauhaus_slider_set(g->scale3, p->rotation);
-  dt_bauhaus_slider_set(g->gslider1, p->hue);
-  dt_bauhaus_slider_set(g->gslider2, p->saturation);
+  dt_bauhaus_slider_set(g->density, p->density);
+  dt_bauhaus_slider_set(g->hardness, p->hardness);
+  dt_bauhaus_slider_set(g->rotation, p->rotation);
+  dt_bauhaus_slider_set(g->hue, p->hue);
+  dt_bauhaus_slider_set(g->saturation, p->saturation);
 
-  // float wd = self->dev->preview_pipe->backbuf_width;
-  // float ht = self->dev->preview_pipe->backbuf_height;
   g->define = 0;
-  // set_points_from_grad(self,&g->xa,&g->ya,&g->xb,&g->yb,p->rotation,p->offset);
-  update_saturation_slider_end_color(g->gslider2, p->hue);
+  update_saturation_slider_end_color(g->saturation, p->hue);
 }
 
-void init(dt_iop_module_t *module)
-{
-  module->params = calloc(1, sizeof(dt_iop_graduatednd_params_t));
-  module->default_params = calloc(1, sizeof(dt_iop_graduatednd_params_t));
-  module->default_enabled = 0;
-  module->params_size = sizeof(dt_iop_graduatednd_params_t);
-  module->gui_data = NULL;
-  dt_iop_graduatednd_params_t tmp = (dt_iop_graduatednd_params_t){ 1.0, 0, 0, 50, 0, 0 };
-  memcpy(module->params, &tmp, sizeof(dt_iop_graduatednd_params_t));
-  memcpy(module->default_params, &tmp, sizeof(dt_iop_graduatednd_params_t));
-}
-
-void cleanup(dt_iop_module_t *module)
-{
-  free(module->params);
-  module->params = NULL;
-  free(module->default_params);
-  module->default_params = NULL;
-}
-
-static void hue_callback(GtkWidget *slider, gpointer user_data)
-{
-  dt_iop_module_t *self = (dt_iop_module_t *)user_data;
-  dt_iop_graduatednd_params_t *p = (dt_iop_graduatednd_params_t *)self->params;
-  dt_iop_graduatednd_gui_data_t *g = (dt_iop_graduatednd_gui_data_t *)self->gui_data;
-
-  const float hue = dt_bauhaus_slider_get(g->gslider1);
-  // fprintf(stderr," hue: %f, saturation: %f\n",hue,dtgtk_gradient_slider_get_value(g->gslider2));
-
-  update_saturation_slider_end_color(g->gslider2, hue);
-
-  if(self->dt->gui->reset) return;
-  gtk_widget_queue_draw(GTK_WIDGET(g->gslider2));
-
-  p->hue = hue;
-  dt_dev_add_history_item(darktable.develop, self, TRUE);
-}
-
-static void saturation_callback(GtkWidget *slider, gpointer user_data)
-{
-  dt_iop_module_t *self = (dt_iop_module_t *)user_data;
-  dt_iop_graduatednd_params_t *p = (dt_iop_graduatednd_params_t *)self->params;
-
-  p->saturation = dt_bauhaus_slider_get(slider);
-  dt_dev_add_history_item(darktable.develop, self, TRUE);
-}
-
-
 void gui_init(struct dt_iop_module_t *self)
 {
   self->gui_data = malloc(sizeof(dt_iop_graduatednd_gui_data_t));
   dt_iop_graduatednd_gui_data_t *g = (dt_iop_graduatednd_gui_data_t *)self->gui_data;
-  dt_iop_graduatednd_params_t *p = (dt_iop_graduatednd_params_t *)self->params;
 
-  self->widget = gtk_box_new(GTK_ORIENTATION_VERTICAL, DT_BAUHAUS_SPACE);
-  dt_gui_add_help_link(self->widget, dt_get_help_url(self->op));
+  g->density = dt_bauhaus_slider_from_params(self, "density");
+  dt_bauhaus_slider_set_format(g->density, _("%.2f EV"));
+  gtk_widget_set_tooltip_text(g->density, _("the density in EV for the filter"));
 
-  /* density */
-  g->scale1 = dt_bauhaus_slider_new_with_range(self, -8.0, 8.0, 0.1, p->density, 2);
-  dt_bauhaus_slider_set_format(g->scale1, _("%.2f EV"));
-  dt_bauhaus_widget_set_label(g->scale1, NULL, _("density"));
-  gtk_widget_set_tooltip_text(g->scale1, _("the density in EV for the filter"));
-  g_signal_connect(G_OBJECT(g->scale1), "value-changed", G_CALLBACK(density_callback), self);
-
-  /* hardness */
-  g->scale2 = dt_bauhaus_slider_new_with_range(self, 0.0, 100.0, 1.0, p->hardness, 0);
-  dt_bauhaus_slider_set_format(g->scale2, "%.0f%%");
-  dt_bauhaus_widget_set_label(g->scale2, NULL, _("hardness"));
+  g->hardness = dt_bauhaus_slider_from_params(self, "hardness");
+  dt_bauhaus_slider_set_format(g->hardness, "%.0f%%");
   /* xgettext:no-c-format */
-  gtk_widget_set_tooltip_text(g->scale2, _("hardness of graduation:\n0% = soft, 100% = hard"));
-  g_signal_connect(G_OBJECT(g->scale2), "value-changed", G_CALLBACK(hardness_callback), self);
+  gtk_widget_set_tooltip_text(g->hardness, _("hardness of graduation:\n0% = soft, 100% = hard"));
 
-  /* rotation */
-  g->scale3 = dt_bauhaus_slider_new_with_range(self, -180, 180, 0.5, p->rotation, 2);
-  dt_bauhaus_widget_set_label(g->scale3, NULL, _("rotation"));
-  dt_bauhaus_slider_set_format(g->scale3, "%.2f°");
-  gtk_widget_set_tooltip_text(g->scale3, _("rotation of filter -180 to 180 degrees"));
-  g_signal_connect(G_OBJECT(g->scale3), "value-changed", G_CALLBACK(rotation_callback), self);
+  g->rotation = dt_bauhaus_slider_from_params(self, "rotation");
+  dt_bauhaus_slider_set_format(g->rotation, "%.2f°");
+  gtk_widget_set_tooltip_text(g->rotation, _("rotation of filter -180 to 180 degrees"));
 
-  /* add widgets to ui */
-  gtk_box_pack_start(GTK_BOX(self->widget), GTK_WIDGET(g->scale1), TRUE, TRUE, 0);
-  gtk_box_pack_start(GTK_BOX(self->widget), GTK_WIDGET(g->scale2), TRUE, TRUE, 0);
-  gtk_box_pack_start(GTK_BOX(self->widget), GTK_WIDGET(g->scale3), TRUE, TRUE, 0);
+  g->hue = dt_color_picker_new(self, DT_COLOR_PICKER_POINT, 
+           dt_bauhaus_slider_from_params(self, "hue"));
+  dt_bauhaus_slider_set_feedback(g->hue, 0);
+  dt_bauhaus_slider_set_stop(g->hue, 0.0f, 1.0f, 0.0f, 0.0f);
+  dt_bauhaus_slider_set_stop(g->hue, 0.166f, 1.0f, 1.0f, 0.0f);
+  dt_bauhaus_slider_set_stop(g->hue, 0.322f, 0.0f, 1.0f, 0.0f);
+  dt_bauhaus_slider_set_stop(g->hue, 0.498f, 0.0f, 1.0f, 1.0f);
+  dt_bauhaus_slider_set_stop(g->hue, 0.664f, 0.0f, 0.0f, 1.0f);
+  dt_bauhaus_slider_set_stop(g->hue, 0.830f, 1.0f, 0.0f, 1.0f);
+  dt_bauhaus_slider_set_stop(g->hue, 1.0f, 1.0f, 0.0f, 0.0f);
+  gtk_widget_set_tooltip_text(g->hue, _("select the hue tone of filter"));
 
-  /* hue slider */
-  g->gslider1 = dt_bauhaus_slider_new_with_range_and_feedback(self, 0.0f, 1.0f, 0.01f, 0.0f, 2, 0);
-  dt_bauhaus_slider_set_stop(g->gslider1, 0.0f, 1.0f, 0.0f, 0.0f);
-  // dt_bauhaus_slider_set_format(g->gslider1, "");
-  dt_bauhaus_widget_set_label(g->gslider1, NULL, _("hue"));
-  dt_bauhaus_slider_set_stop(g->gslider1, 0.166f, 1.0f, 1.0f, 0.0f);
-  dt_bauhaus_slider_set_stop(g->gslider1, 0.322f, 0.0f, 1.0f, 0.0f);
-  dt_bauhaus_slider_set_stop(g->gslider1, 0.498f, 0.0f, 1.0f, 1.0f);
-  dt_bauhaus_slider_set_stop(g->gslider1, 0.664f, 0.0f, 0.0f, 1.0f);
-  dt_bauhaus_slider_set_stop(g->gslider1, 0.830f, 1.0f, 0.0f, 1.0f);
-  dt_bauhaus_slider_set_stop(g->gslider1, 1.0f, 1.0f, 0.0f, 0.0f);
-  gtk_widget_set_tooltip_text(g->gslider1, _("select the hue tone of filter"));
-  g_signal_connect(G_OBJECT(g->gslider1), "value-changed", G_CALLBACK(hue_callback), self);
-  dt_bauhaus_widget_set_quad_paint(g->gslider1, dtgtk_cairo_paint_colorpicker, CPF_STYLE_FLAT | CPF_DO_NOT_USE_BORDER, NULL);
-  dt_bauhaus_widget_set_quad_toggle(g->gslider1, TRUE);
-  g_signal_connect(G_OBJECT(g->gslider1), "quad-pressed", G_CALLBACK(dt_iop_color_picker_callback), &g->color_picker);
-
-  /* saturation slider */
-  g->gslider2 = dt_bauhaus_slider_new_with_range(self, 0.0f, 1.0f, 0.01f, 0.0f, 2);
-  dt_bauhaus_widget_set_label(g->gslider2, NULL, _("saturation"));
-  dt_bauhaus_slider_set_stop(g->gslider2, 0.0f, 0.2f, 0.2f, 0.2f);
-  dt_bauhaus_slider_set_stop(g->gslider2, 1.0f, 1.0f, 1.0f, 1.0f);
-  gtk_widget_set_tooltip_text(g->gslider2, _("select the saturation of filter"));
-  g_signal_connect(G_OBJECT(g->gslider2), "value-changed", G_CALLBACK(saturation_callback), self);
-
-  gtk_box_pack_start(GTK_BOX(self->widget), GTK_WIDGET(g->gslider1), TRUE, TRUE, 0);
-  gtk_box_pack_start(GTK_BOX(self->widget), GTK_WIDGET(g->gslider2), TRUE, TRUE, 0);
+  g->saturation = dt_bauhaus_slider_from_params(self, "saturation");
+  dt_bauhaus_slider_set_stop(g->saturation, 0.0f, 0.2f, 0.2f, 0.2f);
+  dt_bauhaus_slider_set_stop(g->saturation, 1.0f, 1.0f, 1.0f, 1.0f);
+  gtk_widget_set_tooltip_text(g->saturation, _("select the saturation of filter"));
 
   g->selected = 0;
   g->dragging = 0;
   g->define = 0;
-
-  dt_iop_init_single_picker(&g->color_picker,
-                     self,
-                     g->gslider1,
-                     DT_COLOR_PICKER_POINT,
-                     _iop_color_picker_apply);
-}
-
-void gui_cleanup(struct dt_iop_module_t *self)
-{
-  free(self->gui_data);
-  self->gui_data = NULL;
 }
 
 GSList *mouse_actions(struct dt_iop_module_t *self)
