@@ -1,10 +1,6 @@
 /*
     This file is part of darktable,
-    copyright (c) 2011 Rostyslav Pidgornyi
-    copyright (c) 2012 Henrik Andersson
-
-    and the initial plugin `stuck pixels' was
-    copyright (c) 2011 bruce guenter
+    Copyright (C) 2011-2020 darktable developers.
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -27,6 +23,7 @@
 #include "control/control.h"
 #include "develop/imageop.h"
 #include "develop/imageop_math.h"
+#include "develop/imageop_gui.h"
 #include "dtgtk/resetlabel.h"
 #include "gui/accelerators.h"
 #include "gui/gtk.h"
@@ -39,10 +36,10 @@ DT_MODULE_INTROSPECTION(1, dt_iop_hotpixels_params_t)
 
 typedef struct dt_iop_hotpixels_params_t
 {
-  float strength;
-  float threshold;
-  gboolean markfixed;
-  gboolean permissive;
+  float strength;  // $MIN: 0.0 $MAX: 1.0 $DEFAULT: 0.25
+  float threshold; // $MIN: 0.0 $MAX: 1.0 $DEFAULT: 0.05
+  gboolean markfixed;  // $DEFAULT: FALSE $DESCRIPTION: "mark fixed pixels"
+  gboolean permissive; // $DEFAULT: FALSE $DESCRIPTION: "detect by 3 neighbors"
 } dt_iop_hotpixels_params_t;
 
 typedef struct dt_iop_hotpixels_gui_data_t
@@ -301,7 +298,7 @@ void process(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, const
     fixed = process_bayer(data, ivoid, ovoid, roi_out);
   }
 
-  if(g != NULL && self->dev->gui_attached && piece->pipe->type == DT_DEV_PIXELPIPE_FULL)
+  if(g != NULL && self->dev->gui_attached && (piece->pipe->type & DT_DEV_PIXELPIPE_FULL) == DT_DEV_PIXELPIPE_FULL)
   {
     g->pixels_fixed = fixed;
   }
@@ -309,41 +306,11 @@ void process(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, const
 
 void reload_defaults(dt_iop_module_t *module)
 {
-  const dt_iop_hotpixels_params_t tmp
-      = {.strength = 0.25, .threshold = 0.05, .markfixed = FALSE, .permissive = FALSE };
-
   // we might be called from presets update infrastructure => there is no image
-  if(!module->dev) goto end;
+  if(!module->dev) return;
 
   // can't be switched on for non-raw images:
-  if(dt_image_is_raw(&module->dev->image_storage))
-    module->hide_enable_button = 0;
-  else
-    module->hide_enable_button = 1;
-
-end:
-  memcpy(module->params, &tmp, sizeof(dt_iop_hotpixels_params_t));
-  memcpy(module->default_params, &tmp, sizeof(dt_iop_hotpixels_params_t));
-}
-
-void init(dt_iop_module_t *module)
-{
-  module->global_data = NULL;
-  module->params = calloc(1, sizeof(dt_iop_hotpixels_params_t));
-  module->default_params = calloc(1, sizeof(dt_iop_hotpixels_params_t));
-  module->default_enabled = 0;
-  module->params_size = sizeof(dt_iop_hotpixels_params_t);
-  module->gui_data = NULL;
-}
-
-void cleanup(dt_iop_module_t *module)
-{
-  free(module->params);
-  module->params = NULL;
-  free(module->default_params);
-  module->default_params = NULL;
-  free(module->global_data);
-  module->global_data = NULL;
+  module->hide_enable_button = !dt_image_is_raw(&module->dev->image_storage);
 }
 
 void commit_params(struct dt_iop_module_t *self, dt_iop_params_t *params, dt_dev_pixelpipe_t *pipe,
@@ -355,8 +322,8 @@ void commit_params(struct dt_iop_module_t *self, dt_iop_params_t *params, dt_dev
   d->multiplier = p->strength / 2.0;
   d->threshold = p->threshold;
   d->permissive = p->permissive;
-  d->markfixed = p->markfixed && (pipe->type != DT_DEV_PIXELPIPE_EXPORT)
-                 && (pipe->type != DT_DEV_PIXELPIPE_THUMBNAIL);
+  d->markfixed = p->markfixed && ((pipe->type & DT_DEV_PIXELPIPE_EXPORT) != DT_DEV_PIXELPIPE_EXPORT)
+    && ((pipe->type & DT_DEV_PIXELPIPE_THUMBNAIL) != DT_DEV_PIXELPIPE_THUMBNAIL);
   if(!(dt_image_is_raw(&pipe->image)) || p->strength == 0.0) piece->enabled = 0;
 }
 
@@ -372,41 +339,6 @@ void cleanup_pipe(struct dt_iop_module_t *self, dt_dev_pixelpipe_t *pipe, dt_dev
   piece->data = NULL;
 }
 
-static void strength_callback(GtkRange *range, dt_iop_module_t *self)
-{
-  if(darktable.gui->reset) return;
-  dt_iop_hotpixels_gui_data_t *g = (dt_iop_hotpixels_gui_data_t *)self->gui_data;
-  dt_iop_hotpixels_params_t *p = (dt_iop_hotpixels_params_t *)self->params;
-  p->strength = dt_bauhaus_slider_get(g->strength);
-  dt_dev_add_history_item(darktable.develop, self, TRUE);
-}
-
-static void threshold_callback(GtkRange *range, dt_iop_module_t *self)
-{
-  if(darktable.gui->reset) return;
-  dt_iop_hotpixels_gui_data_t *g = (dt_iop_hotpixels_gui_data_t *)self->gui_data;
-  dt_iop_hotpixels_params_t *p = (dt_iop_hotpixels_params_t *)self->params;
-  p->threshold = dt_bauhaus_slider_get(g->threshold);
-  dt_dev_add_history_item(darktable.develop, self, TRUE);
-}
-
-static void markfixed_callback(GtkRange *range, dt_iop_module_t *self)
-{
-  if(darktable.gui->reset) return;
-  dt_iop_hotpixels_gui_data_t *g = (dt_iop_hotpixels_gui_data_t *)self->gui_data;
-  dt_iop_hotpixels_params_t *p = (dt_iop_hotpixels_params_t *)self->params;
-  p->markfixed = gtk_toggle_button_get_active(g->markfixed);
-  dt_dev_add_history_item(darktable.develop, self, TRUE);
-}
-
-static void permissive_callback(GtkRange *range, dt_iop_module_t *self)
-{
-  if(darktable.gui->reset) return;
-  dt_iop_hotpixels_gui_data_t *g = (dt_iop_hotpixels_gui_data_t *)self->gui_data;
-  dt_iop_hotpixels_params_t *p = (dt_iop_hotpixels_params_t *)self->params;
-  p->permissive = gtk_toggle_button_get_active(g->permissive);
-  dt_dev_add_history_item(darktable.develop, self, TRUE);
-}
 
 void gui_update(dt_iop_module_t *self)
 {
@@ -441,10 +373,9 @@ static gboolean draw(GtkWidget *widget, cairo_t *cr, dt_iop_module_t *self)
   char *str = g_strdup_printf(ngettext("fixed %d pixel", "fixed %d pixels", g->pixels_fixed), g->pixels_fixed);
   g->pixels_fixed = -1;
 
-  const int reset = darktable.gui->reset;
-  darktable.gui->reset = 1;
+  ++darktable.gui->reset;
   gtk_label_set_text(g->message, str);
-  darktable.gui->reset = reset;
+  --darktable.gui->reset;
 
   g_free(str);
 
@@ -455,48 +386,33 @@ void gui_init(dt_iop_module_t *self)
 {
   self->gui_data = malloc(sizeof(dt_iop_hotpixels_gui_data_t));
   dt_iop_hotpixels_gui_data_t *g = (dt_iop_hotpixels_gui_data_t *)self->gui_data;
-  dt_iop_hotpixels_params_t *p = (dt_iop_hotpixels_params_t *)self->params;
+
   g->pixels_fixed = -1;
 
-  self->widget = gtk_box_new(GTK_ORIENTATION_VERTICAL, DT_BAUHAUS_SPACE);
-  dt_gui_add_help_link(self->widget, dt_get_help_url(self->op));
-
-  g->box_raw = gtk_box_new(GTK_ORIENTATION_VERTICAL, DT_BAUHAUS_SPACE);
+  g->box_raw = self->widget = gtk_box_new(GTK_ORIENTATION_VERTICAL, DT_BAUHAUS_SPACE);
   g_signal_connect(G_OBJECT(g->box_raw), "draw", G_CALLBACK(draw), self);
 
-  /* threshold */
-  g->threshold = dt_bauhaus_slider_new_with_range(self, 0.0, 1.0, 0.005, p->threshold, 4);
-  dt_bauhaus_slider_set_format(g->threshold, "%.4f");
-  dt_bauhaus_widget_set_label(g->threshold, NULL, _("threshold"));
+  g->threshold = dt_bauhaus_slider_from_params(self, N_("threshold"));
+  dt_bauhaus_slider_set_step(g->threshold, 0.005);
+  dt_bauhaus_slider_set_digits(g->threshold, 4);
   gtk_widget_set_tooltip_text(g->threshold, _("lower threshold for hot pixel"));
-  gtk_box_pack_start(GTK_BOX(g->box_raw), GTK_WIDGET(g->threshold), TRUE, TRUE, 0);
-  g_signal_connect(G_OBJECT(g->threshold), "value-changed", G_CALLBACK(threshold_callback), self);
 
-  /* strength */
-  g->strength = dt_bauhaus_slider_new_with_range(self, 0.0, 1.0, 0.01, p->strength, 4);
-  dt_bauhaus_slider_set_format(g->threshold, "%.4f");
-  dt_bauhaus_widget_set_label(g->strength, NULL, _("strength"));
+  g->strength = dt_bauhaus_slider_from_params(self, N_("strength"));
+  dt_bauhaus_slider_set_digits(g->strength, 4);
   gtk_widget_set_tooltip_text(g->strength, _("strength of hot pixel correction"));
-  gtk_box_pack_start(GTK_BOX(g->box_raw), GTK_WIDGET(g->strength), TRUE, TRUE, 0);
-  g_signal_connect(G_OBJECT(g->strength), "value-changed", G_CALLBACK(strength_callback), self);
 
-  /* 3 neighbours */
-  g->permissive = GTK_TOGGLE_BUTTON(gtk_check_button_new_with_label(_("detect by 3 neighbors")));
-  gtk_toggle_button_set_active(g->permissive, p->permissive);
-  gtk_box_pack_start(GTK_BOX(g->box_raw), GTK_WIDGET(g->permissive), TRUE, TRUE, 0);
-  g_signal_connect(G_OBJECT(g->permissive), "toggled", G_CALLBACK(permissive_callback), self);
-
-
-  GtkBox *hbox1 = GTK_BOX(gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0));
-  g->markfixed = GTK_TOGGLE_BUTTON(gtk_check_button_new_with_label(_("mark fixed pixels")));
-  gtk_toggle_button_set_active(g->markfixed, p->markfixed);
-  gtk_box_pack_start(GTK_BOX(hbox1), GTK_WIDGET(g->markfixed), TRUE, TRUE, 0);
-  g_signal_connect(G_OBJECT(g->markfixed), "toggled", G_CALLBACK(markfixed_callback), self);
+  // 3 neighbours
+  g->permissive = GTK_TOGGLE_BUTTON(dt_bauhaus_toggle_from_params(self, "permissive"));
 
+  // mark fixed pixels
+  GtkWidget *hbox = self->widget = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
+  g->markfixed = GTK_TOGGLE_BUTTON(dt_bauhaus_toggle_from_params(self, "markfixed"));
   g->message = GTK_LABEL(gtk_label_new("")); // This gets filled in by process
-  gtk_box_pack_start(GTK_BOX(hbox1), GTK_WIDGET(g->message), TRUE, TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(hbox), GTK_WIDGET(g->message), TRUE, TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(g->box_raw), hbox, TRUE, TRUE, 0);
 
-  gtk_box_pack_start(GTK_BOX(g->box_raw), GTK_WIDGET(hbox1), TRUE, TRUE, 0);
+  // start building top level widget
+  self->widget = gtk_box_new(GTK_ORIENTATION_VERTICAL, DT_BAUHAUS_SPACE);
 
   gtk_box_pack_start(GTK_BOX(self->widget), g->box_raw, FALSE, FALSE, 0);
 
@@ -505,12 +421,6 @@ void gui_init(dt_iop_module_t *self)
   gtk_box_pack_start(GTK_BOX(self->widget), g->label_non_raw, FALSE, FALSE, 0);
 }
 
-void gui_cleanup(dt_iop_module_t *self)
-{
-  free(self->gui_data);
-  self->gui_data = NULL;
-}
-
 // modelines: These editor modelines have been set for all relevant files by tools/update_modelines.sh
 // vim: shiftwidth=2 expandtab tabstop=2 cindent
 // kate: tab-indents: off; indent-width 2; replace-tabs on; indent-mode cstyle; remove-trailing-spaces modified;
